@@ -18,17 +18,18 @@ Internal implementation of request Body validating middleware.
 
 """
 
-import base64
 import re
 
 import jsonschema
 from jsonschema import exceptions as jsonschema_exc
+from oslo_serialization import base64
 from oslo_utils import strutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import six
 import webob.exc
 
+from cinder.api import api_utils
 from cinder import db
 from cinder import exception
 from cinder.i18n import _
@@ -156,7 +157,7 @@ def _validate_name_skip_leading_trailing_spaces(param_value):
         raise exception.InvalidName(reason=msg)
     elif len(param_value) > 255:
         msg = _("The 'name' can not be greater than 255 characters.")
-        raise exception.InvalidInput(reason=msg)
+        raise exception.InvalidName(reason=msg)
     return True
 
 
@@ -171,11 +172,11 @@ def _validate_status(param_value):
         msg = _("The 'status' can not be empty.")
         raise exception.InvalidGroupSnapshotStatus(reason=msg)
     elif param_value.lower() not in c_fields.GroupSnapshotStatus.ALL:
-            msg = _("Group snapshot status: %(status)s is invalid, "
-                    "valid statuses are: "
-                    "%(valid)s.") % {'status': param_value,
-                                     'valid': c_fields.GroupSnapshotStatus.ALL}
-            raise exception.InvalidGroupSnapshotStatus(reason=msg)
+        msg = _("Group snapshot status: %(status)s is invalid, "
+                "valid statuses are: "
+                "%(valid)s.") % {'status': param_value,
+                                 'valid': c_fields.GroupSnapshotStatus.ALL}
+        raise exception.InvalidGroupSnapshotStatus(reason=msg)
     return True
 
 
@@ -199,9 +200,7 @@ def _validate_base64_format(instance):
     try:
         if isinstance(instance, six.text_type):
             instance = instance.encode('utf-8')
-        base64.decodestring(instance)
-    except base64.binascii.Error:
-        return False
+        base64.decode_as_bytes(instance)
     except TypeError:
         # The name must be string type. If instance isn't string type, the
         # TypeError will be raised at here.
@@ -249,8 +248,8 @@ def _validate_quota_set(quota_set):
         if key in NON_QUOTA_KEYS:
             continue
 
-        utils.validate_integer(value, key, min_value=-1,
-                               max_value=db.MAX_INT)
+        api_utils.validate_integer(value, key, min_value=-1,
+                                   max_value=db.MAX_INT)
 
     if len(bad_keys) > 0:
         msg = _("Bad key(s) in quota set: %s") % ", ".join(bad_keys)
