@@ -54,13 +54,17 @@ db_opts = [
     cfg.StrOpt('snapshot_name_template',
                default='snapshot-%s',
                help='Template string to be used to generate snapshot names'),
+]
+
+backup_opts = [
     cfg.StrOpt('backup_name_template',
                default='backup-%s',
-               help='Template string to be used to generate backup names'), ]
-
+               help='Template string to be used to generate backup names'),
+]
 
 CONF = cfg.CONF
 CONF.register_opts(db_opts)
+CONF.register_opts(backup_opts)
 db_options.set_defaults(CONF)
 
 _BACKEND_MAPPING = {'sqlalchemy': 'cinder.db.sqlalchemy.api'}
@@ -117,7 +121,9 @@ def service_get(context, service_id=None, backend_match_level=None, **filters):
 def service_get_all(context, backend_match_level=None, **filters):
     """Get all services that match the criteria.
 
-    A possible filter is is_up=True and it will filter nodes that are down.
+    A possible filter is is_up=True and it will filter nodes that are down,
+    as well as host_or_cluster, that lets you look for services using both
+    of these properties.
 
     :param filters: Filters for the query in the form of key/value arguments.
     :param backend_match_level: 'pool', 'backend', or 'host' for host and
@@ -140,6 +146,22 @@ def service_update(context, service_id, values):
     return IMPL.service_update(context, service_id, values)
 
 
+def service_get_by_uuid(context, service_uuid):
+    """Get a service by it's uuid.
+
+    Return Service ref or raise if it does not exist.
+    """
+    return IMPL.service_get_by_uuid(context, service_uuid)
+
+
+###############
+
+
+def is_backend_frozen(context, host, cluster_name):
+    """Check if a storage backend is frozen based on host and cluster_name."""
+    return IMPL.is_backend_frozen(context, host, cluster_name)
+
+
 ###############
 
 
@@ -213,10 +235,10 @@ def volume_attach(context, values):
 
 
 def volume_attached(context, volume_id, instance_id, host_name, mountpoint,
-                    attach_mode='rw'):
+                    attach_mode='rw', mark_attached=True):
     """Ensure that a volume is set as attached."""
     return IMPL.volume_attached(context, volume_id, instance_id, host_name,
-                                mountpoint, attach_mode)
+                                mountpoint, attach_mode, mark_attached)
 
 
 def volume_create(context, values):
@@ -231,9 +253,9 @@ def volume_data_get_for_host(context, host, count_only=False):
                                          count_only)
 
 
-def volume_data_get_for_project(context, project_id):
+def volume_data_get_for_project(context, project_id, host=None):
     """Get (volume_count, gigabytes) for project."""
-    return IMPL.volume_data_get_for_project(context, project_id)
+    return IMPL.volume_data_get_for_project(context, project_id, host=host)
 
 
 def volume_destroy(context, volume_id):
@@ -251,14 +273,18 @@ def volume_get(context, volume_id):
     return IMPL.volume_get(context, volume_id)
 
 
-def volume_get_all(context, marker, limit, sort_keys=None, sort_dirs=None,
-                   filters=None, offset=None):
+def volume_get_all(context, marker=None, limit=None, sort_keys=None,
+                   sort_dirs=None, filters=None, offset=None):
     """Get all volumes."""
     return IMPL.volume_get_all(context, marker, limit, sort_keys=sort_keys,
                                sort_dirs=sort_dirs, filters=filters,
                                offset=offset)
 
 
+def calculate_resource_count(context, resource_type, filters):
+    return IMPL.calculate_resource_count(context, resource_type, filters)
+
+
 def volume_get_all_by_host(context, host, filters=None):
     """Get all volumes belonging to a host."""
     return IMPL.volume_get_all_by_host(context, host, filters=filters)
@@ -286,14 +312,9 @@ def volume_get_all_by_project(context, project_id, marker, limit,
                                           offset=offset)
 
 
-def get_volume_summary_all(context):
-    """Get all volume summary."""
-    return IMPL.get_volume_summary_all(context)
-
-
-def get_volume_summary_by_project(context, project_id):
-    """Get all volume summary belonging to a project."""
-    return IMPL.get_volume_summary_by_project(context, project_id)
+def get_volume_summary(context, project_only, filters=None):
+    """Get volume summary."""
+    return IMPL.get_volume_summary(context, project_only, filters)
 
 
 def volume_update(context, volume_id, values):
@@ -336,25 +357,49 @@ def volume_attachment_update(context, attachment_id, values):
     return IMPL.volume_attachment_update(context, attachment_id, values)
 
 
-def volume_attachment_get(context, attachment_id, session=None):
-    return IMPL.volume_attachment_get(context, attachment_id, session)
+def volume_attachment_get(context, attachment_id):
+    return IMPL.volume_attachment_get(context, attachment_id)
 
 
-def volume_attachment_get_all_by_volume_id(context, volume_id):
-    return IMPL.volume_attachment_get_all_by_volume_id(context, volume_id)
+def volume_attachment_get_all_by_volume_id(context, volume_id,
+                                           session=None):
+    return IMPL.volume_attachment_get_all_by_volume_id(context,
+                                                       volume_id,
+                                                       session)
 
 
-def volume_attachment_get_all_by_host(context, volume_id, host):
-    return IMPL.volume_attachment_get_all_by_host(context, volume_id, host)
+def volume_attachment_get_all_by_host(context, host, filters=None):
+    # FIXME(jdg): Not using filters
+    return IMPL.volume_attachment_get_all_by_host(context, host)
 
 
 def volume_attachment_get_all_by_instance_uuid(context,
-                                               volume_id,
-                                               instance_uuid):
-    return IMPL.volume_attachment_get_all_by_instance_uuid(context, volume_id,
+                                               instance_uuid, filters=None):
+    # FIXME(jdg): Not using filters
+    return IMPL.volume_attachment_get_all_by_instance_uuid(context,
                                                            instance_uuid)
 
 
+def volume_attachment_get_all(context, filters=None, marker=None, limit=None,
+                              offset=None, sort_keys=None, sort_dirs=None):
+    return IMPL.volume_attachment_get_all(context, filters, marker, limit,
+                                          offset, sort_keys, sort_dirs)
+
+
+def volume_attachment_get_all_by_project(context, project_id, filters=None,
+                                         marker=None, limit=None, offset=None,
+                                         sort_keys=None, sort_dirs=None):
+    return IMPL.volume_attachment_get_all_by_project(context, project_id,
+                                                     filters, marker, limit,
+                                                     offset, sort_keys,
+                                                     sort_dirs)
+
+
+def attachment_destroy(context, attachment_id):
+    """Destroy the attachment or raise if it does not exist."""
+    return IMPL.attachment_destroy(context, attachment_id)
+
+
 def volume_update_status_based_on_attachment(context, volume_id):
     """Update volume status according to attached instance id"""
     return IMPL.volume_update_status_based_on_attachment(context, volume_id)
@@ -368,6 +413,10 @@ def volume_has_undeletable_snapshots_filter():
     return IMPL.volume_has_undeletable_snapshots_filter()
 
 
+def volume_has_snapshots_in_a_cgsnapshot_filter():
+    return IMPL.volume_has_snapshots_in_a_cgsnapshot_filter()
+
+
 def volume_has_attachments_filter():
     return IMPL.volume_has_attachments_filter()
 
@@ -376,6 +425,10 @@ def volume_qos_allows_retype(new_vol_type):
     return IMPL.volume_qos_allows_retype(new_vol_type)
 
 
+def volume_has_other_project_snp_filter():
+    return IMPL.volume_has_other_project_snp_filter()
+
+
 ####################
 
 
@@ -410,13 +463,13 @@ def snapshot_get_all_by_project(context, project_id, filters=None, marker=None,
                                             sort_dirs, offset)
 
 
-def snapshot_get_by_host(context, host, filters=None):
+def snapshot_get_all_by_host(context, host, filters=None):
     """Get all snapshots belonging to a host.
 
     :param host: Include include snapshots only for specified host.
     :param filters: Filters for the query in the form of key/value.
     """
-    return IMPL.snapshot_get_by_host(context, host, filters)
+    return IMPL.snapshot_get_all_by_host(context, host, filters)
 
 
 def snapshot_get_all_for_cgsnapshot(context, project_id):
@@ -434,6 +487,11 @@ def snapshot_get_all_for_volume(context, volume_id):
     return IMPL.snapshot_get_all_for_volume(context, volume_id)
 
 
+def snapshot_get_latest_for_volume(context, volume_id):
+    """Get latest snapshot for a volume"""
+    return IMPL.snapshot_get_latest_for_volume(context, volume_id)
+
+
 def snapshot_update(context, snapshot_id, values):
     """Set the given properties on an snapshot and update it.
 
@@ -443,19 +501,28 @@ def snapshot_update(context, snapshot_id, values):
     return IMPL.snapshot_update(context, snapshot_id, values)
 
 
-def snapshot_data_get_for_project(context, project_id, volume_type_id=None):
+def snapshot_data_get_for_project(context, project_id, volume_type_id=None,
+                                  host=None):
     """Get count and gigabytes used for snapshots for specified project."""
     return IMPL.snapshot_data_get_for_project(context,
                                               project_id,
-                                              volume_type_id)
+                                              volume_type_id,
+                                              host=host)
 
 
-def snapshot_get_active_by_window(context, begin, end=None, project_id=None):
+def snapshot_get_all_active_by_window(context, begin, end=None,
+                                      project_id=None):
     """Get all the snapshots inside the window.
 
     Specifying a project_id will filter for a certain project.
     """
-    return IMPL.snapshot_get_active_by_window(context, begin, end, project_id)
+    return IMPL.snapshot_get_all_active_by_window(context, begin, end,
+                                                  project_id)
+
+
+def get_snapshot_summary(context, project_only, filters=None):
+    """Get snapshot summary."""
+    return IMPL.get_snapshot_summary(context, project_only, filters)
 
 
 ####################
@@ -620,12 +687,13 @@ def volume_type_destroy(context, id):
     return IMPL.volume_type_destroy(context, id)
 
 
-def volume_get_active_by_window(context, begin, end=None, project_id=None):
+def volume_get_all_active_by_window(context, begin, end=None, project_id=None):
     """Get all the volumes inside the window.
 
     Specifying a project_id will filter for a certain project.
     """
-    return IMPL.volume_get_active_by_window(context, begin, end, project_id)
+    return IMPL.volume_get_all_active_by_window(context, begin, end,
+                                                project_id)
 
 
 def volume_type_access_get_all(context, type_id):
@@ -643,6 +711,27 @@ def volume_type_access_remove(context, type_id, project_id):
     return IMPL.volume_type_access_remove(context, type_id, project_id)
 
 
+def project_default_volume_type_set(context, volume_type_id, project_id):
+    """Set default volume type for a project"""
+    return IMPL.project_default_volume_type_set(context, volume_type_id,
+                                                project_id)
+
+
+def project_default_volume_type_get(context, project_id=None):
+    """Get default volume type for a project"""
+    return IMPL.project_default_volume_type_get(context, project_id)
+
+
+def project_default_volume_type_unset(context, project_id):
+    """Unset default volume type for a project (hard delete)"""
+    return IMPL.project_default_volume_type_unset(context, project_id)
+
+
+def get_all_projects_with_default_type(context, volume_type_id):
+    """Get all the projects associated with a default type"""
+    return IMPL.get_all_projects_with_default_type(context, volume_type_id)
+
+
 ####################
 
 
@@ -996,7 +1085,7 @@ def quota_allocated_update(context, project_id,
                            resource, allocated):
     """Update allocated quota to subprojects or raise if it does not exist.
 
-    :raises: cinder.exception.ProjectQuotaNotFound
+    :raises cinder.exception.ProjectQuotaNotFound:
     """
     return IMPL.quota_allocated_update(context, project_id,
                                        resource, allocated)
@@ -1030,9 +1119,9 @@ def quota_class_get(context, class_name, resource):
     return IMPL.quota_class_get(context, class_name, resource)
 
 
-def quota_class_get_default(context):
+def quota_class_get_defaults(context):
     """Retrieve all default quotas."""
-    return IMPL.quota_class_get_default(context)
+    return IMPL.quota_class_get_defaults(context)
 
 
 def quota_class_get_all_by_name(context, class_name):
@@ -1138,6 +1227,14 @@ def backup_create(context, values):
     return IMPL.backup_create(context, values)
 
 
+def backup_metadata_get(context, backup_id):
+    return IMPL.backup_metadata_get(context, backup_id)
+
+
+def backup_metadata_update(context, backup_id, metadata, delete):
+    return IMPL.backup_metadata_update(context, backup_id, metadata, delete)
+
+
 def backup_get_all_by_project(context, project_id, filters=None, marker=None,
                               limit=None, offset=None, sort_keys=None,
                               sort_dirs=None):
@@ -1149,12 +1246,21 @@ def backup_get_all_by_project(context, project_id, filters=None, marker=None,
                                           sort_dirs=sort_dirs)
 
 
-def backup_get_all_by_volume(context, volume_id, filters=None):
+def backup_get_all_by_volume(context, volume_id, vol_project_id, filters=None):
     """Get all backups belonging to a volume."""
-    return IMPL.backup_get_all_by_volume(context, volume_id,
+    return IMPL.backup_get_all_by_volume(context, volume_id, vol_project_id,
                                          filters=filters)
 
 
+def backup_get_all_active_by_window(context, begin, end=None, project_id=None):
+    """Get all the backups inside the window.
+
+    Specifying a project_id will filter for a certain project.
+    """
+    return IMPL.backup_get_all_active_by_window(context, begin, end,
+                                                project_id)
+
+
 def backup_update(context, backup_id, values):
     """Set the given properties on a backup and update it.
 
@@ -1176,14 +1282,22 @@ def transfer_get(context, transfer_id):
     return IMPL.transfer_get(context, transfer_id)
 
 
-def transfer_get_all(context):
+def transfer_get_all(context, marker=None, limit=None, sort_keys=None,
+                     sort_dirs=None, filters=None, offset=None):
     """Get all volume transfer records."""
-    return IMPL.transfer_get_all(context)
+    return IMPL.transfer_get_all(context, marker=marker, limit=limit,
+                                 sort_keys=sort_keys, sort_dirs=sort_dirs,
+                                 filters=filters, offset=offset)
 
 
-def transfer_get_all_by_project(context, project_id):
+def transfer_get_all_by_project(context, project_id, marker=None,
+                                limit=None, sort_keys=None,
+                                sort_dirs=None, filters=None, offset=None):
     """Get all volume transfer records for specified project."""
-    return IMPL.transfer_get_all_by_project(context, project_id)
+    return IMPL.transfer_get_all_by_project(context, project_id, marker=marker,
+                                            limit=limit, sort_keys=sort_keys,
+                                            sort_dirs=sort_dirs,
+                                            filters=filters, offset=offset)
 
 
 def transfer_create(context, values):
@@ -1196,9 +1310,11 @@ def transfer_destroy(context, transfer_id):
     return IMPL.transfer_destroy(context, transfer_id)
 
 
-def transfer_accept(context, transfer_id, user_id, project_id):
+def transfer_accept(context, transfer_id, user_id, project_id,
+                    no_snapshots=False):
     """Accept a volume transfer."""
-    return IMPL.transfer_accept(context, transfer_id, user_id, project_id)
+    return IMPL.transfer_accept(context, transfer_id, user_id, project_id,
+                                no_snapshots=no_snapshots)
 
 
 ###################
@@ -1298,6 +1414,23 @@ def consistencygroup_include_in_cluster(context, cluster, partial_rename=True,
                                                     **filters)
 
 
+def group_include_in_cluster(context, cluster, partial_rename=True, **filters):
+    """Include all generic groups matching the filters into a cluster.
+
+    When partial_rename is set we will not set the cluster_name with cluster
+    parameter value directly, we'll replace provided cluster_name or host
+    filter value with cluster instead.
+
+    This is useful when we want to replace just the cluster name but leave
+    the backend and pool information as it is.  If we are using cluster_name
+    to filter, we'll use that same DB field to replace the cluster value and
+    leave the rest as it is.  Likewise if we use the host to filter.
+
+    Returns the number of generic groups that have been changed.
+    """
+    return IMPL.group_include_in_cluster(context, cluster, partial_rename,
+                                         **filters)
+
 ###################
 
 
@@ -1435,9 +1568,11 @@ def group_snapshot_get(context, group_snapshot_id):
     return IMPL.group_snapshot_get(context, group_snapshot_id)
 
 
-def group_snapshot_get_all(context, filters=None):
+def group_snapshot_get_all(context, filters=None, marker=None, limit=None,
+                           offset=None, sort_keys=None, sort_dirs=None):
     """Get all group snapshots."""
-    return IMPL.group_snapshot_get_all(context, filters)
+    return IMPL.group_snapshot_get_all(context, filters, marker, limit,
+                                       offset, sort_keys, sort_dirs)
 
 
 def group_snapshot_create(context, values):
@@ -1445,14 +1580,24 @@ def group_snapshot_create(context, values):
     return IMPL.group_snapshot_create(context, values)
 
 
-def group_snapshot_get_all_by_group(context, group_id, filters=None):
+def group_snapshot_get_all_by_group(context, group_id, filters=None,
+                                    marker=None, limit=None,
+                                    offset=None, sort_keys=None,
+                                    sort_dirs=None):
     """Get all group snapshots belonging to a group."""
-    return IMPL.group_snapshot_get_all_by_group(context, group_id, filters)
+    return IMPL.group_snapshot_get_all_by_group(context, group_id,
+                                                filters, marker, limit,
+                                                offset, sort_keys, sort_dirs)
 
 
-def group_snapshot_get_all_by_project(context, project_id, filters=None):
+def group_snapshot_get_all_by_project(context, project_id, filters=None,
+                                      marker=None, limit=None,
+                                      offset=None, sort_keys=None,
+                                      sort_dirs=None):
     """Get all group snapshots belonging to a project."""
-    return IMPL.group_snapshot_get_all_by_project(context, project_id, filters)
+    return IMPL.group_snapshot_get_all_by_project(context, project_id,
+                                                  filters, marker, limit,
+                                                  offset, sort_keys, sort_dirs)
 
 
 def group_snapshot_update(context, group_snapshot_id, values):
@@ -1492,6 +1637,16 @@ def get_booleans_for_table(table_name):
 ###################
 
 
+def reset_active_backend(context, enable_replication, active_backend_id,
+                         backend_host):
+    """Reset the active backend for a host."""
+    return IMPL.reset_active_backend(context, enable_replication,
+                                     active_backend_id, backend_host)
+
+
+###################
+
+
 def driver_initiator_data_insert_by_key(context, initiator,
                                         namespace, key, value):
     """Updates DriverInitiatorData entry.
@@ -1516,11 +1671,12 @@ def driver_initiator_data_get(context, initiator, namespace):
 ###################
 
 
-def image_volume_cache_create(context, host, image_id, image_updated_at,
-                              volume_id, size):
+def image_volume_cache_create(context, host, cluster_name, image_id,
+                              image_updated_at, volume_id, size):
     """Create a new image volume cache entry."""
     return IMPL.image_volume_cache_create(context,
                                           host,
+                                          cluster_name,
                                           image_id,
                                           image_updated_at,
                                           volume_id,
@@ -1532,11 +1688,11 @@ def image_volume_cache_delete(context, volume_id):
     return IMPL.image_volume_cache_delete(context, volume_id)
 
 
-def image_volume_cache_get_and_update_last_used(context, image_id, host):
+def image_volume_cache_get_and_update_last_used(context, image_id, **filters):
     """Query for an image volume cache entry."""
     return IMPL.image_volume_cache_get_and_update_last_used(context,
                                                             image_id,
-                                                            host)
+                                                            **filters)
 
 
 def image_volume_cache_get_by_volume_id(context, volume_id):
@@ -1544,9 +1700,28 @@ def image_volume_cache_get_by_volume_id(context, volume_id):
     return IMPL.image_volume_cache_get_by_volume_id(context, volume_id)
 
 
-def image_volume_cache_get_all_for_host(context, host):
+def image_volume_cache_get_all(context, **filters):
     """Query for all image volume cache entry for a host."""
-    return IMPL.image_volume_cache_get_all_for_host(context, host)
+    return IMPL.image_volume_cache_get_all(context, **filters)
+
+
+def image_volume_cache_include_in_cluster(context, cluster,
+                                          partial_rename=True, **filters):
+    """Include in cluster image volume cache entries matching the filters.
+
+    When partial_rename is set we will not set the cluster_name with cluster
+    parameter value directly, we'll replace provided cluster_name or host
+    filter value with cluster instead.
+
+    This is useful when we want to replace just the cluster name but leave
+    the backend and pool information as it is.  If we are using cluster_name
+    to filter, we'll use that same DB field to replace the cluster value and
+    leave the rest as it is.  Likewise if we use the host to filter.
+
+    Returns the number of volumes that have been changed.
+    """
+    return IMPL.image_volume_cache_include_in_cluster(
+        context, cluster, partial_rename, **filters)
 
 
 ###################
@@ -1574,9 +1749,25 @@ def message_destroy(context, message_id):
     return IMPL.message_destroy(context, message_id)
 
 
+def cleanup_expired_messages(context):
+    """Soft delete expired messages"""
+    return IMPL.cleanup_expired_messages(context)
+
+
 ###################
 
 
+def workers_init():
+    """Check if DB supports subsecond resolution and set global flag.
+
+    MySQL 5.5 doesn't support subsecond resolution in datetime fields, so we
+    have to take it into account when working with the worker's table.
+
+    Once we drop support for MySQL 5.5 we can remove this method.
+    """
+    return IMPL.workers_init()
+
+
 def worker_create(context, **values):
     """Create a worker entry from optional arguments."""
     return IMPL.worker_create(context, **values)
@@ -1642,6 +1833,39 @@ class Condition(object):
             raise ValueError(_('Condition has no field.'))
         return field
 
+###################
+
+
+def attachment_specs_exist(context):
+    """Check if there are attachment specs left."""
+    return IMPL.attachment_specs_exist(context)
+
+
+def attachment_specs_get(context, attachment_id):
+    """DEPRECATED: Get all specs for an attachment."""
+    return IMPL.attachment_specs_get(context, attachment_id)
+
+
+def attachment_specs_delete(context, attachment_id, key):
+    """DEPRECATED: Delete the given attachment specs item."""
+    return IMPL.attachment_specs_delete(context, attachment_id, key)
+
+
+def attachment_specs_update_or_create(context,
+                                      attachment_id,
+                                      specs):
+    """DEPRECATED: Create or update attachment specs.
+
+    This adds or modifies the key/value pairs specified in the attachment
+    specs dict argument.
+    """
+    return IMPL.attachment_specs_update_or_create(context,
+                                                  attachment_id,
+                                                  specs)
+
+
+###################
+
 
 class Not(Condition):
     """Class for negated condition values for conditional_update.
@@ -1729,7 +1953,7 @@ def conditional_update(context, model, values, expected_values, filters=(),
                             equivalent to read_deleted.
     :param project_only: Should the query be limited to context's project.
     :param order: Specific order of fields in which to update the values
-    :returns number of db rows that were updated.
+    :returns: Number of db rows that were updated.
     """
     return IMPL.conditional_update(context, model, values, expected_values,
                                    filters, include_deleted, project_only,
