@@ -31,6 +31,10 @@ import os
 import re
 import tempfile
 
+import cryptography
+from cursive import exception as cursive_exception
+from cursive import signature_utils
+from eventlet import tpool
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -39,29 +43,42 @@ from oslo_utils import imageutils
 from oslo_utils import timeutils
 from oslo_utils import units
 import psutil
+import six
 
 from cinder import exception
 from cinder.i18n import _
+from cinder.image import accelerator
+from cinder.image import glance
 from cinder import utils
 from cinder.volume import throttling
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
-image_helper_opts = [cfg.StrOpt('image_conversion_dir',
-                                default='$state_path/conversion',
-                                help='Directory used for temporary storage '
-                                'during image conversion'), ]
+image_opts = [
+    cfg.StrOpt('image_conversion_dir',
+               default='$state_path/conversion',
+               help='Directory used for temporary storage '
+               'during image conversion'),
+    cfg.BoolOpt('image_compress_on_upload',
+                default=True,
+                help='When possible, compress images uploaded '
+                'to the image service'),
+    cfg.IntOpt('image_conversion_cpu_limit',
+               default=60,
+               help='CPU time limit in seconds to convert the image'),
+    cfg.IntOpt('image_conversion_address_space_limit',
+               default=1,
+               help='Address space limit in gigabytes to convert the image'),
+]
 
 CONF = cfg.CONF
-CONF.register_opts(image_helper_opts)
+CONF.register_opts(image_opts)
 
 QEMU_IMG_LIMITS = processutils.ProcessLimits(
-    cpu_time=8,
-    address_space=1 * units.Gi)
+    cpu_time=CONF.image_conversion_cpu_limit,
+    address_space=CONF.image_conversion_address_space_limit * units.Gi)
 
-VALID_DISK_FORMATS = ('raw', 'vmdk', 'vdi', 'qcow2',
-                      'vhd', 'vhdx', 'ploop')
 
 QEMU_IMG_FORMAT_MAP = {
     # Convert formats of Glance images to how they are processed with qemu-img.
@@ -75,9 +92,20 @@ QEMU_IMG_VERSION = None
 QEMU_IMG_MIN_FORCE_SHARE_VERSION = [2, 10, 0]
 QEMU_IMG_MIN_CONVERT_LUKS_VERSION = '2.10'
 
+COMPRESSIBLE_IMAGE_FORMATS = ('qcow2',)
 
-def validate_disk_format(disk_format):
-    return disk_format in VALID_DISK_FORMATS
+
+def validate_stores_id(context, image_service_store_id):
+    image_service = glance.get_default_image_service()
+    stores_info = image_service.get_stores(context)['stores']
+    for info in stores_info:
+        if image_service_store_id == info['id']:
+            if info.get('read-only') == "true":
+                raise exception.GlanceStoreReadOnly(
+                    store_id=image_service_store_id)
+            return
+
+    raise exception.GlanceStoreNotFound(store_id=image_service_store_id)
 
 
 def fixup_disk_format(disk_format):
@@ -136,12 +164,63 @@ def get_qemu_img_version():
 
 
 def qemu_img_supports_force_share():
-    return get_qemu_img_version() > [2, 10, 0]
+    return get_qemu_img_version() >= [2, 10, 0]
+
+
+def _get_qemu_convert_luks_cmd(src, dest, out_format, src_format=None,
+                               out_subformat=None, cache_mode=None,
+                               prefix=None, cipher_spec=None,
+                               passphrase_file=None,
+                               src_passphrase_file=None):
+
+    cmd = ['qemu-img', 'convert']
+
+    if prefix:
+        cmd = list(prefix) + cmd
+
+    if cache_mode:
+        cmd += ('-t', cache_mode)
+
+    obj1 = ['--object',
+            'secret,id=sec1,format=raw,file=%s' % src_passphrase_file]
+    obj2 = ['--object',
+            'secret,id=sec2,format=raw,file=%s' % passphrase_file]
+
+    src_opts = 'encrypt.format=luks,encrypt.key-secret=sec1,' \
+               'file.filename=%s' % src
+
+    image_opts = ['--image-opts', src_opts]
+    output_opts = ['-O', 'luks', '-o', 'key-secret=sec2', dest]
+
+    command = cmd + obj1 + obj2 + image_opts + output_opts
+    return command
 
 
 def _get_qemu_convert_cmd(src, dest, out_format, src_format=None,
                           out_subformat=None, cache_mode=None,
-                          prefix=None, cipher_spec=None, passphrase_file=None):
+                          prefix=None, cipher_spec=None,
+                          passphrase_file=None, compress=False,
+                          src_passphrase_file=None):
+
+    if src_passphrase_file is not None:
+        if passphrase_file is None:
+            message = _("Can't create unencrypted volume %(format)s "
+                        "from an encrypted source volume."
+                        ) % {'format': out_format}
+            LOG.error(message)
+            # TODO(enriquetaso): handle encrypted->unencrypted
+            raise exception.NotSupportedOperation(operation=message)
+        return _get_qemu_convert_luks_cmd(
+            src,
+            dest,
+            out_format,
+            src_format=src_format,
+            out_subformat=out_subformat,
+            cache_mode=cache_mode,
+            prefix=None,
+            cipher_spec=cipher_spec,
+            passphrase_file=passphrase_file,
+            src_passphrase_file=src_passphrase_file)
 
     if out_format == 'vhd':
         # qemu-img still uses the legacy vpc name
@@ -155,6 +234,10 @@ def _get_qemu_convert_cmd(src, dest, out_format, src_format=None,
     if cache_mode:
         cmd += ('-t', cache_mode)
 
+    if CONF.image_compress_on_upload and compress:
+        if out_format in COMPRESSIBLE_IMAGE_FORMATS:
+            cmd += ('-c',)
+
     if out_subformat:
         cmd += ('-o', 'subformat=%s' % out_subformat)
 
@@ -206,8 +289,28 @@ def check_qemu_img_version(minimum_version):
 
 def _convert_image(prefix, source, dest, out_format,
                    out_subformat=None, src_format=None,
-                   run_as_root=True, cipher_spec=None, passphrase_file=None):
-    """Convert image to other format."""
+                   run_as_root=True, cipher_spec=None,
+                   passphrase_file=None, compress=False,
+                   src_passphrase_file=None):
+    """Convert image to other format.
+
+    NOTE: If the qemu-img convert command fails and this function raises an
+    exception, a non-empty dest file may be left in the filesystem.
+    It is the responsibility of the caller to decide what to do with this file.
+
+    :param prefix: command prefix, i.e. cgexec for throttling
+    :param source: source filename
+    :param dest: destination filename
+    :param out_format: output image format of qemu-img
+    :param out_subformat: output image subformat
+    :param src_format: source image format
+    :param run_as_root: run qemu-img as root
+    :param cipher_spec: encryption details
+    :param passphrase_file: filename containing luks passphrase
+    :param compress: compress w/ qemu-img when possible (best effort)
+    :param src_passphrase_file: filename containing source volume's
+                                luks passphrase
+    """
 
     # Check whether O_DIRECT is supported and set '-t none' if it is
     # This is needed to ensure that all data hit the device before
@@ -234,10 +337,27 @@ def _convert_image(prefix, source, dest, out_format,
                                 cache_mode=cache_mode,
                                 prefix=prefix,
                                 cipher_spec=cipher_spec,
-                                passphrase_file=passphrase_file)
+                                passphrase_file=passphrase_file,
+                                compress=compress,
+                                src_passphrase_file=src_passphrase_file)
 
     start_time = timeutils.utcnow()
-    utils.execute(*cmd, run_as_root=run_as_root)
+
+    # If there is not enough space on the conversion partition, include
+    # the partitions's name in the error message.
+    try:
+        utils.execute(*cmd, run_as_root=run_as_root)
+    except processutils.ProcessExecutionError as ex:
+        if "No space left" in ex.stderr and CONF.image_conversion_dir in dest:
+            conversion_dir = CONF.image_conversion_dir
+            while not os.path.ismount(conversion_dir):
+                conversion_dir = os.path.dirname(conversion_dir)
+
+            message = _("Insufficient free space on %(location)s for image "
+                        "conversion.") % {'location': conversion_dir}
+            LOG.error(message)
+
+        raise
     duration = timeutils.delta_seconds(start_time, timeutils.utcnow())
 
     # NOTE(jdg): use a default of 1, mostly for unit test, but in
@@ -270,7 +390,8 @@ def _convert_image(prefix, source, dest, out_format,
 
 def convert_image(source, dest, out_format, out_subformat=None,
                   src_format=None, run_as_root=True, throttle=None,
-                  cipher_spec=None, passphrase_file=None):
+                  cipher_spec=None, passphrase_file=None,
+                  compress=False, src_passphrase_file=None):
     if not throttle:
         throttle = throttling.Throttle.get_default()
     with throttle.subcommand(source, dest) as throttle_cmd:
@@ -281,7 +402,9 @@ def convert_image(source, dest, out_format, out_subformat=None,
                        src_format=src_format,
                        run_as_root=run_as_root,
                        cipher_spec=cipher_spec,
-                       passphrase_file=passphrase_file)
+                       passphrase_file=passphrase_file,
+                       compress=compress,
+                       src_passphrase_file=src_passphrase_file)
 
 
 def resize_image(source, size, run_as_root=False):
@@ -290,6 +413,81 @@ def resize_image(source, size, run_as_root=False):
     utils.execute(*cmd, run_as_root=run_as_root)
 
 
+def _verify_image(img_file, verifier):
+    # This methods must be called from a native thread, as the file I/O may
+    # not yield to other greenthread in some cases, and since the update and
+    # verify operations are CPU bound there would not be any yielding either,
+    # which could lead to thread starvation.
+    while True:
+        chunk = img_file.read(1024)
+        if not chunk:
+            break
+        verifier.update(chunk)
+    verifier.verify()
+
+
+def verify_glance_image_signature(context, image_service, image_id, path):
+    verifier = None
+    image_meta = image_service.show(context, image_id)
+    image_properties = image_meta.get('properties', {})
+    img_signature = image_properties.get('img_signature')
+    img_sig_hash_method = image_properties.get('img_signature_hash_method')
+    img_sig_cert_uuid = image_properties.get('img_signature_certificate_uuid')
+    img_sig_key_type = image_properties.get('img_signature_key_type')
+    if all(m is None for m in [img_signature,
+                               img_sig_cert_uuid,
+                               img_sig_hash_method,
+                               img_sig_key_type]):
+        # NOTE(tommylikehu): We won't verify the image signature
+        # if none of the signature metadata presents.
+        return False
+    if any(m is None for m in [img_signature,
+                               img_sig_cert_uuid,
+                               img_sig_hash_method,
+                               img_sig_key_type]):
+        LOG.error('Image signature metadata for image %s is '
+                  'incomplete.', image_id)
+        raise exception.InvalidSignatureImage(image_id=image_id)
+
+    try:
+        verifier = signature_utils.get_verifier(
+            context=context,
+            img_signature_certificate_uuid=img_sig_cert_uuid,
+            img_signature_hash_method=img_sig_hash_method,
+            img_signature=img_signature,
+            img_signature_key_type=img_sig_key_type,
+        )
+    except cursive_exception.SignatureVerificationError:
+        message = _('Failed to get verifier for image: %s') % image_id
+        LOG.error(message)
+        raise exception.ImageSignatureVerificationException(
+            reason=message)
+    if verifier:
+        with fileutils.remove_path_on_error(path):
+            with open(path, "rb") as tem_file:
+                try:
+                    tpool.execute(_verify_image, tem_file, verifier)
+                    LOG.info('Image signature verification succeeded '
+                             'for image: %s', image_id)
+                    return True
+                except cryptography.exceptions.InvalidSignature:
+                    message = _('Image signature verification '
+                                'failed for image: %s') % image_id
+                    LOG.error(message)
+                    raise exception.ImageSignatureVerificationException(
+                        reason=message)
+                except Exception as ex:
+                    message = _('Failed to verify signature for '
+                                'image: %(image)s due to '
+                                'error: %(error)s ') % {'image': image_id,
+                                                        'error':
+                                                            six.text_type(ex)}
+                    LOG.error(message)
+                    raise exception.ImageSignatureVerificationException(
+                        reason=message)
+    return False
+
+
 def fetch(context, image_service, image_id, path, _user_id, _project_id):
     # TODO(vish): Improve context handling and add owner and auth data
     #             when it is added to glance.  Right now there is no
@@ -299,7 +497,8 @@ def fetch(context, image_service, image_id, path, _user_id, _project_id):
     with fileutils.remove_path_on_error(path):
         with open(path, "wb") as image_file:
             try:
-                image_service.download(context, image_id, image_file)
+                image_service.download(context, image_id,
+                                       tpool.Proxy(image_file))
             except IOError as e:
                 if e.errno == errno.ENOSPC:
                     params = {'path': os.path.dirname(path),
@@ -311,6 +510,12 @@ def fetch(context, image_service, image_id, path, _user_id, _project_id):
                     raise exception.ImageTooBig(image_id=image_id,
                                                 reason=reason)
 
+                reason = ("IOError: %(errno)s %(strerror)s" %
+                          {'errno': e.errno, 'strerror': e.strerror})
+                LOG.error(reason)
+                raise exception.ImageDownloadFailed(image_href=image_id,
+                                                    reason=reason)
+
     duration = timeutils.delta_seconds(start_time, timeutils.utcnow())
 
     # NOTE(jdg): use a default of 1, mostly for unit test, but in
@@ -397,19 +602,16 @@ def fetch_verify_image(context, image_service, image_id, dest,
             # NOTE(xqueralt): If the image virtual size doesn't fit in the
             # requested volume there is no point on resizing it because it will
             # generate an unusable image.
-            if size is not None and data.virtual_size > size:
-                params = {'image_size': data.virtual_size, 'volume_size': size}
-                reason = _("Size is %(image_size)dGB and doesn't fit in a "
-                           "volume of size %(volume_size)dGB.") % params
-                raise exception.ImageUnacceptable(image_id=image_id,
-                                                  reason=reason)
+            if size is not None:
+                check_virtual_size(data.virtual_size, size, image_id)
 
 
 def fetch_to_vhd(context, image_service,
-                 image_id, dest, blocksize,
+                 image_id, dest, blocksize, volume_subformat=None,
                  user_id=None, project_id=None, run_as_root=True):
     fetch_to_volume_format(context, image_service, image_id, dest, 'vpc',
-                           blocksize, user_id, project_id,
+                           blocksize, volume_subformat=volume_subformat,
+                           user_id=user_id, project_id=project_id,
                            run_as_root=run_as_root)
 
 
@@ -417,8 +619,8 @@ def fetch_to_raw(context, image_service,
                  image_id, dest, blocksize,
                  user_id=None, project_id=None, size=None, run_as_root=True):
     fetch_to_volume_format(context, image_service, image_id, dest, 'raw',
-                           blocksize, user_id, project_id, size,
-                           run_as_root=run_as_root)
+                           blocksize, user_id=user_id, project_id=project_id,
+                           size=size, run_as_root=run_as_root)
 
 
 def fetch_to_volume_format(context, image_service,
@@ -428,6 +630,17 @@ def fetch_to_volume_format(context, image_service,
     qemu_img = True
     image_meta = image_service.show(context, image_id)
 
+    allow_image_compression = CONF.allow_compression_on_image_upload
+    if image_meta and (image_meta.get('container_format') == 'compressed'):
+        if allow_image_compression is False:
+            compression_param = {'container_format':
+                                 image_meta.get('container_format')}
+            raise exception.ImageUnacceptable(
+                image_id=image_id,
+                reason=_("Image compression disallowed, "
+                         "but container_format is "
+                         "%(container_format)s.") % compression_param)
+
     # NOTE(avishay): I'm not crazy about creating temp files which may be
     # large and cause disk full errors which would confuse users.
     # Unfortunately it seems that you can't pipe to 'qemu-img convert' because
@@ -465,16 +678,12 @@ def fetch_to_volume_format(context, image_service,
             return
 
         data = qemu_img_info(tmp, run_as_root=run_as_root)
-        virt_size = int(math.ceil(float(data.virtual_size) / units.Gi))
 
         # NOTE(xqueralt): If the image virtual size doesn't fit in the
         # requested volume there is no point on resizing it because it will
         # generate an unusable image.
-        if size is not None and virt_size > size:
-            params = {'image_size': virt_size, 'volume_size': size}
-            reason = _("Size is %(image_size)dGB and doesn't fit in a "
-                       "volume of size %(volume_size)dGB.") % params
-            raise exception.ImageUnacceptable(image_id=image_id, reason=reason)
+        if size is not None:
+            check_virtual_size(data.virtual_size, size, image_id)
 
         fmt = data.file_format
         if fmt is None:
@@ -489,9 +698,20 @@ def fetch_to_volume_format(context, image_service,
                 reason=_("fmt=%(fmt)s backed by:%(backing_file)s")
                 % {'fmt': fmt, 'backing_file': backing_file, })
 
-        # NOTE(e0ne): check for free space in destination directory before
-        # image conversion.
-        check_available_space(dest, data.virtual_size, image_id)
+        # NOTE(ZhengMa): This is used to do image decompression on image
+        # downloading with 'compressed' container_format. It is a
+        # transparent level between original image downloaded from
+        # Glance and Cinder image service. So the source file path is
+        # the same with destination file path.
+        if image_meta.get('container_format') == 'compressed':
+            LOG.debug("Found image with compressed container format")
+            if not accelerator.is_gzip_compressed(tmp):
+                raise exception.ImageUnacceptable(
+                    image_id=image_id,
+                    reason=_("Unsupported compressed image format found. "
+                             "Only gzip is supported currently"))
+            accel = accelerator.ImageAccel(tmp, tmp)
+            accel.decompress_img(run_as_root=run_as_root)
 
         # NOTE(jdg): I'm using qemu-img convert to write
         # to the volume regardless if it *needs* conversion or not
@@ -500,8 +720,8 @@ def fetch_to_volume_format(context, image_service,
         # check via 'qemu-img info' that what we copied was in fact a raw
         # image and not a different format with a backing file, which may be
         # malicious.
-        LOG.debug("%s was %s, converting to %s ", image_id, fmt, volume_format)
         disk_format = fixup_disk_format(image_meta['disk_format'])
+        LOG.debug("%s was %s, converting to %s", image_id, fmt, volume_format)
 
         convert_image(tmp, dest, volume_format,
                       out_subformat=volume_subformat,
@@ -519,19 +739,29 @@ def _validate_file_format(image_data, expected_format):
 
 
 def upload_volume(context, image_service, image_meta, volume_path,
-                  volume_format='raw', run_as_root=True):
+                  volume_format='raw', run_as_root=True, compress=True,
+                  store_id=None, base_image_ref=None):
+    # NOTE: You probably want to use volume_utils.upload_volume(),
+    # not this function.
     image_id = image_meta['id']
-    if (image_meta['disk_format'] == volume_format):
-        LOG.debug("%s was %s, no need to convert to %s",
-                  image_id, volume_format, image_meta['disk_format'])
-        if os.name == 'nt' or os.access(volume_path, os.R_OK):
-            with open(volume_path, 'rb') as image_file:
-                image_service.update(context, image_id, {}, image_file)
-        else:
-            with utils.temporary_chown(volume_path):
+    if image_meta.get('container_format') != 'compressed':
+        if (image_meta['disk_format'] == volume_format):
+            LOG.debug("%s was %s, no need to convert to %s",
+                      image_id, volume_format, image_meta['disk_format'])
+            if os.name == 'nt' or os.access(volume_path, os.R_OK):
                 with open(volume_path, 'rb') as image_file:
-                    image_service.update(context, image_id, {}, image_file)
-        return
+                    image_service.update(context, image_id, {},
+                                         tpool.Proxy(image_file),
+                                         store_id=store_id,
+                                         base_image_ref=base_image_ref)
+            else:
+                with utils.temporary_chown(volume_path):
+                    with open(volume_path, 'rb') as image_file:
+                        image_service.update(context, image_id, {},
+                                             tpool.Proxy(image_file),
+                                             store_id=store_id,
+                                             base_image_ref=base_image_ref)
+            return
 
     with temporary_file() as tmp:
         LOG.debug("%s was %s, converting to %s",
@@ -552,7 +782,8 @@ def upload_volume(context, image_service, image_meta, volume_path,
 
         out_format = fixup_disk_format(image_meta['disk_format'])
         convert_image(volume_path, tmp, out_format,
-                      run_as_root=run_as_root)
+                      run_as_root=run_as_root,
+                      compress=compress)
 
         data = qemu_img_info(tmp, run_as_root=run_as_root)
         if data.file_format != out_format:
@@ -561,8 +792,19 @@ def upload_volume(context, image_service, image_meta, volume_path,
                 reason=_("Converted to %(f1)s, but format is now %(f2)s") %
                 {'f1': out_format, 'f2': data.file_format})
 
+        # NOTE(ZhengMa): This is used to do image compression on image
+        # uploading with 'compressed' container_format.
+        # Compress file 'tmp' in-place
+        if image_meta.get('container_format') == 'compressed':
+            LOG.debug("Container_format set to 'compressed', compressing "
+                      "image before uploading.")
+            accel = accelerator.ImageAccel(tmp, tmp)
+            accel.compress_img(run_as_root=run_as_root)
         with open(tmp, 'rb') as image_file:
-            image_service.update(context, image_id, {}, image_file)
+            image_service.update(context, image_id, {},
+                                 tpool.Proxy(image_file),
+                                 store_id=store_id,
+                                 base_image_ref=base_image_ref)
 
 
 def check_virtual_size(virtual_size, volume_size, image_id):
@@ -587,9 +829,11 @@ def check_available_space(dest, image_size, image_id):
 
     free_space = psutil.disk_usage(dest).free
     if free_space <= image_size:
-        msg = ('There is no space to convert image. '
-               'Requested: %(image_size)s, available: %(free_space)s'
-               ) % {'image_size': image_size, 'free_space': free_space}
+        msg = ('There is no space on %(dest_dir)s to convert image. '
+               'Requested: %(image_size)s, available: %(free_space)s.'
+               ) % {'dest_dir': dest,
+                    'image_size': image_size,
+                    'free_space': free_space}
         raise exception.ImageTooBig(image_id=image_id, reason=msg)
 
 
@@ -629,9 +873,7 @@ def coalesce_vhd(vhd_path):
 
 
 def create_temporary_file(*args, **kwargs):
-    if (CONF.image_conversion_dir and not
-            os.path.exists(CONF.image_conversion_dir)):
-        os.makedirs(CONF.image_conversion_dir)
+    fileutils.ensure_tree(CONF.image_conversion_dir)
 
     fd, tmp = tempfile.mkstemp(dir=CONF.image_conversion_dir, *args, **kwargs)
     os.close(fd)
@@ -673,9 +915,7 @@ def temporary_file(*args, **kwargs):
 
 
 def temporary_dir():
-    if (CONF.image_conversion_dir and not
-            os.path.exists(CONF.image_conversion_dir)):
-        os.makedirs(CONF.image_conversion_dir)
+    fileutils.ensure_tree(CONF.image_conversion_dir)
 
     return utils.tempdir(dir=CONF.image_conversion_dir)
 
