@@ -12,21 +12,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from keystoneauth1 import identity
+from keystoneauth1 import loading as ka_loading
+from keystoneclient import client
+from keystoneclient import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from keystoneclient.auth.identity.generic import token
-from keystoneclient import client
-from keystoneclient import exceptions
-from keystoneclient import session
-
 from cinder import db
 from cinder import exception
-from cinder.i18n import _, _LW
+from cinder.i18n import _
 
 CONF = cfg.CONF
-CONF.import_opt('auth_uri', 'keystonemiddleware.auth_token.__init__',
-                'keystone_authtoken')
+CONF.import_group('keystone_authtoken',
+                  'keystonemiddleware.auth_token.__init__')
 
 LOG = logging.getLogger(__name__)
 
@@ -37,8 +36,10 @@ class GenericProjectInfo(object):
                  project_parent_id=None,
                  project_subtree=None,
                  project_parent_tree=None,
-                 is_admin_project=False):
+                 is_admin_project=False,
+                 domain_id=None):
         self.id = project_id
+        self.domain_id = domain_id
         self.keystone_api_version = project_keystone_api_version
         self.parent_id = project_parent_id
         self.subtree = project_subtree
@@ -107,6 +108,7 @@ def get_project_hierarchy(context, project_id, subtree_as_ids=False,
                                         parents_as_ids=parents_as_ids)
 
         generic_project.parent_id = None
+        generic_project.domain_id = project.domain_id
         if project.parent_id != project.domain_id:
             generic_project.parent_id = project.parent_id
 
@@ -147,6 +149,7 @@ def update_alloc_to_next_hard_limit(context, resources, deltas, res,
                                     expire, project_id):
     from cinder import quota
     QUOTAS = quota.QUOTAS
+    GROUP_QUOTAS = quota.GROUP_QUOTAS
     reservations = []
     projects = get_project_hierarchy(context, project_id,
                                      parents_as_ids=True).parents
@@ -156,8 +159,12 @@ def update_alloc_to_next_hard_limit(context, resources, deltas, res,
     while projects and not hard_limit_found:
         cur_proj_id = list(projects)[0]
         projects = projects[cur_proj_id]
-        cur_quota_lim = QUOTAS.get_by_project_or_default(
-            context, cur_proj_id, res)
+        if res == 'groups':
+            cur_quota_lim = GROUP_QUOTAS.get_by_project_or_default(
+                context, cur_proj_id, res)
+        else:
+            cur_quota_lim = QUOTAS.get_by_project_or_default(
+                context, cur_proj_id, res)
         hard_limit_found = (cur_quota_lim != -1)
         cur_quota = {res: cur_quota_lim}
         cur_delta = {res: deltas[res]}
@@ -221,21 +228,46 @@ def _keystone_client(context, version=(3, 0)):
     :param version: version of Keystone to request
     :return: keystoneclient.client.Client object
     """
-    auth_plugin = token.Token(
-        auth_url=CONF.keystone_authtoken.auth_uri,
-        token=context.auth_token,
-        project_id=context.project_id)
-    client_session = session.Session(auth=auth_plugin,
-                                     verify=False if
-                                     CONF.keystone_authtoken.insecure else
-                                     (CONF.keystone_authtoken.cafile or True))
-    return client.Client(auth_url=CONF.keystone_authtoken.auth_uri,
+    if context.system_scope is not None:
+        auth_plugin = identity.Token(
+            auth_url=CONF.keystone_authtoken.auth_url,
+            token=context.auth_token,
+            system_scope=context.system_scope
+        )
+    elif context.domain_id is not None:
+        auth_plugin = identity.Token(
+            auth_url=CONF.keystone_authtoken.auth_url,
+            token=context.auth_token,
+            domain_id=context.domain_id
+        )
+    elif context.project_id is not None:
+        auth_plugin = identity.Token(
+            auth_url=CONF.keystone_authtoken.auth_url,
+            token=context.auth_token,
+            project_id=context.project_id
+        )
+    else:
+        # We're dealing with an unscoped token from keystone that doesn't
+        # carry any authoritative power outside of the user simplify proving
+        # they know their own password. This token isn't associated with any
+        # authorization target (e.g., system, domain, or project).
+        auth_plugin = context.get_auth_plugin()
+
+    client_session = ka_loading.session.Session().load_from_options(
+        auth=auth_plugin,
+        insecure=CONF.keystone_authtoken.insecure,
+        cacert=CONF.keystone_authtoken.cafile,
+        key=CONF.keystone_authtoken.keyfile,
+        cert=CONF.keystone_authtoken.certfile,
+        split_loggers=CONF.service_user.split_loggers)
+    return client.Client(auth_url=CONF.keystone_authtoken.auth_url,
                          session=client_session, version=version)
 
 
 OVER_QUOTA_RESOURCE_EXCEPTIONS = {'snapshots': exception.SnapshotLimitExceeded,
                                   'backups': exception.BackupLimitExceeded,
-                                  'volumes': exception.VolumeLimitExceeded, }
+                                  'volumes': exception.VolumeLimitExceeded,
+                                  'groups': exception.GroupLimitExceeded}
 
 
 def process_reserve_over_quota(context, over_quota_exception,
@@ -261,9 +293,9 @@ def process_reserve_over_quota(context, over_quota_exception,
 
     for over in overs:
         if 'gigabytes' in over:
-            msg = _LW("Quota exceeded for %(s_pid)s, tried to create "
-                      "%(s_size)dG %(s_resource)s (%(d_consumed)dG of "
-                      "%(d_quota)dG already consumed).")
+            msg = ("Quota exceeded for %(s_pid)s, tried to create "
+                   "%(s_size)dG %(s_resource)s (%(d_consumed)dG of "
+                   "%(d_quota)dG already consumed).")
             LOG.warning(msg, {'s_pid': context.project_id,
                               's_size': size,
                               's_resource': resource[:-1],
@@ -280,9 +312,9 @@ def process_reserve_over_quota(context, over_quota_exception,
                 quota=quotas[over])
         if (resource in OVER_QUOTA_RESOURCE_EXCEPTIONS.keys() and
                 resource in over):
-            msg = _LW("Quota exceeded for %(s_pid)s, tried to create "
-                      "%(s_resource)s (%(d_consumed)d %(s_resource)ss "
-                      "already consumed).")
+            msg = ("Quota exceeded for %(s_pid)s, tried to create "
+                   "%(s_resource)s (%(d_consumed)d %(s_resource)ss "
+                   "already consumed).")
             LOG.warning(msg, {'s_pid': context.project_id,
                               'd_consumed': _consumed(over),
                               's_resource': resource[:-1]})
