@@ -15,25 +15,29 @@
 
 import datetime
 
+import ddt
 from oslo_serialization import jsonutils
+from six.moves import http_client
 
 import cinder
 from cinder.api.openstack import wsgi
 from cinder import context
-from cinder import test
 from cinder.tests.unit.api import fakes
-from cinder.tests.unit.api.v2 import stubs
+from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
 
 UUID = fakes.FAKE_UUID
 
 
+@ddt.ddt
 class SchedulerHintsTestCase(test.TestCase):
 
     def setUp(self):
         super(SchedulerHintsTestCase, self).setUp()
-        self.fake_instance = stubs.stub_volume(fake.VOLUME_ID, uuid=UUID)
+        self.fake_instance = v2_fakes.create_fake_volume(fake.VOLUME_ID,
+                                                         uuid=UUID)
         self.fake_instance['created_at'] =\
             datetime.datetime(2013, 1, 1, 1, 1, 1)
         self.fake_instance['launched_at'] =\
@@ -45,10 +49,16 @@ class SchedulerHintsTestCase(test.TestCase):
         self.user_ctxt = context.RequestContext(
             fake.USER_ID, fake.PROJECT_ID, auth_token=True)
         self.app = fakes.wsgi_app(fake_auth_context=self.user_ctxt)
+        self.admin_ctxt = context.get_admin_context()
+        cinder.db.volume_type_create(self.admin_ctxt,
+                                     v2_fakes.fake_default_type_get(
+                                         fake.VOLUME_TYPE2_ID))
+        self.vol_type = cinder.db.volume_type_get_by_name(self.admin_ctxt,
+                                                          'vol_type_name')
 
     def test_create_server_without_hints(self):
 
-        @wsgi.response(202)
+        @wsgi.response(http_client.ACCEPTED)
         def fake_create(*args, **kwargs):
             self.assertNotIn('scheduler_hints', kwargs['body'])
             return self.fake_instance
@@ -64,11 +74,11 @@ class SchedulerHintsTestCase(test.TestCase):
                 'volume_id': fake.VOLUME_ID, }
         req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(self.app)
-        self.assertEqual(202, res.status_int)
+        self.assertEqual(http_client.ACCEPTED, res.status_int)
 
     def test_create_server_with_hints(self):
 
-        @wsgi.response(202)
+        @wsgi.response(http_client.ACCEPTED)
         def fake_create(*args, **kwargs):
             self.assertIn('scheduler_hints', kwargs['body'])
             self.assertEqual({"a": "b"}, kwargs['body']['scheduler_hints'])
@@ -87,7 +97,7 @@ class SchedulerHintsTestCase(test.TestCase):
 
         req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(self.app)
-        self.assertEqual(202, res.status_int)
+        self.assertEqual(http_client.ACCEPTED, res.status_int)
 
     def test_create_server_bad_hints(self):
         req = fakes.HTTPRequest.blank('/v2/%s/volumes' % fake.PROJECT_ID)
@@ -101,4 +111,45 @@ class SchedulerHintsTestCase(test.TestCase):
 
         req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(self.app)
-        self.assertEqual(400, res.status_int)
+        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+
+    @ddt.data({'local_to_instance': UUID},
+              {'local_to_instance': None},
+              {'different_host': [fake.UUID1, fake.UUID2]},
+              {'same_host': UUID},
+              {'same_host': [fake.UUID1, fake.UUID2]},
+              {'fake_key': 'fake_value'},
+              {'query': 'query_testing'},
+              {'query': {}},
+              None)
+    def test_scheduler_hints_with_valid_body(self, value):
+        req = fakes.HTTPRequest.blank('/v2/%s/volumes' % fake.PROJECT_ID)
+        req.method = 'POST'
+        req.content_type = 'application/json'
+        body = {'volume': {'size': 1, 'volume_type': self.vol_type['id']},
+                'OS-SCH-HNT:scheduler_hints': value}
+
+        req.body = jsonutils.dump_as_bytes(body)
+        res = req.get_response(self.app)
+        self.assertEqual(http_client.ACCEPTED, res.status_int)
+
+    @ddt.data({'local_to_instance': 'local_to_instance'},
+              {'different_host': 'different_host'},
+              {'different_host': ['different_host']},
+              {'different_host': [UUID, UUID]},
+              {'same_host': 'same_host'},
+              {'same_host': ['same_host']},
+              {'same_host': [UUID, UUID]},
+              {'query': None},
+              {'scheduler_hints'})
+    def test_scheduler_hints_with_invalid_body(self, value):
+        req = fakes.HTTPRequest.blank('/v2/%s/volumes' % fake.PROJECT_ID)
+        req.method = 'POST'
+        req.content_type = 'application/json'
+        body = {'volume': {'size': 1},
+                'OS-SCH-HNT:scheduler_hints': value}
+
+        req.body = jsonutils.dump_as_bytes(body)
+        res = req.get_response(self.app)
+
+        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
