@@ -12,7 +12,8 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-import mock
+from unittest import mock
+
 from oslo_serialization import jsonutils
 from six.moves import http_client
 import webob
@@ -20,10 +21,10 @@ import webob
 from cinder import context
 from cinder import exception
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
+from cinder.tests.unit import test
 
 
 # This list of fake snapshot is used by our tests.
