@@ -14,14 +14,16 @@
 #    under the License.
 
 from oslo_serialization import jsonutils
+from six.moves import http_client
 import webob
 
 from cinder.api.contrib import volume_encryption_metadata
 from cinder import context
 from cinder import db
-from cinder import test
+from cinder.objects import fields
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
 
 def return_volume_type_encryption_metadata(context, volume_type_id):
@@ -57,10 +59,11 @@ class VolumeEncryptionMetadataTest(test.TestCase):
             'status': status,
             'display_name': display_name,
             'display_description': display_description,
-            'attach_status': 'detached',
+            'attach_status': fields.VolumeAttachStatus.DETACHED,
             'availability_zone': availability_zone,
             'host': host,
             'encryption_key_id': encryption_key_id,
+            'volume_type_id': fake.VOLUME_TYPE_ID
         }
         return db.volume_create(context, volume)['id']
 
@@ -80,7 +83,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
         req = webob.Request.blank('/v2/%s/volumes/%s/encryption' % (
                                   fake.PROJECT_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
         res_dict = jsonutils.loads(res.body)
 
         expected = {
@@ -96,10 +99,10 @@ class VolumeEncryptionMetadataTest(test.TestCase):
         req = webob.Request.blank('/v2/%s/volumes/%s/encryption' % (
                                   fake.WILL_NOT_BE_FOUND_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(400, res.status_code)
+        self.assertEqual(http_client.BAD_REQUEST, res.status_code)
 
         res_dict = jsonutils.loads(res.body)
-        expected = {'badRequest': {'code': 400,
+        expected = {'badRequest': {'code': http_client.BAD_REQUEST,
                                    'message': 'Malformed request url'}}
         self.assertEqual(expected, res_dict)
 
@@ -108,10 +111,10 @@ class VolumeEncryptionMetadataTest(test.TestCase):
         req = webob.Request.blank('/v2/%s/volumes/%s/encryption' % (
                                   fake.PROJECT_ID, bad_volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(404, res.status_code)
+        self.assertEqual(http_client.NOT_FOUND, res.status_code)
 
         res_dict = jsonutils.loads(res.body)
-        expected = {'itemNotFound': {'code': 404,
+        expected = {'itemNotFound': {'code': http_client.NOT_FOUND,
                                      'message': 'Volume %s could not be found.'
                                                 % bad_volume_id}}
         self.assertEqual(expected, res_dict)
@@ -121,7 +124,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'encryption_key_id' % (
                                       fake.PROJECT_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
 
         self.assertEqual(fake.ENCRYPTION_KEY_ID, res.body.decode())
 
@@ -130,7 +133,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'control_location' % (
                                       fake.PROJECT_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
 
         self.assertEqual(b'front-end', res.body)
 
@@ -139,7 +142,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'provider' % (
                                       fake.PROJECT_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
 
         self.assertEqual(b'nova.volume.encryptors.base.VolumeEncryptor',
                          res.body)
@@ -150,10 +153,10 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   (fake.WILL_NOT_BE_FOUND_ID,
                                    self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(400, res.status_code)
+        self.assertEqual(http_client.BAD_REQUEST, res.status_code)
 
         res_dict = jsonutils.loads(res.body)
-        expected = {'badRequest': {'code': 400,
+        expected = {'badRequest': {'code': http_client.BAD_REQUEST,
                                    'message': 'Malformed request url'}}
         self.assertEqual(expected, res_dict)
 
@@ -163,10 +166,10 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'encryption_key_id' % (
                                       fake.PROJECT_ID, bad_volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(404, res.status_code)
+        self.assertEqual(http_client.NOT_FOUND, res.status_code)
 
         res_dict = jsonutils.loads(res.body)
-        expected = {'itemNotFound': {'code': 404,
+        expected = {'itemNotFound': {'code': http_client.NOT_FOUND,
                                      'message': 'Volume %s could not be found.'
                                                 % bad_volume_id}}
         self.assertEqual(expected, res_dict)
@@ -179,7 +182,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'encryption_key_id' % (
                                       fake.PROJECT_ID, self.volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
 
         self.assertEqual(fake.ENCRYPTION_KEY_ID, res.body.decode())
 
@@ -194,7 +197,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
                                   'encryption_key_id' % (
                                       fake.PROJECT_ID, volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
         self.assertEqual(0, len(res.body))
 
     def test_index_volume_not_encrypted_type(self):
@@ -208,7 +211,7 @@ class VolumeEncryptionMetadataTest(test.TestCase):
             fake.PROJECT_ID, volume_id))
         res = req.get_response(fakes.wsgi_app(fake_auth_context=self.ctxt))
 
-        self.assertEqual(200, res.status_code)
+        self.assertEqual(http_client.OK, res.status_code)
         res_dict = jsonutils.loads(res.body)
 
         expected = {
