@@ -13,8 +13,9 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from oslo_config import cfg
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
@@ -28,10 +29,10 @@ from cinder.api.openstack import api_version_request as api_version
 from cinder import context
 from cinder import exception
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 
 CONF = cfg.CONF
 
@@ -69,6 +70,7 @@ def service_get(context, service_id, backend_match_level=None, host=None,
                 'uuid': '4200b32b-0bf9-436c-86b2-0675f6ac218e'}
     raise exception.ServiceNotFound(service_id=host)
 
+
 # Some of the tests check that volume types are correctly validated during a
 # volume manage operation.  This data structure represents an existing volume
 # type.
