@@ -12,18 +12,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import uuid
-
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 import webob
 
 from cinder import context
 from cinder import objects
-from cinder import test
+from cinder.objects import fields
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder import volume
 
 
@@ -35,7 +34,7 @@ def fake_db_volume_get(*args, **kwargs):
         'size': 5,
         'availability_zone': 'somewhere',
         'created_at': timeutils.utcnow(),
-        'attach_status': 'detached',
+        'attach_status': fields.VolumeAttachStatus.DETACHED,
         'display_name': 'anothervolume',
         'display_description': 'Just another volume!',
         'volume_type_id': None,
@@ -70,7 +69,7 @@ class VolumeMigStatusAttributeTest(test.TestCase):
         super(VolumeMigStatusAttributeTest, self).setUp()
         self.mock_object(volume.api.API, 'get', fake_volume_api_get)
         self.mock_object(volume.api.API, 'get_all', fake_volume_get_all)
-        self.UUID = uuid.uuid4()
+        self.UUID = fake.UUID1
 
     def test_get_volume_allowed(self):
         ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, True)
