@@ -13,9 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from six.moves import http_client
@@ -27,11 +26,11 @@ from cinder.api.v2 import volumes
 from cinder import db
 from cinder import exception
 from cinder import objects
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder import volume
 from cinder.volume import api as volume_api
 
@@ -100,7 +99,9 @@ def fake_max_volume_metadata():
 
 def get_volume(*args, **kwargs):
     vol = {'name': 'fake',
-           'metadata': {}}
+           'metadata': {},
+           'project_id': fake.PROJECT_ID
+           }
     return fake_volume.fake_volume_obj(args[0], **vol)
 
 
@@ -126,7 +127,7 @@ class VolumeMetaDataTest(test.TestCase):
         self.ext_mgr.extensions = {}
         self.volume_controller = volumes.VolumeController(self.ext_mgr)
         self.controller = volume_metadata.Controller()
-        self.req_id = str(uuid.uuid4())
+        self.req_id = fake.REQUEST_ID
         self.url = '/v2/%s/volumes/%s/metadata' % (
             fake.PROJECT_ID, self.req_id)
 
@@ -134,9 +135,10 @@ class VolumeMetaDataTest(test.TestCase):
                "display_name": "Volume Test Name",
                "display_description": "Volume Test Desc",
                "availability_zone": "zone1:host1",
-               "metadata": {}}
+               "metadata": {},
+               "volume_type": self.vt['id']}
         body = {"volume": vol}
-        req = fakes.HTTPRequest.blank('/v2/volumes')
+        req = fakes.HTTPRequest.blank('/v2/%s/volumes' % fake.PROJECT_ID)
         self.volume_controller.create(req, body=body)
 
     def test_index(self):
