@@ -12,8 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from oslo_utils import timeutils
 import pytz
 import six
@@ -21,6 +22,8 @@ import six
 from cinder import context
 from cinder import exception
 from cinder import objects
+from cinder.objects import base as ovo_base
+from cinder.objects import fields
 from cinder.tests.unit.consistencygroup import fake_consistencygroup
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
@@ -61,15 +64,28 @@ class TestVolume(test_objects.BaseObjectsTestCase):
         self.assertEqual(db_volume['id'], volume.id)
 
     @mock.patch('cinder.db.volume_update')
-    def test_save(self, volume_update):
+    @ddt.data(False, True)
+    def test_save(self, test_cg, volume_update):
         db_volume = fake_volume.fake_db_volume()
         volume = objects.Volume._from_db_object(self.context,
                                                 objects.Volume(), db_volume)
         volume.display_name = 'foobar'
+        if test_cg:
+            volume.consistencygroup = None
         volume.save()
         volume_update.assert_called_once_with(self.context, volume.id,
                                               {'display_name': 'foobar'})
 
+    def test_save_error(self):
+        db_volume = fake_volume.fake_db_volume()
+        volume = objects.Volume._from_db_object(self.context,
+                                                objects.Volume(), db_volume)
+        volume.display_name = 'foobar'
+        volume.consistencygroup = (
+            fake_consistencygroup.fake_consistencyobject_obj(self.context))
+        self.assertRaises(exception.ObjectActionError,
+                          volume.save)
+
     @mock.patch('cinder.db.volume_metadata_update',
                 return_value={'key1': 'value1'})
     @mock.patch('cinder.db.volume_update')
@@ -191,8 +207,10 @@ class TestVolume(test_objects.BaseObjectsTestCase):
                            mock_va_get_all_by_vol, mock_vt_get_by_id,
                            mock_admin_metadata_get, mock_glance_metadata_get,
                            mock_metadata_get):
+        fake_db_volume = fake_volume.fake_db_volume(
+            consistencygroup_id=fake.CONSISTENCY_GROUP_ID)
         volume = objects.Volume._from_db_object(
-            self.context, objects.Volume(), fake_volume.fake_db_volume())
+            self.context, objects.Volume(), fake_db_volume)
 
         # Test metadata lazy-loaded field
         metadata = {'foo': 'bar'}
@@ -263,6 +281,24 @@ class TestVolume(test_objects.BaseObjectsTestCase):
         self.assertEqual(adm_metadata, volume.admin_metadata)
         mock_admin_metadata_get.assert_called_once_with(adm_context, volume.id)
 
+    @mock.patch('cinder.objects.consistencygroup.ConsistencyGroup.get_by_id')
+    def test_obj_load_attr_cgroup_not_exist(self, mock_cg_get_by_id):
+        fake_db_volume = fake_volume.fake_db_volume(consistencygroup_id=None)
+        volume = objects.Volume._from_db_object(
+            self.context, objects.Volume(), fake_db_volume)
+
+        self.assertIsNone(volume.consistencygroup)
+        mock_cg_get_by_id.assert_not_called()
+
+    @mock.patch('cinder.objects.group.Group.get_by_id')
+    def test_obj_load_attr_group_not_exist(self, mock_group_get_by_id):
+        fake_db_volume = fake_volume.fake_db_volume(group_id=None)
+        volume = objects.Volume._from_db_object(
+            self.context, objects.Volume(), fake_db_volume)
+
+        self.assertIsNone(volume.group)
+        mock_group_get_by_id.assert_not_called()
+
     def test_from_db_object_with_all_expected_attributes(self):
         expected_attrs = ['metadata', 'admin_metadata', 'glance_metadata',
                           'volume_type', 'volume_attachment',
@@ -272,7 +308,7 @@ class TestVolume(test_objects.BaseObjectsTestCase):
         db_admin_metadata = [{'key': 'admin_foo', 'value': 'admin_bar'}]
         db_glance_metadata = [{'key': 'glance_foo', 'value': 'glance_bar'}]
         db_volume_type = fake_volume.fake_db_volume_type()
-        db_volume_attachments = fake_volume.fake_db_volume_attachment()
+        db_volume_attachments = fake_volume.volume_attachment_db_obj()
         db_consistencygroup = fake_consistencygroup.fake_db_consistencygroup()
         db_snapshots = fake_snapshot.fake_db_snapshot()
 
@@ -411,7 +447,144 @@ class TestVolume(test_objects.BaseObjectsTestCase):
         volume = serializer.deserialize_entity(self.context, serialized_volume)
         self.assertDictEqual({}, volume.obj_get_changes())
 
+    @mock.patch('cinder.db.volume_admin_metadata_update')
+    @mock.patch('cinder.db.sqlalchemy.api.volume_attach')
+    def test_begin_attach(self, volume_attach, metadata_update):
+        volume = fake_volume.fake_volume_obj(self.context)
+        db_attachment = fake_volume.volume_attachment_db_obj(
+            volume_id=volume.id,
+            attach_status=fields.VolumeAttachStatus.ATTACHING)
+        volume_attach.return_value = db_attachment
+        metadata_update.return_value = {'attached_mode': 'rw'}
 
+        with mock.patch.object(self.context, 'elevated') as mock_elevated:
+            mock_elevated.return_value = context.get_admin_context()
+            attachment = volume.begin_attach("rw")
+            self.assertIsInstance(attachment, objects.VolumeAttachment)
+            self.assertEqual(volume.id, attachment.volume_id)
+            self.assertEqual(fields.VolumeAttachStatus.ATTACHING,
+                             attachment.attach_status)
+            metadata_update.assert_called_once_with(self.context.elevated(),
+                                                    volume.id,
+                                                    {'attached_mode': u'rw'},
+                                                    True)
+            self.assertEqual('rw', volume.admin_metadata['attached_mode'])
+
+    @mock.patch('cinder.db.volume_admin_metadata_delete')
+    @mock.patch('cinder.db.sqlalchemy.api.volume_detached')
+    @mock.patch('cinder.objects.volume_attachment.VolumeAttachmentList.'
+                'get_all_by_volume_id')
+    def test_volume_detached_with_attachment(
+            self, volume_attachment_get,
+            volume_detached,
+            metadata_delete):
+        va_objs = [objects.VolumeAttachment(context=self.context, id=i)
+                   for i in [fake.OBJECT_ID, fake.OBJECT2_ID, fake.OBJECT3_ID]]
+        # As changes are not saved, we need reset it here. Later changes
+        # will be checked.
+        for obj in va_objs:
+            obj.obj_reset_changes()
+        va_list = objects.VolumeAttachmentList(context=self.context,
+                                               objects=va_objs)
+        va_list.obj_reset_changes()
+        volume_attachment_get.return_value = va_list
+        admin_context = context.get_admin_context()
+        volume = fake_volume.fake_volume_obj(
+            admin_context,
+            volume_attachment=va_list,
+            volume_admin_metadata=[{'key': 'attached_mode',
+                                    'value': 'rw'}])
+        self.assertEqual(3, len(volume.volume_attachment))
+        volume_detached.return_value = ({'status': 'in-use'},
+                                        {'attached_mode': 'rw'})
+        with mock.patch.object(admin_context, 'elevated') as mock_elevated:
+            mock_elevated.return_value = admin_context
+            volume.finish_detach(fake.OBJECT_ID)
+            volume_detached.assert_called_once_with(admin_context,
+                                                    volume.id,
+                                                    fake.OBJECT_ID)
+            metadata_delete.assert_called_once_with(admin_context,
+                                                    volume.id,
+                                                    'attached_mode')
+            self.assertEqual('in-use', volume.status)
+            self.assertEqual({}, volume.cinder_obj_get_changes())
+            self.assertEqual(2, len(volume.volume_attachment))
+            self.assertNotIn('attached_mode', volume.admin_metadata)
+
+    @mock.patch('cinder.db.volume_admin_metadata_delete')
+    @mock.patch('cinder.db.sqlalchemy.api.volume_detached')
+    @mock.patch('cinder.objects.volume_attachment.VolumeAttachmentList.'
+                'get_all_by_volume_id')
+    def test_volume_detached_without_attachment(
+            self, volume_attachment_get, volume_detached, metadata_delete):
+        admin_context = context.get_admin_context()
+        volume = fake_volume.fake_volume_obj(
+            admin_context,
+            volume_admin_metadata=[{'key': 'attached_mode',
+                                    'value': 'rw'}])
+        self.assertFalse(volume.obj_attr_is_set('volume_attachment'))
+        volume_detached.return_value = ({'status': 'in-use'}, None)
+        with mock.patch.object(admin_context, 'elevated') as mock_elevated:
+            mock_elevated.return_value = admin_context
+            volume.finish_detach(fake.OBJECT_ID)
+            metadata_delete.assert_called_once_with(admin_context,
+                                                    volume.id,
+                                                    'attached_mode')
+            volume_detached.assert_called_once_with(admin_context,
+                                                    volume.id,
+                                                    fake.OBJECT_ID)
+            self.assertEqual('in-use', volume.status)
+            self.assertEqual({}, volume.cinder_obj_get_changes())
+            self.assertFalse(volume_attachment_get.called)
+
+    @ddt.data('1.6', '1.7')
+    def test_obj_make_compatible_cluster_added(self, version):
+        extra_data = {'cluster_name': 'cluster_name',
+                      'cluster': objects.Cluster()}
+        volume = objects.Volume(self.context, host='host', **extra_data)
+
+        serializer = ovo_base.CinderObjectSerializer(version)
+        primitive = serializer.serialize_entity(self.context, volume)
+
+        converted_volume = objects.Volume.obj_from_primitive(primitive)
+        is_set = version == '1.7'
+        for key in extra_data:
+            self.assertEqual(is_set, converted_volume.obj_attr_is_set(key))
+        self.assertEqual('host', converted_volume.host)
+
+    @ddt.data('1.9', '1.10')
+    def test_obj_make_compatible_groups_added(self, version):
+        extra_data = {'group_id': fake.GROUP_ID,
+                      'group': objects.Group()}
+        volume = objects.Volume(self.context, host='host', **extra_data)
+
+        serializer = ovo_base.CinderObjectSerializer(version)
+        primitive = serializer.serialize_entity(self.context, volume)
+
+        converted_volume = objects.Volume.obj_from_primitive(primitive)
+        is_set = version == '1.10'
+        for key in extra_data:
+            self.assertEqual(is_set, converted_volume.obj_attr_is_set(key))
+        self.assertEqual('host', converted_volume.host)
+
+    @ddt.data(True, False)
+    def test_is_replicated(self, result):
+        volume_type = fake_volume.fake_volume_type_obj(self.context)
+        volume = fake_volume.fake_volume_obj(
+            self.context, volume_type_id=volume_type.id)
+        volume.volume_type = volume_type
+        with mock.patch.object(volume_type, 'is_replicated',
+                               return_value=result) as is_replicated:
+            self.assertEqual(result, volume.is_replicated())
+            is_replicated.assert_called_once_with()
+
+    def test_is_replicated_no_type(self):
+        volume = fake_volume.fake_volume_obj(
+            self.context, volume_type_id=None, volume_type=None)
+        self.assertFalse(bool(volume.is_replicated()))
+
+
+@ddt.ddt
 class TestVolumeList(test_objects.BaseObjectsTestCase):
     @mock.patch('cinder.db.volume_get_all')
     def test_get_all(self, volume_get_all):
@@ -458,6 +631,16 @@ class TestVolumeList(test_objects.BaseObjectsTestCase):
         self.assertEqual(1, len(volumes))
         TestVolume._compare(self, db_volume, volumes[0])
 
+    @ddt.data(['name_id'], ['__contains__'])
+    def test_get_by_project_with_sort_key(self, sort_keys):
+        fake_volume.fake_db_volume()
+
+        self.assertRaises(exception.InvalidInput,
+                          objects.VolumeList.get_all_by_project,
+                          self.context,
+                          self.context.project_id,
+                          sort_keys=sort_keys)
+
     @mock.patch('cinder.db.volume_include_in_cluster')
     def test_include_in_cluster(self, include_mock):
         filters = {'host': mock.sentinel.host,
