@@ -13,11 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 
 from cinder import context
-from cinder import test
+from cinder.tests.unit import test
 
 
 @ddt.ddt
