@@ -1,5 +1,3 @@
-# Copyright (c) 2013 eNovance , Inc.
-# All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,18 +14,18 @@
 
 import errno
 import math
+from unittest import mock
 
 import cryptography
 import ddt
-import mock
 from oslo_concurrency import processutils
 from oslo_utils import units
 from six.moves import builtins
 
 from cinder import exception
 from cinder.image import image_utils
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder.volume import throttling
 
 
@@ -124,6 +122,7 @@ class TestQemuImgInfo(test.TestCase):
             current_version=[1, 8])
 
 
+@ddt.ddt
 class TestConvertImage(test.TestCase):
     @mock.patch('cinder.image.image_utils.qemu_img_info')
     @mock.patch('cinder.utils.execute')
@@ -136,7 +135,7 @@ class TestConvertImage(test.TestCase):
         mock_info.return_value.virtual_size = 1048576
         throttle = throttling.Throttle(prefix=['cgcmd'])
 
-        with mock.patch('cinder.volume.utils.check_for_odirect_support',
+        with mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                         return_value=True):
             output = image_utils.convert_image(source, dest, out_format,
                                                throttle=throttle)
@@ -148,7 +147,7 @@ class TestConvertImage(test.TestCase):
 
         mock_exec.reset_mock()
 
-        with mock.patch('cinder.volume.utils.check_for_odirect_support',
+        with mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                         return_value=False):
             output = image_utils.convert_image(source, dest, out_format)
 
@@ -169,7 +168,7 @@ class TestConvertImage(test.TestCase):
         mock_info.side_effect = ValueError
         throttle = throttling.Throttle(prefix=['cgcmd'])
 
-        with mock.patch('cinder.volume.utils.check_for_odirect_support',
+        with mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                         return_value=True):
             output = image_utils.convert_image(source, dest, out_format,
                                                throttle=throttle)
@@ -182,7 +181,7 @@ class TestConvertImage(test.TestCase):
 
         mock_exec.reset_mock()
 
-        with mock.patch('cinder.volume.utils.check_for_odirect_support',
+        with mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                         return_value=False):
             output = image_utils.convert_image(source, dest, out_format)
 
@@ -191,7 +190,7 @@ class TestConvertImage(test.TestCase):
                                               '-O', out_format, source, dest,
                                               run_as_root=True)
 
-    @mock.patch('cinder.volume.utils.check_for_odirect_support',
+    @mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                 return_value=True)
     @mock.patch('cinder.image.image_utils.qemu_img_info')
     @mock.patch('cinder.utils.execute')
@@ -216,7 +215,7 @@ class TestConvertImage(test.TestCase):
                                           source, dest,
                                           run_as_root=True)
 
-    @mock.patch('cinder.volume.utils.check_for_odirect_support',
+    @mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                 return_value=True)
     @mock.patch('cinder.image.image_utils.qemu_img_info')
     @mock.patch('cinder.utils.execute')
@@ -252,7 +251,7 @@ class TestConvertImage(test.TestCase):
         out_format = mock.sentinel.out_format
         mock_info.return_value.virtual_size = 1048576
 
-        with mock.patch('cinder.volume.utils.check_for_odirect_support',
+        with mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
                         return_value=True):
             output = image_utils.convert_image(source, dest, out_format,
                                                src_format='AMI')
@@ -265,7 +264,7 @@ class TestConvertImage(test.TestCase):
     @mock.patch('cinder.image.image_utils.qemu_img_info')
     @mock.patch('cinder.utils.execute')
     @mock.patch('cinder.utils.is_blk_device', return_value=False)
-    @mock.patch('cinder.volume.utils.check_for_odirect_support')
+    @mock.patch('cinder.volume.volume_utils.check_for_odirect_support')
     def test_convert_to_vhd(self, mock_check_odirect, mock_isblk,
                             mock_exec, mock_info):
         source = mock.sentinel.source
@@ -280,6 +279,66 @@ class TestConvertImage(test.TestCase):
                                           '-O', 'vpc',
                                           source, dest, run_as_root=True)
 
+    @ddt.data(True, False)
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.utils.execute')
+    @mock.patch('cinder.utils.is_blk_device', return_value=False)
+    def test_convert_to_qcow2(self,
+                              compress_option,
+                              mock_isblk, mock_exec, mock_info):
+        self.override_config('image_compress_on_upload', compress_option)
+        source = mock.sentinel.source
+        dest = mock.sentinel.dest
+        out_format = 'qcow2'
+        mock_info.return_value.virtual_size = 1048576
+
+        image_utils.convert_image(source,
+                                  dest,
+                                  out_format,
+                                  compress=True)
+
+        exec_args = ['qemu-img', 'convert', '-O', 'qcow2']
+        if compress_option:
+            exec_args.append('-c')
+        exec_args.extend((source, dest))
+        mock_exec.assert_called_once_with(*exec_args,
+                                          run_as_root=True)
+
+    @mock.patch('cinder.image.image_utils.CONF')
+    @mock.patch('cinder.volume.volume_utils.check_for_odirect_support',
+                return_value=True)
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.utils.execute')
+    @mock.patch('cinder.utils.is_blk_device', return_value=False)
+    @mock.patch('os.path.dirname', return_value='fakedir')
+    @mock.patch('os.path.ismount', return_value=True)
+    @mock.patch('oslo_utils.fileutils.ensure_tree')
+    @mock.patch('cinder.image.image_utils.utils.tempdir')
+    @mock.patch.object(image_utils.LOG, 'error')
+    def test_not_enough_conversion_space(self,
+                                         mock_log,
+                                         mock_tempdir,
+                                         mock_make,
+                                         mock_ismount,
+                                         mock_dirname,
+                                         mock_isblk,
+                                         mock_exec,
+                                         mock_info,
+                                         mock_odirect,
+                                         mock_conf):
+        source = mock.sentinel.source
+        mock_conf.image_conversion_dir = 'fakedir'
+        dest = [mock_conf.image_conversion_dir]
+        out_format = mock.sentinel.out_format
+        mock_info.side_effect = ValueError
+        mock_exec.side_effect = processutils.ProcessExecutionError(
+            stderr='No space left on device')
+        self.assertRaises(processutils.ProcessExecutionError,
+                          image_utils.convert_image,
+                          source, dest, out_format)
+        mock_log.assert_called_with('Insufficient free space on fakedir for'
+                                    ' image conversion.')
+
 
 class TestResizeImage(test.TestCase):
     @mock.patch('cinder.utils.execute')
@@ -302,9 +361,10 @@ class TestResizeImage(test.TestCase):
 
 
 class TestFetch(test.TestCase):
+    @mock.patch('eventlet.tpool.Proxy')
     @mock.patch('os.stat')
     @mock.patch('cinder.image.image_utils.fileutils')
-    def test_defaults(self, mock_fileutils, mock_stat):
+    def test_defaults(self, mock_fileutils, mock_stat, mock_proxy):
         ctxt = mock.sentinel.context
         image_service = mock.Mock()
         image_id = mock.sentinel.image_id
@@ -319,8 +379,9 @@ class TestFetch(test.TestCase):
             output = image_utils.fetch(ctxt, image_service, image_id, path,
                                        _user_id, _project_id)
         self.assertIsNone(output)
+        mock_proxy.assert_called_once_with(mock_open.return_value)
         image_service.download.assert_called_once_with(ctxt, image_id,
-                                                       mock_open.return_value)
+                                                       mock_proxy.return_value)
         mock_open.assert_called_once_with(path, 'wb')
         mock_fileutils.remove_path_on_error.assert_called_once_with(path)
         (mock_fileutils.remove_path_on_error.return_value.__enter__
@@ -346,6 +407,26 @@ class TestFetch(test.TestCase):
                               context, image_service, image_id, path,
                               _user_id, _project_id)
 
+    def test_fetch_ioerror(self):
+        context = mock.sentinel.context
+        image_service = mock.Mock()
+        image_id = mock.sentinel.image_id
+        e = IOError()
+        e.errno = errno.ECONNRESET
+        e.strerror = 'Some descriptive message'
+        image_service.download.side_effect = e
+        path = '/test_path'
+        _user_id = mock.sentinel._user_id
+        _project_id = mock.sentinel._project_id
+
+        with mock.patch('cinder.image.image_utils.open',
+                        new=mock.mock_open(), create=True):
+            self.assertRaisesRegex(exception.ImageDownloadFailed,
+                                   e.strerror,
+                                   image_utils.fetch,
+                                   context, image_service, image_id, path,
+                                   _user_id, _project_id)
+
 
 class MockVerifier(object):
     def update(self, data):
@@ -441,10 +522,12 @@ class TestVerifyImageSignature(test.TestCase):
                           FakeImageService(), 'fake_id', 'fake_path')
         mock_get.assert_not_called()
 
+    @mock.patch('six.moves.builtins.open')
+    @mock.patch('eventlet.tpool.execute')
     @mock.patch('cursive.signature_utils.get_verifier')
     @mock.patch('oslo_utils.fileutils.remove_path_on_error')
-    def test_image_signature_verify_success(self, mock_remove, mock_get):
-        self.mock_object(builtins, 'open', mock.mock_open())
+    def test_image_signature_verify_success(self, mock_remove, mock_get,
+                                            mock_exec, mock_open):
         ctxt = mock.sentinel.context
         metadata = {'name': 'test image',
                     'is_public': False,
@@ -465,6 +548,11 @@ class TestVerifyImageSignature(test.TestCase):
         result = image_utils.verify_glance_image_signature(
             ctxt, FakeImageService(), 'fake_id', 'fake_path')
         self.assertTrue(result)
+        mock_exec.assert_called_once_with(
+            image_utils._verify_image,
+            mock_open.return_value.__enter__.return_value,
+            mock_get.return_value)
+
         mock_get.assert_called_once_with(
             context=ctxt,
             img_signature_certificate_uuid='fake_uuid',
@@ -633,8 +721,11 @@ class TestTemporaryDir(test.TestCase):
 
 @ddt.ddt
 class TestUploadVolume(test.TestCase):
-    @ddt.data((mock.sentinel.disk_format, mock.sentinel.disk_format),
-              ('ploop', 'parallels'))
+    @ddt.data((mock.sentinel.disk_format, mock.sentinel.disk_format, True),
+              (mock.sentinel.disk_format, mock.sentinel.disk_format, False),
+              ('ploop', 'parallels', True),
+              ('ploop', 'parallels', False))
+    @mock.patch('eventlet.tpool.Proxy')
     @mock.patch('cinder.image.image_utils.CONF')
     @mock.patch('six.moves.builtins.open')
     @mock.patch('cinder.image.image_utils.qemu_img_info')
@@ -642,12 +733,13 @@ class TestUploadVolume(test.TestCase):
     @mock.patch('cinder.image.image_utils.temporary_file')
     @mock.patch('cinder.image.image_utils.os')
     def test_diff_format(self, image_format, mock_os, mock_temp, mock_convert,
-                         mock_info, mock_open, mock_conf):
-        input_format, output_format = image_format
+                         mock_info, mock_open, mock_conf, mock_proxy):
+        input_format, output_format, do_compress = image_format
         ctxt = mock.sentinel.context
         image_service = mock.Mock()
         image_meta = {'id': 'test_id',
-                      'disk_format': input_format}
+                      'disk_format': input_format,
+                      'container_format': mock.sentinel.container_format}
         volume_path = mock.sentinel.volume_path
         mock_os.name = 'posix'
         data = mock_info.return_value
@@ -656,20 +748,24 @@ class TestUploadVolume(test.TestCase):
         temp_file = mock_temp.return_value.__enter__.return_value
 
         output = image_utils.upload_volume(ctxt, image_service, image_meta,
-                                           volume_path)
+                                           volume_path, compress=do_compress)
 
         self.assertIsNone(output)
         mock_convert.assert_called_once_with(volume_path,
                                              temp_file,
                                              output_format,
-                                             run_as_root=True)
+                                             run_as_root=True,
+                                             compress=do_compress)
         mock_info.assert_called_with(temp_file, run_as_root=True)
         self.assertEqual(2, mock_info.call_count)
         mock_open.assert_called_once_with(temp_file, 'rb')
-        image_service.update.assert_called_once_with(
-            ctxt, image_meta['id'], {},
+        mock_proxy.assert_called_once_with(
             mock_open.return_value.__enter__.return_value)
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref=None)
 
+    @mock.patch('eventlet.tpool.Proxy')
     @mock.patch('cinder.image.image_utils.utils.temporary_chown')
     @mock.patch('cinder.image.image_utils.CONF')
     @mock.patch('six.moves.builtins.open')
@@ -678,11 +774,12 @@ class TestUploadVolume(test.TestCase):
     @mock.patch('cinder.image.image_utils.temporary_file')
     @mock.patch('cinder.image.image_utils.os')
     def test_same_format(self, mock_os, mock_temp, mock_convert, mock_info,
-                         mock_open, mock_conf, mock_chown):
+                         mock_open, mock_conf, mock_chown, mock_proxy):
         ctxt = mock.sentinel.context
         image_service = mock.Mock()
         image_meta = {'id': 'test_id',
-                      'disk_format': 'raw'}
+                      'disk_format': 'raw',
+                      'container_format': mock.sentinel.container_format}
         volume_path = mock.sentinel.volume_path
         mock_os.name = 'posix'
         mock_os.access.return_value = False
@@ -695,10 +792,70 @@ class TestUploadVolume(test.TestCase):
         self.assertFalse(mock_info.called)
         mock_chown.assert_called_once_with(volume_path)
         mock_open.assert_called_once_with(volume_path, 'rb')
-        image_service.update.assert_called_once_with(
-            ctxt, image_meta['id'], {},
+        mock_proxy.assert_called_once_with(
             mock_open.return_value.__enter__.return_value)
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref=None)
 
+    @mock.patch('cinder.image.accelerator.ImageAccel._get_engine')
+    @mock.patch('cinder.image.accelerator.ImageAccel.is_engine_ready',
+                return_value = True)
+    @mock.patch('eventlet.tpool.Proxy')
+    @mock.patch('cinder.image.image_utils.utils.temporary_chown')
+    @mock.patch('cinder.image.image_utils.CONF')
+    @mock.patch('six.moves.builtins.open')
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.image.image_utils.convert_image')
+    @mock.patch('cinder.image.image_utils.temporary_file')
+    @mock.patch('cinder.image.image_utils.os')
+    def test_same_format_compressed(self, mock_os, mock_temp, mock_convert,
+                                    mock_info, mock_open, mock_conf,
+                                    mock_chown, mock_proxy,
+                                    mock_engine_ready, mock_get_engine):
+        class fakeEngine(object):
+
+            def __init__(self):
+                pass
+
+            def compress_img(self, src, dest, run_as_root):
+                pass
+
+        ctxt = mock.sentinel.context
+        image_service = mock.Mock()
+        image_meta = {'id': 'test_id',
+                      'disk_format': 'raw',
+                      'container_format': 'compressed'}
+        mock_conf.allow_compression_on_image_upload = True
+        volume_path = mock.sentinel.volume_path
+        mock_os.name = 'posix'
+        data = mock_info.return_value
+        data.file_format = 'raw'
+        data.backing_file = None
+        temp_file = mock_temp.return_value.__enter__.return_value
+        mock_engine = mock.Mock(spec=fakeEngine)
+        mock_get_engine.return_value = mock_engine
+
+        output = image_utils.upload_volume(ctxt, image_service, image_meta,
+                                           volume_path)
+
+        self.assertIsNone(output)
+        mock_convert.assert_called_once_with(volume_path,
+                                             temp_file,
+                                             'raw',
+                                             compress=True,
+                                             run_as_root=True)
+        mock_info.assert_called_with(temp_file, run_as_root=True)
+        self.assertEqual(2, mock_info.call_count)
+        mock_open.assert_called_once_with(temp_file, 'rb')
+        mock_proxy.assert_called_once_with(
+            mock_open.return_value.__enter__.return_value)
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref=None)
+        mock_engine.compress_img.assert_called()
+
+    @mock.patch('eventlet.tpool.Proxy')
     @mock.patch('cinder.image.image_utils.utils.temporary_chown')
     @mock.patch('cinder.image.image_utils.CONF')
     @mock.patch('six.moves.builtins.open')
@@ -707,11 +864,13 @@ class TestUploadVolume(test.TestCase):
     @mock.patch('cinder.image.image_utils.temporary_file')
     @mock.patch('cinder.image.image_utils.os')
     def test_same_format_on_nt(self, mock_os, mock_temp, mock_convert,
-                               mock_info, mock_open, mock_conf, mock_chown):
+                               mock_info, mock_open, mock_conf, mock_chown,
+                               mock_proxy):
         ctxt = mock.sentinel.context
         image_service = mock.Mock()
         image_meta = {'id': 'test_id',
-                      'disk_format': 'raw'}
+                      'disk_format': 'raw',
+                      'container_format': 'bare'}
         volume_path = mock.sentinel.volume_path
         mock_os.name = 'nt'
         mock_os.access.return_value = False
@@ -723,9 +882,69 @@ class TestUploadVolume(test.TestCase):
         self.assertFalse(mock_convert.called)
         self.assertFalse(mock_info.called)
         mock_open.assert_called_once_with(volume_path, 'rb')
-        image_service.update.assert_called_once_with(
-            ctxt, image_meta['id'], {},
+        mock_proxy.assert_called_once_with(
             mock_open.return_value.__enter__.return_value)
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref=None)
+
+    @mock.patch('cinder.image.accelerator.ImageAccel._get_engine')
+    @mock.patch('cinder.image.accelerator.ImageAccel.is_engine_ready',
+                return_value = True)
+    @mock.patch('eventlet.tpool.Proxy')
+    @mock.patch('cinder.image.image_utils.utils.temporary_chown')
+    @mock.patch('cinder.image.image_utils.CONF')
+    @mock.patch('six.moves.builtins.open')
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.image.image_utils.convert_image')
+    @mock.patch('cinder.image.image_utils.temporary_file')
+    @mock.patch('cinder.image.image_utils.os')
+    def test_same_format_on_nt_compressed(self, mock_os, mock_temp,
+                                          mock_convert, mock_info,
+                                          mock_open, mock_conf,
+                                          mock_chown, mock_proxy,
+                                          mock_engine_ready, mock_get_engine):
+        class fakeEngine(object):
+
+            def __init__(self):
+                pass
+
+            def compress_img(self, src, dest, run_as_root):
+                pass
+
+        ctxt = mock.sentinel.context
+        image_service = mock.Mock()
+        image_meta = {'id': 'test_id',
+                      'disk_format': 'raw',
+                      'container_format': 'compressed'}
+        mock_conf.allow_compression_on_image_upload = True
+        volume_path = mock.sentinel.volume_path
+        mock_os.name = 'posix'
+        data = mock_info.return_value
+        data.file_format = 'raw'
+        data.backing_file = None
+        temp_file = mock_temp.return_value.__enter__.return_value
+        mock_engine = mock.Mock(spec=fakeEngine)
+        mock_get_engine.return_value = mock_engine
+
+        output = image_utils.upload_volume(ctxt, image_service, image_meta,
+                                           volume_path)
+
+        self.assertIsNone(output)
+        mock_convert.assert_called_once_with(volume_path,
+                                             temp_file,
+                                             'raw',
+                                             compress=True,
+                                             run_as_root=True)
+        mock_info.assert_called_with(temp_file, run_as_root=True)
+        self.assertEqual(2, mock_info.call_count)
+        mock_open.assert_called_once_with(temp_file, 'rb')
+        mock_proxy.assert_called_once_with(
+            mock_open.return_value.__enter__.return_value)
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref=None)
+        mock_engine.compress_img.assert_called()
 
     @mock.patch('cinder.image.image_utils.CONF')
     @mock.patch('six.moves.builtins.open')
@@ -738,7 +957,8 @@ class TestUploadVolume(test.TestCase):
         ctxt = mock.sentinel.context
         image_service = mock.Mock()
         image_meta = {'id': 'test_id',
-                      'disk_format': mock.sentinel.disk_format}
+                      'disk_format': mock.sentinel.disk_format,
+                      'container_format': mock.sentinel.container_format}
         volume_path = mock.sentinel.volume_path
         mock_os.name = 'posix'
         data = mock_info.return_value
@@ -752,11 +972,38 @@ class TestUploadVolume(test.TestCase):
         mock_convert.assert_called_once_with(volume_path,
                                              temp_file,
                                              mock.sentinel.disk_format,
-                                             run_as_root=True)
+                                             run_as_root=True,
+                                             compress=True)
         mock_info.assert_called_with(temp_file, run_as_root=True)
         self.assertEqual(2, mock_info.call_count)
         self.assertFalse(image_service.update.called)
 
+    @mock.patch('eventlet.tpool.Proxy')
+    @mock.patch('cinder.image.image_utils.utils.temporary_chown')
+    @mock.patch('cinder.image.image_utils.CONF')
+    @mock.patch('six.moves.builtins.open')
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.image.image_utils.convert_image')
+    @mock.patch('cinder.image.image_utils.temporary_file')
+    @mock.patch('cinder.image.image_utils.os')
+    def test_base_image_ref(self, mock_os, mock_temp, mock_convert, mock_info,
+                            mock_open, mock_conf, mock_chown, mock_proxy):
+        ctxt = mock.sentinel.context
+        image_service = mock.Mock()
+        image_meta = {'id': 'test_id',
+                      'disk_format': 'raw',
+                      'container_format': mock.sentinel.container_format}
+        volume_path = mock.sentinel.volume_path
+        mock_os.name = 'posix'
+        mock_os.access.return_value = False
+
+        image_utils.upload_volume(ctxt, image_service, image_meta,
+                                  volume_path, base_image_ref='xyz')
+
+        image_service.update.assert_called_once_with(
+            ctxt, image_meta['id'], {}, mock_proxy.return_value,
+            store_id=None, base_image_ref='xyz')
+
 
 class TestFetchToVhd(test.TestCase):
     @mock.patch('cinder.image.image_utils.fetch_to_volume_format')
@@ -1542,6 +1789,83 @@ class TestFetchToVolumeFormat(test.TestCase):
             image_utils.get_qemu_data, image_id, has_meta, disk_format_raw,
             dest, run_as_root=run_as_root)
 
+    @mock.patch('cinder.image.accelerator.is_gzip_compressed',
+                return_value = True)
+    @mock.patch('cinder.image.accelerator.ImageAccel._get_engine')
+    @mock.patch('cinder.image.accelerator.ImageAccel.is_engine_ready',
+                return_value = True)
+    @mock.patch('cinder.image.image_utils.check_available_space')
+    @mock.patch('cinder.image.image_utils.convert_image')
+    @mock.patch('cinder.image.image_utils.volume_utils.copy_volume')
+    @mock.patch(
+        'cinder.image.image_utils.replace_xenserver_image_with_coalesced_vhd')
+    @mock.patch('cinder.image.image_utils.is_xenserver_format',
+                return_value=False)
+    @mock.patch('cinder.image.image_utils.fetch')
+    @mock.patch('cinder.image.image_utils.qemu_img_info')
+    @mock.patch('cinder.image.image_utils.temporary_file')
+    @mock.patch('cinder.image.image_utils.CONF')
+    def test_defaults_compressed(self, mock_conf, mock_temp, mock_info,
+                                 mock_fetch, mock_is_xen, mock_repl_xen,
+                                 mock_copy, mock_convert, mock_check_space,
+                                 mock_engine_ready, mock_get_engine,
+                                 mock_gzip_compressed):
+        class fakeEngine(object):
+            def __init__(self):
+                pass
+
+            def decompress_img(self, src, dest, run_as_root):
+                pass
+
+        class FakeImageService(object):
+            def __init__(self, db_driver=None,
+                         image_service=None, disk_format='raw'):
+                self.temp_images = None
+                self.disk_format = disk_format
+
+            def show(self, context, image_id):
+                return {'size': 2 * units.Gi,
+                        'disk_format': self.disk_format,
+                        'container_format': 'compressed',
+                        'status': 'active'}
+
+        ctxt = mock.sentinel.context
+        ctxt.user_id = mock.sentinel.user_id
+        image_service = FakeImageService()
+        image_id = mock.sentinel.image_id
+        dest = mock.sentinel.dest
+        volume_format = mock.sentinel.volume_format
+        out_subformat = None
+        blocksize = mock.sentinel.blocksize
+
+        data = mock_info.return_value
+        data.file_format = volume_format
+        data.backing_file = None
+        data.virtual_size = 1234
+        tmp = mock_temp.return_value.__enter__.return_value
+
+        mock_engine = mock.Mock(spec=fakeEngine)
+        mock_get_engine.return_value = mock_engine
+
+        output = image_utils.fetch_to_volume_format(ctxt, image_service,
+                                                    image_id, dest,
+                                                    volume_format, blocksize)
+
+        self.assertIsNone(output)
+        mock_temp.assert_called_once_with()
+        mock_info.assert_has_calls([
+            mock.call(tmp, force_share=False, run_as_root=True),
+            mock.call(tmp, run_as_root=True)])
+        mock_fetch.assert_called_once_with(ctxt, image_service, image_id,
+                                           tmp, None, None)
+        self.assertFalse(mock_repl_xen.called)
+        self.assertFalse(mock_copy.called)
+        mock_convert.assert_called_once_with(tmp, dest, volume_format,
+                                             out_subformat=out_subformat,
+                                             run_as_root=True,
+                                             src_format='raw')
+        mock_engine.decompress_img.assert_called()
+
 
 class TestXenserverUtils(test.TestCase):
     def test_is_xenserver_format(self):
