@@ -13,17 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
+from keystoneclient import exceptions
+from oslo_config import cfg
+from oslo_config import fixture as config_fixture
 
 from cinder import context
 from cinder import exception
 from cinder import quota_utils
-from cinder import test
+from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
-from keystoneclient import exceptions
-
-from oslo_config import cfg
-from oslo_config import fixture as config_fixture
 
 CONF = cfg.CONF
 
@@ -43,7 +44,7 @@ class QuotaUtilsTest(test.TestCase):
         self.auth_url = 'http://localhost:5000'
         self.context = context.RequestContext('fake_user', 'fake_proj_id')
         self.fixture = self.useFixture(config_fixture.Config(CONF))
-        self.fixture.config(auth_uri=self.auth_url, group='keystone_authtoken')
+        self.fixture.config(auth_url=self.auth_url, group='keystone_authtoken')
 
     @mock.patch('keystoneclient.client.Client')
     @mock.patch('keystoneauth1.session.Session')
@@ -54,6 +55,42 @@ class QuotaUtilsTest(test.TestCase):
                                                session=ksclient_session(),
                                                version=(3, 0))
 
+    @mock.patch('keystoneclient.client.Client')
+    @mock.patch('keystoneauth1.session.Session')
+    @mock.patch('keystoneauth1.identity.Token')
+    def test_keystone_client_instantiation_system_scope(
+            self, ks_token, ksclient_session, ksclient_class):
+        system_context = context.RequestContext(
+            'fake_user', 'fake_proj_id', system_scope='all')
+        quota_utils._keystone_client(system_context)
+        ks_token.assert_called_once_with(
+            auth_url=self.auth_url, token=system_context.auth_token,
+            system_scope=system_context.system_scope)
+
+    @mock.patch('keystoneclient.client.Client')
+    @mock.patch('keystoneauth1.session.Session')
+    @mock.patch('keystoneauth1.identity.Token')
+    def test_keystone_client_instantiation_domain_scope(
+            self, ks_token, ksclient_session, ksclient_class):
+        domain_context = context.RequestContext(
+            'fake_user', 'fake_proj_id', domain_id='default')
+        quota_utils._keystone_client(domain_context)
+        ks_token.assert_called_once_with(
+            auth_url=self.auth_url, token=domain_context.auth_token,
+            domain_id=domain_context.domain_id)
+
+    @mock.patch('keystoneclient.client.Client')
+    @mock.patch('keystoneauth1.session.Session')
+    @mock.patch('keystoneauth1.identity.Token')
+    def test_keystone_client_instantiation_project_scope(
+            self, ks_token, ksclient_session, ksclient_class):
+        project_context = context.RequestContext(
+            'fake_user', project_id=fake.PROJECT_ID)
+        quota_utils._keystone_client(project_context)
+        ks_token.assert_called_once_with(
+            auth_url=self.auth_url, token=project_context.auth_token,
+            project_id=project_context.project_id)
+
     @mock.patch('keystoneclient.client.Client')
     def test_get_project_keystoneclient_v2(self, ksclient_class):
         keystoneclient = ksclient_class.return_value
@@ -72,7 +109,7 @@ class QuotaUtilsTest(test.TestCase):
         del returned_project.subtree
         keystoneclient.projects.get.return_value = returned_project
         expected_project = quota_utils.GenericProjectInfo(
-            self.context.project_id, 'v3', 'bar')
+            self.context.project_id, 'v3', 'bar', domain_id='default')
         project = quota_utils.get_project_hierarchy(
             self.context, self.context.project_id)
         self.assertEqual(expected_project.__dict__, project.__dict__)
@@ -86,7 +123,8 @@ class QuotaUtilsTest(test.TestCase):
         returned_project.subtree = subtree_dict
         keystoneclient.projects.get.return_value = returned_project
         expected_project = quota_utils.GenericProjectInfo(
-            self.context.project_id, 'v3', 'bar', subtree_dict)
+            self.context.project_id, 'v3', 'bar', subtree_dict,
+            domain_id='default')
         project = quota_utils.get_project_hierarchy(
             self.context, self.context.project_id, subtree_as_ids=True)
         keystoneclient.projects.get.assert_called_once_with(
