@@ -12,25 +12,39 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
 import datetime
 import functools
+import json
 import os
+import sys
 import time
+from unittest import mock
 
 import ddt
-import mock
-from oslo_concurrency import processutils as putils
 from oslo_utils import timeutils
 import six
-from six.moves import range
 import webob.exc
 
 import cinder
+from cinder.api import api_utils
 from cinder import exception
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder import utils
+from cinder.volume import volume_utils
+
+POOL_CAPS = {'total_capacity_gb': 0,
+             'free_capacity_gb': 0,
+             'allocated_capacity_gb': 0,
+             'provisioned_capacity_gb': 0,
+             'max_over_subscription_ratio': '1.0',
+             'thin_provisioning_support': False,
+             'thick_provisioning_support': True,
+             'reserved_percentage': 0,
+             'volume_backend_name': 'lvm1',
+             'timestamp': timeutils.utcnow(),
+             'multiattach': True,
+             'uuid': 'a3a593da-7f8d-4bb7-8b4c-f2bc1e0b4824'}
 
 
 class ExecuteTestCase(test.TestCase):
@@ -63,6 +77,7 @@ class ExecuteTestCase(test.TestCase):
                                                 root_helper=mock_helper)
 
 
+@ddt.ddt
 class GenericUtilsTestCase(test.TestCase):
     def test_as_int(self):
         test_obj_int = '2'
@@ -108,40 +123,26 @@ class GenericUtilsTestCase(test.TestCase):
     def test_hostname_unicode_sanitization(self):
         hostname = u"\u7684.test.example.com"
         self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+                         volume_utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_periods(self):
         hostname = "....test.example.com..."
         self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+                         volume_utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_dashes(self):
         hostname = "----test.example.com---"
         self.assertEqual("test.example.com",
-                         utils.sanitize_hostname(hostname))
+                         volume_utils.sanitize_hostname(hostname))
 
     def test_hostname_sanitize_characters(self):
         hostname = "(#@&$!(@*--#&91)(__=+--test-host.example!!.com-0+"
         self.assertEqual("91----test-host.example.com-0",
-                         utils.sanitize_hostname(hostname))
+                         volume_utils.sanitize_hostname(hostname))
 
     def test_hostname_translate(self):
         hostname = "<}\x1fh\x10e\x08l\x02l\x05o\x12!{>"
-        self.assertEqual("hello", utils.sanitize_hostname(hostname))
-
-    def test_is_valid_boolstr(self):
-        self.assertTrue(utils.is_valid_boolstr(True))
-        self.assertTrue(utils.is_valid_boolstr('trUe'))
-        self.assertTrue(utils.is_valid_boolstr(False))
-        self.assertTrue(utils.is_valid_boolstr('faLse'))
-        self.assertTrue(utils.is_valid_boolstr('yeS'))
-        self.assertTrue(utils.is_valid_boolstr('nO'))
-        self.assertTrue(utils.is_valid_boolstr('y'))
-        self.assertTrue(utils.is_valid_boolstr('N'))
-        self.assertTrue(utils.is_valid_boolstr(1))
-        self.assertTrue(utils.is_valid_boolstr('1'))
-        self.assertTrue(utils.is_valid_boolstr(0))
-        self.assertTrue(utils.is_valid_boolstr('0'))
+        self.assertEqual("hello", volume_utils.sanitize_hostname(hostname))
 
     @mock.patch('os.path.join', side_effect=lambda x, y: '/'.join((x, y)))
     def test_make_dev_path(self, mock_join):
@@ -149,25 +150,7 @@ class GenericUtilsTestCase(test.TestCase):
         self.assertEqual('/dev/xvdb1', utils.make_dev_path('xvdb', 1))
         self.assertEqual('/foo/xvdc1', utils.make_dev_path('xvdc', 1, '/foo'))
 
-    @mock.patch('cinder.utils.execute')
-    def test_read_file_as_root(self, mock_exec):
-        out = mock.Mock()
-        err = mock.Mock()
-        mock_exec.return_value = (out, err)
-        test_filepath = '/some/random/path'
-        output = utils.read_file_as_root(test_filepath)
-        mock_exec.assert_called_once_with('cat', test_filepath,
-                                          run_as_root=True)
-        self.assertEqual(out, output)
-
-    @mock.patch('cinder.utils.execute',
-                side_effect=putils.ProcessExecutionError)
-    def test_read_file_as_root_fails(self, mock_exec):
-        test_filepath = '/some/random/path'
-        self.assertRaises(exception.FileNotFound,
-                          utils.read_file_as_root,
-                          test_filepath)
-
+    @test.testtools.skipIf(sys.platform == "darwin", "SKIP on OSX")
     @mock.patch('tempfile.NamedTemporaryFile')
     @mock.patch.object(os, 'open')
     @mock.patch.object(os, 'fdatasync')
@@ -212,33 +195,6 @@ class GenericUtilsTestCase(test.TestCase):
         mock_isfile.assert_called_once_with(tempfile)
         mock_unlink.assert_called_once_with(tempfile)
 
-    @mock.patch('oslo_utils.timeutils.utcnow')
-    def test_service_is_up(self, mock_utcnow):
-        fts_func = datetime.datetime.fromtimestamp
-        fake_now = 1000
-        down_time = 5
-
-        self.flags(service_down_time=down_time)
-        mock_utcnow.return_value = fts_func(fake_now)
-
-        # Up (equal)
-        service = {'updated_at': fts_func(fake_now - down_time),
-                   'created_at': fts_func(fake_now - down_time)}
-        result = utils.service_is_up(service)
-        self.assertTrue(result)
-
-        # Up
-        service = {'updated_at': fts_func(fake_now - down_time + 1),
-                   'created_at': fts_func(fake_now - down_time + 1)}
-        result = utils.service_is_up(service)
-        self.assertTrue(result)
-
-        # Down
-        service = {'updated_at': fts_func(fake_now - down_time - 1),
-                   'created_at': fts_func(fake_now - down_time - 1)}
-        result = utils.service_is_up(service)
-        self.assertFalse(result)
-
     def test_check_ssh_injection(self):
         cmd_list = ['ssh', '-D', 'my_name@name_of_remote_computer']
         self.assertIsNone(utils.check_ssh_injection(cmd_list))
@@ -320,6 +276,21 @@ class GenericUtilsTestCase(test.TestCase):
         self.assertEqual('sudo cinder-rootwrap /path/to/conf',
                          utils.get_root_helper())
 
+    @ddt.data({'path_a': 'test', 'path_b': 'test', 'exp_eq': True})
+    @ddt.data({'path_a': 'test', 'path_b': 'other', 'exp_eq': False})
+    @ddt.unpack
+    @mock.patch('os.path.normcase')
+    def test_paths_normcase_equal(self, mock_normcase, path_a,
+                                  path_b, exp_eq):
+        # os.path.normcase will lower the path string on Windows
+        # while doing nothing on other platforms.
+        mock_normcase.side_effect = lambda x: x
+
+        result = utils.paths_normcase_equal(path_a, path_b)
+        self.assertEqual(exp_eq, result)
+
+        mock_normcase.assert_has_calls([mock.call(path_a), mock.call(path_b)])
+
 
 class TemporaryChownTestCase(test.TestCase):
     @mock.patch('os.stat')
@@ -364,6 +335,16 @@ class TemporaryChownTestCase(test.TestCase):
         mock_stat.assert_called_once_with(test_filename)
         self.assertFalse(mock_exec.called)
 
+    @mock.patch('os.name', 'nt')
+    @mock.patch('os.stat')
+    @mock.patch('cinder.utils.execute')
+    def test_temporary_chown_win32(self, mock_exec, mock_stat):
+        with utils.temporary_chown(mock.sentinel.path):
+            pass
+
+        mock_exec.assert_not_called()
+        mock_stat.assert_not_called()
+
 
 class TempdirTestCase(test.TestCase):
     @mock.patch('tempfile.mkdtemp')
@@ -403,11 +384,11 @@ class WalkClassHierarchyTestCase(test.TestCase):
             pass
 
         class_pairs = zip((D, B, E),
-                          utils.walk_class_hierarchy(A, encountered=[C]))
+                          api_utils.walk_class_hierarchy(A, encountered=[C]))
         for actual, expected in class_pairs:
             self.assertEqual(expected, actual)
 
-        class_pairs = zip((D, B, C, E), utils.walk_class_hierarchy(A))
+        class_pairs = zip((D, B, C, E), api_utils.walk_class_hierarchy(A))
         for actual, expected in class_pairs:
             self.assertEqual(expected, actual)
 
@@ -469,6 +450,7 @@ class GetBlkdevMajorMinorTestCase(test.TestCase):
         self.assertEqual(size, stat_result.st_size)
         mock_stat.assert_called_once_with(test_file)
 
+    @test.testtools.skipIf(sys.platform == 'darwin', 'Not supported on macOS')
     @mock.patch('os.stat')
     def test_get_blkdev_major_minor(self, mock_stat):
 
@@ -535,7 +517,7 @@ class GetBlkdevMajorMinorTestCase(test.TestCase):
     @mock.patch('stat.S_ISBLK', return_value=False)
     def test_get_blkdev_failure(self, mock_isblk, mock_ischr, mock_stat):
         path = '/some/path'
-        self.assertRaises(exception.Error,
+        self.assertRaises(exception.CinderException,
                           utils.get_blkdev_major_minor,
                           path, lookup_for_file=False)
         mock_stat.assert_called_once_with(path)
@@ -551,7 +533,7 @@ class GetBlkdevMajorMinorTestCase(test.TestCase):
         mock_stat.assert_called_once_with(path)
         mock_isblk.assert_called_once_with(mock_stat.return_value.st_mode)
         mock_ischr.assert_called_once_with(mock_stat.return_value.st_mode)
-        self.assertIs(None, output)
+        self.assertIsNone(output)
 
 
 class MonkeyPatchTestCase(test.TestCase):
@@ -838,7 +820,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
                     {"key": "readonly", "value": "existing"}]
         volume = {'volume_admin_metadata': admin_metadata,
                   'volume_metadata': metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual([{"key": "key", "value": "value"},
                           {"key": "readonly", "value": "visible"},
                           {"key": "attached_mode", "value": "visible"}],
@@ -850,7 +832,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
         metadata = {"key": "value", "readonly": "existing"}
         volume = {'admin_metadata': admin_metadata,
                   'metadata': metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual({'key': 'value',
                           'attached_mode': 'visible',
                           'readonly': 'visible'},
@@ -864,7 +846,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
         metadata = [{"key": "key", "value": "value"}]
         volume = {'volume_admin_metadata': admin_metadata,
                   'volume_metadata': metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual([{"key": "key", "value": "value"}],
                          volume['volume_metadata'])
 
@@ -874,7 +856,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
         metadata = {"key": "value"}
         volume = {'admin_metadata': admin_metadata,
                   'metadata': metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual({'key': 'value'}, volume['metadata'])
 
     def test_add_visible_admin_metadata_no_existing_metadata(self):
@@ -882,7 +864,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
                           {"key": "readonly", "value": "visible"},
                           {"key": "attached_mode", "value": "visible"}]
         volume = {'volume_admin_metadata': admin_metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual({'attached_mode': 'visible', 'readonly': 'visible'},
                          volume['metadata'])
 
@@ -890,7 +872,7 @@ class AddVisibleAdminMetadataTestCase(test.TestCase):
                           "readonly": "visible",
                           "attached_mode": "visible"}
         volume = {'admin_metadata': admin_metadata}
-        utils.add_visible_admin_metadata(volume)
+        api_utils.add_visible_admin_metadata(volume)
         self.assertEqual({'attached_mode': 'visible', 'readonly': 'visible'},
                          volume['metadata'])
 
@@ -905,8 +887,8 @@ class InvalidFilterTestCase(test.TestCase):
         allowed_search_options = ('allowed1', 'allowed2')
         allowed_orig = ('allowed1', 'allowed2')
 
-        utils.remove_invalid_filter_options(ctxt, filters,
-                                            allowed_search_options)
+        api_utils.remove_invalid_filter_options(ctxt, filters,
+                                                allowed_search_options)
 
         self.assertEqual(allowed_orig, allowed_search_options)
         self.assertEqual(fltrs_orig, filters)
@@ -920,8 +902,8 @@ class InvalidFilterTestCase(test.TestCase):
         allowed_search_options = ('allowed1', 'allowed2')
         allowed_orig = ('allowed1', 'allowed2')
 
-        utils.remove_invalid_filter_options(ctxt, filters,
-                                            allowed_search_options)
+        api_utils.remove_invalid_filter_options(ctxt, filters,
+                                                allowed_search_options)
 
         self.assertEqual(allowed_orig, allowed_search_options)
         self.assertNotEqual(fltrs_orig, filters)
@@ -953,13 +935,10 @@ class WrongException(Exception):
 
 
 class TestRetryDecorator(test.TestCase):
-    def setUp(self):
-        super(TestRetryDecorator, self).setUp()
-
     def test_no_retry_required(self):
         self.counter = 0
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException,
                          interval=2,
                          retries=3,
@@ -976,7 +955,7 @@ class TestRetryDecorator(test.TestCase):
     def test_no_retry_required_random(self):
         self.counter = 0
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException,
                          interval=2,
                          retries=3,
@@ -997,7 +976,7 @@ class TestRetryDecorator(test.TestCase):
         backoff_rate = 2
         retries = 3
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException,
                          interval,
                          retries,
@@ -1013,7 +992,7 @@ class TestRetryDecorator(test.TestCase):
             self.assertEqual('success', ret)
             self.assertEqual(2, self.counter)
             self.assertEqual(1, mock_sleep.call_count)
-            mock_sleep.assert_called_with(interval * backoff_rate)
+            mock_sleep.assert_called_with(interval)
 
     def test_retries_once_random(self):
         self.counter = 0
@@ -1021,7 +1000,7 @@ class TestRetryDecorator(test.TestCase):
         backoff_rate = 2
         retries = 3
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException,
                          interval,
                          retries,
@@ -1046,7 +1025,7 @@ class TestRetryDecorator(test.TestCase):
         interval = 2
         backoff_rate = 4
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException,
                          interval,
                          retries,
@@ -1060,17 +1039,17 @@ class TestRetryDecorator(test.TestCase):
             self.assertEqual(retries, self.counter)
 
             expected_sleep_arg = []
-
             for i in range(retries):
                 if i > 0:
-                    interval *= backoff_rate
+                    interval *= (backoff_rate ** (i - 1))
                     expected_sleep_arg.append(float(interval))
 
-            mock_sleep.assert_has_calls(map(mock.call, expected_sleep_arg))
+            mock_sleep.assert_has_calls(
+                list(map(mock.call, expected_sleep_arg)))
 
     def test_wrong_exception_no_retry(self):
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
             @utils.retry(exception.VolumeBackendAPIException)
             def raise_unexpected_error():
                 raise WrongException("wrong exception")
@@ -1158,6 +1137,38 @@ class LogTracingTestCase(test.TestCase):
         self.assertEqual('OK', result)
         self.assertEqual(2, utils.LOG.debug.call_count)
 
+    def test_utils_trace_api_filtered(self):
+        self.mock_object(utils, 'LOG')
+
+        def filter_func(all_args):
+            return False
+
+        @utils.trace_api(filter_function=filter_func)
+        def _trace_test_api(*args, **kwargs):
+            return 'OK'
+
+        utils.setup_tracing(['api'])
+
+        result = _trace_test_api()
+        self.assertEqual('OK', result)
+        self.assertEqual(0, utils.LOG.debug.call_count)
+
+    def test_utils_trace_filtered(self):
+        self.mock_object(utils, 'LOG')
+
+        def filter_func(all_args):
+            return False
+
+        @utils.trace(filter_function=filter_func)
+        def _trace_test(*args, **kwargs):
+            return 'OK'
+
+        utils.setup_tracing(['api'])
+
+        result = _trace_test()
+        self.assertEqual('OK', result)
+        self.assertEqual(0, utils.LOG.debug.call_count)
+
     def test_utils_trace_method_default_logger(self):
         mock_log = self.mock_object(utils, 'LOG')
 
@@ -1348,6 +1359,29 @@ class LogTracingTestCase(test.TestCase):
         self.assertIn("'adminPass': '***'",
                       str(mock_log.debug.call_args_list[1]))
 
+    def test_utils_trace_method_with_password_in_formal_params(self):
+        mock_logging = self.mock_object(utils, 'logging')
+        mock_log = mock.Mock()
+        mock_log.isEnabledFor = lambda x: True
+        mock_logging.getLogger = mock.Mock(return_value=mock_log)
+
+        @utils.trace
+        def _trace_test_method(*args, **kwargs):
+            self.assertEqual('verybadpass',
+                             kwargs['test_args']['data']['password'])
+            pass
+
+        test_args = {
+            'data': {
+                'password': 'verybadpass'
+            }
+        }
+        _trace_test_method(self, test_args=test_args)
+
+        self.assertEqual(2, mock_log.debug.call_count)
+        self.assertIn("'password': '***'",
+                      str(mock_log.debug.call_args_list[0]))
+
     @ddt.data(
         {'total': 30.01, 'free': 28.01, 'provisioned': 2.0, 'max_ratio': 1.0,
          'thin_support': False, 'thick_support': True,
@@ -1436,22 +1470,244 @@ class TestComparableMixin(test.TestCase):
                          self.one._compare(1, self.one._cmpkey))
 
 
+@ddt.ddt
 class TestValidateInteger(test.TestCase):
 
-    def test_validate_integer_greater_than_max_int_limit(self):
-        value = (2 ** 31) + 1
+    @ddt.data(
+        (2 ** 31) + 1,  # More than max value
+        -12,  # Less than min value
+        2.05,  # Float value
+        "12.05",  # Float value in string format
+        "should be int",  # String
+        u"test"  # String in unicode format
+    )
+    def test_validate_integer_raise_assert(self, value):
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          utils.validate_integer,
+                          api_utils.validate_integer,
                           value, 'limit', min_value=-1, max_value=(2 ** 31))
 
-    def test_validate_integer_less_than_min_int_limit(self):
-        value = -12
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          utils.validate_integer,
-                          value, 'limit', min_value=-1, max_value=(2 ** 31))
+    @ddt.data(
+        "123",  # integer in string format
+        123,  # integer
+        u"123"  # integer in unicode format
+    )
+    def test_validate_integer(self, value):
+        res = api_utils.validate_integer(value, 'limit', min_value=-1,
+                                         max_value=(2 ** 31))
+        self.assertEqual(123, res)
 
-    def test_validate_integer_invalid_limit(self):
-        value = "should_be_int"
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          utils.validate_integer,
-                          value, 'limit', min_value=-1, max_value=(2 ** 31))
+
+@ddt.ddt
+class TestNotificationShortCircuit(test.TestCase):
+    def test_do_nothing_getter(self):
+        """Test any attribute will always return the same instance (self)."""
+        donothing = utils.DoNothing()
+        self.assertIs(donothing, donothing.anyname)
+
+    def test_do_nothing_caller(self):
+        """Test calling the object will always return the same instance."""
+        donothing = utils.DoNothing()
+        self.assertIs(donothing, donothing())
+
+    def test_do_nothing_json_serializable(self):
+        """Test calling the object will always return the same instance."""
+        donothing = utils.DoNothing()
+        self.assertEqual('""', json.dumps(donothing))
+
+    @utils.if_notifications_enabled
+    def _decorated_method(self):
+        return mock.sentinel.success
+
+    def test_if_notification_enabled_when_enabled(self):
+        """Test method is called when notifications are enabled."""
+        result = self._decorated_method()
+        self.assertEqual(mock.sentinel.success, result)
+
+    @ddt.data([], ['noop'], ['noop', 'noop'])
+    def test_if_notification_enabled_when_disabled(self, driver):
+        """Test method is not called when notifications are disabled."""
+        self.override_config('driver', driver,
+                             group='oslo_messaging_notifications')
+        result = self._decorated_method()
+        self.assertEqual(utils.DO_NOTHING, result)
+
+
+@ddt.ddt
+class TestLogLevels(test.TestCase):
+    @ddt.data(None, '', 'wronglevel')
+    def test_get_log_method_invalid(self, level):
+        self.assertRaises(exception.InvalidInput,
+                          utils.get_log_method, level)
+
+    @ddt.data(('info', utils.logging.INFO), ('warning', utils.logging.WARNING),
+              ('INFO', utils.logging.INFO), ('wArNiNg', utils.logging.WARNING),
+              ('error', utils.logging.ERROR), ('debug', utils.logging.DEBUG))
+    @ddt.unpack
+    def test_get_log_method(self, level, logger):
+        result = utils.get_log_method(level)
+        self.assertEqual(logger, result)
+
+    def test_get_log_levels(self):
+        levels = utils.get_log_levels('cinder.api')
+        self.assertTrue(len(levels) > 1)
+        self.assertSetEqual({'INFO'}, set(levels.values()))
+
+    @ddt.data(None, '', 'wronglevel')
+    def test_set_log_levels_invalid(self, level):
+        self.assertRaises(exception.InvalidInput,
+                          utils.set_log_levels, '', level)
+
+    def test_set_log_levels(self):
+        prefix = 'cinder.utils'
+        levels = utils.get_log_levels(prefix)
+
+        utils.set_log_levels(prefix, 'debug')
+        levels = utils.get_log_levels(prefix)
+        self.assertEqual('DEBUG', levels[prefix])
+
+        utils.set_log_levels(prefix, 'warning')
+        levels = utils.get_log_levels(prefix)
+        self.assertEqual('WARNING', levels[prefix])
+
+
+@ddt.ddt
+class TestCheckMetadataProperties(test.TestCase):
+    @ddt.data(
+        {'a': {'foo': 'bar'}},  # value is a nested dict
+        {'a': 123},  # value is an integer
+        {'a': 123.4},  # value is a float
+        {'a': True},  # value is a bool
+        {'a': ('foo', 'bar')},  # value is a tuple
+        {'a': []},  # value is a list
+        {'a': None}  # value is None
+    )
+    def test_metadata_value_not_string_raise(self, meta):
+        self.assertRaises(exception.InvalidVolumeMetadata,
+                          utils.check_metadata_properties,
+                          meta)
+
+    def test_metadata_value_not_dict_raise(self):
+        meta = 123
+        self.assertRaises(exception.InvalidInput,
+                          utils.check_metadata_properties,
+                          meta)
+
+
+POOL_CAP1 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+             'thin_provisioning_support': False, 'total_capacity_gb': 10,
+             'free_capacity_gb': 10, 'max_over_subscription_ratio': 1.0}
+POOL_CAP2 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+             'thin_provisioning_support': True, 'total_capacity_gb': 100,
+             'free_capacity_gb': 95, 'max_over_subscription_ratio': None}
+POOL_CAP3 = {'allocated_capacity_gb': 0, 'provisioned_capacity_gb': 0,
+             'thin_provisioning_support': True, 'total_capacity_gb': 100,
+             'free_capacity_gb': 100, 'max_over_subscription_ratio': 'auto'}
+POOL_CAP4 = {'allocated_capacity_gb': 100,
+             'thin_provisioning_support': True, 'total_capacity_gb': 2500,
+             'free_capacity_gb': 500, 'max_over_subscription_ratio': 'auto'}
+POOL_CAP5 = {'allocated_capacity_gb': 10000,
+             'thin_provisioning_support': True, 'total_capacity_gb': 2500,
+             'free_capacity_gb': 0.1, 'max_over_subscription_ratio': 'auto'}
+POOL_CAP6 = {'allocated_capacity_gb': 1000, 'provisioned_capacity_gb': 1010,
+             'thin_provisioning_support': True, 'total_capacity_gb': 2500,
+             'free_capacity_gb': 2500, 'max_over_subscription_ratio': 'auto'}
+POOL_CAP7 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+             'thin_provisioning_support': True, 'total_capacity_gb': 10,
+             'free_capacity_gb': 10}
+POOL_CAP8 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+             'thin_provisioning_support': True, 'total_capacity_gb': 10,
+             'free_capacity_gb': 10, 'max_over_subscription_ratio': '15.5'}
+POOL_CAP9 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+             'thin_provisioning_support': True, 'total_capacity_gb': 10,
+             'free_capacity_gb': 'unknown',
+             'max_over_subscription_ratio': '15.5'}
+POOL_CAP10 = {'allocated_capacity_gb': 10, 'provisioned_capacity_gb': 10,
+              'thin_provisioning_support': True,
+              'total_capacity_gb': 'infinite', 'free_capacity_gb': 10,
+              'max_over_subscription_ratio': '15.5'}
+
+
+@ddt.ddt
+class TestAutoMaxOversubscriptionRatio(test.TestCase):
+    @ddt.data({'data': POOL_CAP1,
+               'global_max_over_subscription_ratio': 'auto',
+               'expected_result': 1.0},
+              {'data': POOL_CAP2,
+               'global_max_over_subscription_ratio': 'auto',
+               'expected_result': 2.67},
+              {'data': POOL_CAP3,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 20},
+              {'data': POOL_CAP4,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 1.05},
+              {'data': POOL_CAP5,
+               'global_max_over_subscription_ratio': '10.0',
+               'expected_result': 5.0},
+              {'data': POOL_CAP6,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 1011.0},
+              {'data': POOL_CAP7,
+               'global_max_over_subscription_ratio': 'auto',
+               'expected_result': 11.0},
+              {'data': POOL_CAP8,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 15.5},
+              {'data': POOL_CAP9,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 1.0},
+              {'data': POOL_CAP10,
+               'global_max_over_subscription_ratio': '20.0',
+               'expected_result': 1.0},
+              )
+    @ddt.unpack
+    def test_calculate_max_over_subscription_ratio(
+            self, data, expected_result, global_max_over_subscription_ratio):
+
+        result = utils.calculate_max_over_subscription_ratio(
+            data, global_max_over_subscription_ratio)
+        # Just for sake of testing we reduce the float precision
+        if result is not None:
+            result = round(result, 2)
+        self.assertEqual(expected_result, result)
+
+
+@ddt.ddt
+class LimitOperationsTestCase(test.TestCase):
+    @ddt.data(1, 5)
+    @mock.patch('contextlib.suppress')
+    def test_semaphore_factory_no_limit(self, processes, mock_suppress):
+        res = utils.semaphore_factory(0, processes)
+        mock_suppress.assert_called_once_with()
+        self.assertEqual(mock_suppress.return_value, res)
+
+    @mock.patch('eventlet.Semaphore')
+    def test_semaphore_factory_with_limit(self, mock_semaphore):
+        max_operations = 15
+        res = utils.semaphore_factory(max_operations, 1)
+        mock_semaphore.assert_called_once_with(max_operations)
+        self.assertEqual(mock_semaphore.return_value, res)
+
+    @mock.patch('cinder.utils.Semaphore')
+    def test_semaphore_factory_with_limit_and_workers(self, mock_semaphore):
+        max_operations = 15
+        processes = 5
+        res = utils.semaphore_factory(max_operations, processes)
+        mock_semaphore.assert_called_once_with(max_operations)
+        self.assertEqual(mock_semaphore.return_value, res)
+
+    @mock.patch('multiprocessing.Semaphore')
+    @mock.patch('eventlet.tpool.execute')
+    def test_semaphore(self, mock_exec, mock_semaphore):
+        limit = 15
+        res = utils.Semaphore(limit)
+        self.assertEqual(limit, res.limit)
+
+        mocked_semaphore = mock_semaphore.return_value
+        self.assertEqual(mocked_semaphore, res.semaphore)
+        mock_semaphore.assert_called_once_with(limit)
+
+        with res:
+            mock_exec.assert_called_once_with(mocked_semaphore.__enter__)
+            mocked_semaphore.__exit__.assert_not_called()
+        mocked_semaphore.__exit__.assert_called_once_with(None, None, None)
