@@ -13,21 +13,21 @@
 #    under the License.
 """Unit Tests for volume types code."""
 
-
 import datetime
-import mock
 import time
+from unittest import mock
 
-from oslo_config import cfg
+from oslo_db import exception as db_exc
+from oslo_utils import uuidutils
 
 from cinder import context
 from cinder import db
 from cinder.db.sqlalchemy import api as db_api
 from cinder.db.sqlalchemy import models
 from cinder import exception
-from cinder import test
 from cinder.tests.unit import conf_fixture
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder.volume import qos_specs
 from cinder.volume import volume_types
 
@@ -46,6 +46,35 @@ class VolumeTypeTestCase(test.TestCase):
                                     visible="True")
         self.vol_type1_description = self.vol_type1_name + '_desc'
 
+    def test_volume_type_destroy_with_encryption(self):
+        volume_type = volume_types.create(self.ctxt, "type1")
+        volume_type_id = volume_type.get('id')
+
+        encryption = {
+            'control_location': 'front-end',
+            'provider': 'fake_provider',
+        }
+        db_api.volume_type_encryption_create(self.ctxt, volume_type_id,
+                                             encryption)
+        ret = volume_types.get_volume_type_encryption(self.ctxt,
+                                                      volume_type_id)
+        self.assertIsNotNone(ret)
+
+        volume_types.destroy(self.ctxt, volume_type_id)
+        ret = volume_types.get_volume_type_encryption(self.ctxt,
+                                                      volume_type_id)
+        self.assertIsNone(ret)
+
+    def test_get_volume_type_by_name_with_uuid_name(self):
+        """Ensure volume types can be created and found."""
+        uuid_format_name = uuidutils.generate_uuid()
+        volume_types.create(self.ctxt, uuid_format_name,
+                            self.vol_type1_specs,
+                            description=self.vol_type1_description)
+        type_ref = volume_types.get_by_name_or_id(self.ctxt,
+                                                  uuid_format_name)
+        self.assertEqual(uuid_format_name, type_ref['name'])
+
     def test_volume_type_create_then_destroy(self):
         """Ensure volume types can be created and deleted."""
         project_id = fake.PROJECT_ID
@@ -78,10 +107,9 @@ class VolumeTypeTestCase(test.TestCase):
         # update
         new_type_name = self.vol_type1_name + '_updated'
         new_type_desc = self.vol_type1_description + '_updated'
-        type_ref_updated = volume_types.update(self.ctxt,
-                                               type_ref.id,
-                                               new_type_name,
-                                               new_type_desc)
+        volume_types.update(self.ctxt, type_ref.id, new_type_name,
+                            new_type_desc)
+        type_ref_updated = volume_types.get_volume_type(self.ctxt, type_ref.id)
         self.assertEqual(new_type_name, type_ref_updated['name'])
         self.assertEqual(new_type_desc, type_ref_updated['description'])
 
@@ -95,7 +123,7 @@ class VolumeTypeTestCase(test.TestCase):
         # on destroying the volume type
         vol_type_access = db_api._volume_type_access_query(
             self.ctxt).filter_by(volume_type_id=type_ref['id']).all()
-        self.assertFalse(vol_type_access)
+        self.assertEqual([], vol_type_access)
 
     @mock.patch('cinder.quota.VolumeTypeQuotaEngine.'
                 'update_quota_resource')
@@ -114,6 +142,30 @@ class VolumeTypeTestCase(test.TestCase):
                                                   new_type_name)
         volume_types.destroy(self.ctxt, type_ref.id)
 
+    @mock.patch('cinder.quota.VolumeTypeQuotaEngine.'
+                'update_quota_resource')
+    def test_update_volume_type_name_with_db_error(self, mock_update_quota):
+        type_ref = volume_types.create(self.ctxt,
+                                       self.vol_type1_name,
+                                       self.vol_type1_specs,
+                                       description=self.vol_type1_description)
+        mock_update_quota.side_effect = db_exc.DBError
+        new_type_name = self.vol_type1_name + '_updated'
+        description = 'new_test'
+        is_public = False
+        self.assertRaises(exception.VolumeTypeUpdateFailed,
+                          volume_types.update, self.ctxt, type_ref.id,
+                          new_type_name, description, is_public)
+        mock_update_quota.assert_called_once_with(self.ctxt,
+                                                  self.vol_type1_name,
+                                                  new_type_name)
+        new = volume_types.get_volume_type_by_name(self.ctxt,
+                                                   self.vol_type1_name)
+        self.assertEqual(self.vol_type1_name, new.get('name'))
+        self.assertEqual(self.vol_type1_description, new.get('description'))
+        self.assertTrue(new.get('is_public'))
+        volume_types.destroy(self.ctxt, type_ref.id)
+
     def test_volume_type_create_then_destroy_with_non_admin(self):
         """Ensure volume types can be created and deleted by non-admin user.
 
@@ -139,10 +191,9 @@ class VolumeTypeTestCase(test.TestCase):
         # update
         new_type_name = self.vol_type1_name + '_updated'
         new_type_desc = self.vol_type1_description + '_updated'
-        type_ref_updated = volume_types.update(self.ctxt,
-                                               type_ref.id,
-                                               new_type_name,
-                                               new_type_desc)
+        volume_types.update(self.ctxt, type_ref.id, new_type_name,
+                            new_type_desc)
+        type_ref_updated = volume_types.get_volume_type(self.ctxt, type_ref.id)
         self.assertEqual(new_type_name, type_ref_updated['name'])
         self.assertEqual(new_type_desc, type_ref_updated['description'])
 
@@ -165,17 +216,40 @@ class VolumeTypeTestCase(test.TestCase):
     def test_get_all_volume_types(self):
         """Ensures that all volume types can be retrieved."""
         session = db_api.get_session()
-        total_volume_types = session.query(models.VolumeTypes).count()
+        total_volume_types = session.query(models.VolumeType).count()
         vol_types = volume_types.get_all_types(self.ctxt)
         self.assertEqual(total_volume_types, len(vol_types))
 
     def test_get_default_volume_type(self):
         """Ensures default volume type can be retrieved."""
-        volume_types.create(self.ctxt, conf_fixture.def_vol_type, {})
         default_vol_type = volume_types.get_default_volume_type()
         self.assertEqual(conf_fixture.def_vol_type,
                          default_vol_type.get('name'))
 
+    def test_get_default_volume_type_not_found(self):
+        """Ensure setting non-existent default type raises error."""
+        self.flags(default_volume_type='fake_type')
+        self.assertRaises(exception.VolumeTypeDefaultMisconfiguredError,
+                          volume_types.get_default_volume_type)
+
+    def test_delete_default_volume_type(self):
+        """Ensures default volume type cannot be deleted."""
+        default = volume_types.create(self.ctxt, 'default_type')
+
+        self.flags(default_volume_type='default_type')
+        self.assertRaises(exception.VolumeTypeDefaultDeletionError,
+                          volume_types.destroy, self.ctxt, default['id'])
+
+    def test_delete_when_default_volume_type_not_found(self):
+        """Ensures volume types cannot be deleted until valid default is set.
+
+        """
+        default = volume_types.create(self.ctxt, 'default_type')
+
+        self.flags(default_volume_type='fake_default')
+        self.assertRaises(exception.VolumeTypeDefaultMisconfiguredError,
+                          volume_types.destroy, self.ctxt, default['id'])
+
     def test_default_volume_type_missing_in_db(self):
         """Test default volume type is missing in database.
 
@@ -183,15 +257,24 @@ class VolumeTypeTestCase(test.TestCase):
         is not in database.
         """
         default_vol_type = volume_types.get_default_volume_type()
-        self.assertEqual({}, default_vol_type)
-
-    def test_get_default_volume_type_under_non_default(self):
-        cfg.CONF.set_default('default_volume_type', None)
-
-        self.assertEqual({}, volume_types.get_default_volume_type())
+        self.assertEqual(
+            {'created_at': default_vol_type['created_at'],
+             'deleted': False,
+             'deleted_at': None,
+             'description': u'Default Volume Type',
+             'extra_specs': {},
+             'id': default_vol_type['id'],
+             'is_public': True,
+             'name': u'__DEFAULT__',
+             'qos_specs_id': None,
+             'updated_at': default_vol_type['updated_at']},
+            default_vol_type)
 
     def test_non_existent_vol_type_shouldnt_delete(self):
         """Ensures that volume type creation fails with invalid args."""
+        # create a dummy type as DB requires at least 1 type to perform the
+        # delete operation
+        volume_types.create(self.ctxt, self.vol_type1_name)
         self.assertRaises(exception.VolumeTypeNotFound,
                           volume_types.destroy, self.ctxt, "sfsfsdfdfs")
 
@@ -363,7 +446,9 @@ class VolumeTypeTestCase(test.TestCase):
                                       'k2': 'v2',
                                       'k3': 'v3'}}}
         res = volume_types.get_volume_type_qos_specs(type_ref['id'])
-        self.assertDictMatch(expected, res)
+        specs = db.qos_specs_get(self.ctxt, qos_ref['id'])
+        expected['qos_specs']['created_at'] = specs['created_at']
+        self.assertDictEqual(expected, res)
 
     def test_volume_types_diff(self):
         # type_ref 1 and 2 have the same extra_specs, while 3 has different
@@ -384,7 +469,7 @@ class VolumeTypeTestCase(test.TestCase):
         self.assertEqual(('val1', 'val0'), diff['extra_specs']['key1'])
 
         # qos_ref 1 and 2 have the same specs, while 3 has different
-        qos_keyvals1 = {'k1': 'v1', 'k2': 'v2', 'k3': 'v3'}
+        qos_keyvals1 = {'k1': 'v1', 'k2': 'v2', 'k3': 'v3', 'created_at': 'v4'}
         qos_keyvals2 = {'k1': 'v0', 'k2': 'v2', 'k3': 'v3'}
         qos_ref1 = qos_specs.create(self.ctxt, 'qos-specs-1', qos_keyvals1)
         qos_ref2 = qos_specs.create(self.ctxt, 'qos-specs-2', qos_keyvals1)
@@ -416,11 +501,9 @@ class VolumeTypeTestCase(test.TestCase):
 
         # And add encryption for good measure
         enc_keyvals1 = {'cipher': 'c1', 'key_size': 256, 'provider': 'p1',
-                        'control_location': 'front-end',
-                        'encryption_id': 'uuid1'}
+                        'control_location': 'front-end'}
         enc_keyvals2 = {'cipher': 'c1', 'key_size': 128, 'provider': 'p1',
-                        'control_location': 'front-end',
-                        'encryption_id': 'uuid2'}
+                        'control_location': 'front-end'}
         db.volume_type_encryption_create(self.ctxt, type_ref1['id'],
                                          enc_keyvals1)
         db.volume_type_encryption_create(self.ctxt, type_ref2['id'],
@@ -441,13 +524,13 @@ class VolumeTypeTestCase(test.TestCase):
         self.assertEqual({'consumer': (None, 'back-end'),
                           'k1': (None, 'v1'),
                           'k2': (None, 'v2'),
-                          'k3': (None, 'v3')}, diff['qos_specs'])
+                          'k3': (None, 'v3'),
+                          'created_at': (None, 'v4')}, diff['qos_specs'])
         self.assertEqual({'cipher': (None, 'c1'),
                           'control_location': (None, 'front-end'),
                           'deleted': (None, False),
                           'key_size': (None, 256),
-                          'provider': (None, 'p1'),
-                          'encryption_id': (None, 'uuid1')},
+                          'provider': (None, 'p1')},
                          diff['encryption'])
 
     def test_encryption_create(self):
@@ -496,15 +579,15 @@ class VolumeTypeTestCase(test.TestCase):
         self.assertFalse(volume_types.is_public_volume_type(self.ctxt,
                                                             volume_type_id))
 
-    def test_ensure_no_extra_specs_for_non_admin(self):
-        # non-admin users shouldn't get extra-specs back in type-get/list etc
+    def test_ensure__extra_specs_for_non_admin(self):
+        # non-admin users get extra-specs back in type-get/list etc at DB layer
         ctxt = context.RequestContext('average-joe',
                                       'd802f078-0af1-4e6b-8c02-7fac8d4339aa',
                                       auth_token='token',
                                       is_admin=False)
         volume_types.create(self.ctxt, "type-test", is_public=False)
         vtype = volume_types.get_volume_type_by_name(ctxt, 'type-test')
-        self.assertIsNone(vtype.get('extra_specs', None))
+        self.assertIsNotNone(vtype.get('extra_specs', None))
 
     def test_ensure_extra_specs_for_admin(self):
         # admin users should get extra-specs back in type-get/list etc
@@ -554,3 +637,72 @@ class VolumeTypeTestCase(test.TestCase):
                 'cipher': 'fake1',
                 'created_at': 'time1', }
         self._exec_volume_types_encryption_changed(enc1, None, True)
+
+    @mock.patch('cinder.volume.volume_types.CONF')
+    @mock.patch('cinder.volume.volume_types.rpc')
+    def test_notify_about_volume_type_access_usage(self, mock_rpc,
+                                                   mock_conf):
+        mock_conf.host = 'host1'
+        project_id = fake.PROJECT_ID
+        volume_type_id = fake.VOLUME_TYPE_ID
+
+        output = volume_types.notify_about_volume_type_access_usage(
+            mock.sentinel.context,
+            volume_type_id,
+            project_id,
+            'test_suffix')
+
+        self.assertIsNone(output)
+        mock_rpc.get_notifier.assert_called_once_with('volume_type_project',
+                                                      'host1')
+        mock_rpc.get_notifier.return_value.info.assert_called_once_with(
+            mock.sentinel.context,
+            'volume_type_project.test_suffix',
+            {'volume_type_id': volume_type_id,
+             'project_id': project_id})
+
+    def test_provision_filter_on_size(self):
+        volume_types.create(self.ctxt, "type1",
+                            {"key1": "val1", "key2": "val2"})
+        volume_types.create(self.ctxt, "type2",
+                            {volume_types.MIN_SIZE_KEY: "12",
+                             "key3": "val3"})
+        volume_types.create(self.ctxt, "type3",
+                            {volume_types.MAX_SIZE_KEY: "99",
+                             "key4": "val4"})
+        volume_types.create(self.ctxt, "type4",
+                            {volume_types.MIN_SIZE_KEY: "24",
+                             volume_types.MAX_SIZE_KEY: "99",
+                             "key4": "val4"})
+
+        # Make sure we don't raise if there are no min/max set
+        type1 = volume_types.get_by_name_or_id(self.ctxt, 'type1')
+        volume_types.provision_filter_on_size(self.ctxt, type1, "11")
+
+        # verify minimum size requirements
+        type2 = volume_types.get_by_name_or_id(self.ctxt, 'type2')
+        self.assertRaises(exception.InvalidInput,
+                          volume_types.provision_filter_on_size,
+                          self.ctxt, type2, "11")
+        volume_types.provision_filter_on_size(self.ctxt, type2, "12")
+        volume_types.provision_filter_on_size(self.ctxt, type2, "100")
+
+        # verify max size requirements
+        type3 = volume_types.get_by_name_or_id(self.ctxt, 'type3')
+        self.assertRaises(exception.InvalidInput,
+                          volume_types.provision_filter_on_size,
+                          self.ctxt, type3, "100")
+        volume_types.provision_filter_on_size(self.ctxt, type3, "99")
+        volume_types.provision_filter_on_size(self.ctxt, type3, "1")
+
+        # verify min and max
+        type4 = volume_types.get_by_name_or_id(self.ctxt, 'type4')
+        self.assertRaises(exception.InvalidInput,
+                          volume_types.provision_filter_on_size,
+                          self.ctxt, type4, "20")
+        self.assertRaises(exception.InvalidInput,
+                          volume_types.provision_filter_on_size,
+                          self.ctxt, type4, "130")
+        volume_types.provision_filter_on_size(self.ctxt, type4, "24")
+        volume_types.provision_filter_on_size(self.ctxt, type4, "99")
+        volume_types.provision_filter_on_size(self.ctxt, type4, "30")
