@@ -13,25 +13,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-"""
-Tests for the Inspur InStorage volume driver.
-"""
+"""Tests for the Inspur InStorage volume driver."""
+
+from unittest import mock
 
 from eventlet import greenthread
-import mock
 from oslo_utils import importutils
 import six
 
 from cinder import context
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as testutils
+from cinder.tests.unit.volume.drivers.inspur.instorage import fakes
 from cinder.volume import configuration as conf
 from cinder.volume.drivers.inspur.instorage import instorage_iscsi
 from cinder.volume import volume_types
 
-from cinder.tests.unit.volume.drivers.inspur.instorage import fakes
-
 
 class InStorageMCSISCSIDriverTestCase(test.TestCase):
     def setUp(self):
