@@ -14,15 +14,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
+from unittest import mock
 import uuid
 
+import ddt
 from lxml import etree
-import mock
 import six
-import time
 
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.netapp.dataontap.client import (
     fakes as fake_client)
 import cinder.tests.unit.volume.drivers.netapp.dataontap.fakes as fake
@@ -34,9 +35,11 @@ CONNECTION_INFO = {'hostname': 'hostname',
                    'transport_type': 'https',
                    'port': 443,
                    'username': 'admin',
-                   'password': 'passw0rd'}
+                   'password': 'passw0rd',
+                   'api_trace_pattern': 'fake_regex'}
 
 
+@ddt.ddt
 class NetAppBaseClientTestCase(test.TestCase):
 
     def setUp(self):
@@ -46,16 +49,15 @@ class NetAppBaseClientTestCase(test.TestCase):
         self.mock_object(client_base.Client, '_init_ssh_client')
         self.client = client_base.Client(**CONNECTION_INFO)
         self.client.connection = mock.MagicMock()
+        self.client.connection.get_api_version.return_value = (1, 100)
         self.client.ssh_client = mock.MagicMock()
         self.connection = self.client.connection
         self.fake_volume = six.text_type(uuid.uuid4())
         self.fake_lun = six.text_type(uuid.uuid4())
         self.fake_size = '1024'
         self.fake_metadata = {'OsType': 'linux', 'SpaceReserved': 'true'}
-        self.mock_send_request = self.mock_object(self.client, 'send_request')
-
-    def tearDown(self):
-        super(NetAppBaseClientTestCase, self).tearDown()
+        self.mock_send_request = self.mock_object(
+            self.client.connection, 'send_request')
 
     def test_get_ontapi_version(self):
         version_response = netapp_api.NaElement(
@@ -87,8 +89,26 @@ class NetAppBaseClientTestCase(test.TestCase):
         self.assertIsNone(self.client.check_is_naelement(element))
         self.assertRaises(ValueError, self.client.check_is_naelement, None)
 
-    def test_create_lun(self):
+    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
+              {'ontap_version': '9.4', 'space_reservation': 'false'},
+              {'ontap_version': '9.6', 'space_reservation': 'true'},
+              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.unpack
+    def test_create_lun(self, ontap_version, space_reservation):
         expected_path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
+        self.fake_metadata['SpaceReserved'] = space_reservation
+        expected_space_reservation = space_reservation
+        self.mock_object(self.client, 'get_ontap_version',
+                         return_value=ontap_version)
+        mock_resize_lun = self.mock_object(
+            client_base.Client, 'do_direct_resize')
+        mock_set_space_reservation = self.mock_object(
+            client_base.Client, 'set_lun_space_reservation')
+        initial_size = self.fake_size
+
+        if ontap_version < '9.5':
+            initial_size = fake.MAX_SIZE_FOR_A_LUN
+            expected_space_reservation = 'false'
 
         with mock.patch.object(netapp_api.NaElement,
                                'create_node_with_children',
@@ -98,21 +118,102 @@ class NetAppBaseClientTestCase(test.TestCase):
                                    self.fake_size,
                                    self.fake_metadata)
 
-            mock_create_node.assert_called_once_with(
+            mock_create_node.assert_called_with(
                 'lun-create-by-size',
                 **{'path': expected_path,
-                   'size': self.fake_size,
+                   'size': initial_size,
                    'ostype': self.fake_metadata['OsType'],
                    'space-reservation-enabled':
-                   self.fake_metadata['SpaceReserved']})
-            self.connection.invoke_successfully.assert_called_once_with(
+                       expected_space_reservation})
+            self.connection.invoke_successfully.assert_called_with(
                 mock.ANY, True)
 
-    def test_create_lun_with_qos_policy_group_name(self):
+        if ontap_version < '9.5':
+            mock_resize_lun.assert_called_once_with(
+                expected_path, self.fake_size)
+
+        if ontap_version < '9.5' and space_reservation == 'true':
+            mock_set_space_reservation.assert_called_once_with(
+                expected_path, True)
+        else:
+            mock_set_space_reservation.assert_not_called()
+
+    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
+              {'ontap_version': '9.4', 'space_reservation': 'false'},
+              {'ontap_version': '9.6', 'space_reservation': 'true'},
+              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.unpack
+    def test_create_lun_exact_size(self, ontap_version, space_reservation):
+        expected_path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
+        self.connection.get_api_version.return_value = (1, 110)
+        self.fake_metadata['SpaceReserved'] = space_reservation
+        expected_space_reservation = self.fake_metadata['SpaceReserved']
+        self.mock_object(self.client, 'get_ontap_version',
+                         return_value=ontap_version)
+        mock_resize_lun = self.mock_object(
+            client_base.Client, 'do_direct_resize')
+        mock_set_space_reservation = self.mock_object(
+            client_base.Client, 'set_lun_space_reservation')
+        initial_size = self.fake_size
+
+        if ontap_version < '9.5':
+            initial_size = fake.MAX_SIZE_FOR_A_LUN
+            expected_space_reservation = 'false'
+
+        with mock.patch.object(netapp_api.NaElement,
+                               'create_node_with_children',
+                               ) as mock_create_node:
+            self.client.create_lun(self.fake_volume,
+                                   self.fake_lun,
+                                   self.fake_size,
+                                   self.fake_metadata)
+
+            mock_create_node.assert_called_with(
+                'lun-create-by-size',
+                **{'path': expected_path,
+                   'size': initial_size,
+                   'ostype': self.fake_metadata['OsType'],
+                   'use-exact-size': 'true',
+                   'space-reservation-enabled':
+                       expected_space_reservation})
+            self.connection.invoke_successfully.assert_called_with(
+                mock.ANY, True)
+
+        if ontap_version < '9.5':
+            mock_resize_lun.assert_called_once_with(
+                expected_path, self.fake_size)
+
+        if ontap_version < '9.5' and space_reservation == 'true':
+            mock_set_space_reservation.assert_called_once_with(
+                expected_path, True)
+        else:
+            mock_set_space_reservation.assert_not_called()
+
+    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
+              {'ontap_version': '9.4', 'space_reservation': 'false'},
+              {'ontap_version': '9.6', 'space_reservation': 'true'},
+              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.unpack
+    def test_create_lun_with_qos_policy_group_name(
+            self, ontap_version, space_reservation):
         expected_path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
         expected_qos_group_name = 'qos_1'
         mock_request = mock.Mock()
 
+        self.fake_metadata['SpaceReserved'] = space_reservation
+        expected_space_reservation = self.fake_metadata['SpaceReserved']
+        self.mock_object(self.client, 'get_ontap_version',
+                         return_value=ontap_version)
+        mock_resize_lun = self.mock_object(
+            client_base.Client, 'do_direct_resize')
+        mock_set_space_reservation = self.mock_object(
+            client_base.Client, 'set_lun_space_reservation')
+        initial_size = self.fake_size
+
+        if ontap_version < '9.5':
+            initial_size = fake.MAX_SIZE_FOR_A_LUN
+            expected_space_reservation = 'false'
+
         with mock.patch.object(netapp_api.NaElement,
                                'create_node_with_children',
                                return_value=mock_request
@@ -124,20 +225,65 @@ class NetAppBaseClientTestCase(test.TestCase):
                 self.fake_metadata,
                 qos_policy_group_name=expected_qos_group_name)
 
-            mock_create_node.assert_called_once_with(
+            mock_create_node.assert_called_with(
                 'lun-create-by-size',
-                **{'path': expected_path, 'size': self.fake_size,
+                **{'path': expected_path, 'size': initial_size,
                     'ostype': self.fake_metadata['OsType'],
                     'space-reservation-enabled':
-                    self.fake_metadata['SpaceReserved']})
-            mock_request.add_new_child.assert_called_once_with(
+                        expected_space_reservation})
+            mock_request.add_new_child.assert_called_with(
                 'qos-policy-group', expected_qos_group_name)
+            self.connection.invoke_successfully.assert_called_with(
+                mock.ANY, True)
+
+        if ontap_version < '9.5':
+            mock_resize_lun.assert_called_once_with(
+                expected_path, self.fake_size)
+
+        if ontap_version < '9.5' and space_reservation == 'true':
+            mock_set_space_reservation.assert_called_once_with(
+                expected_path, True)
+        else:
+            mock_set_space_reservation.assert_not_called()
+
+    def test_get_ontap_version(self):
+        version_response = netapp_api.NaElement(
+            fake.SYSTEM_GET_VERSION_RESPONSE)
+        self.connection.invoke_successfully.return_value = version_response
+
+        result = self.client.get_ontap_version(cached=False)
+
+        self.assertEqual(('9.6'), result)
+
+    def test_get_ontap_version_cached(self):
+        self.connection.get_ontap_version.return_value = '9.6'
+
+        result = self.client.get_ontap_version()
+
+        self.connection.get_ontap_version.assert_called_once_with()
+        self.assertEqual(('9.6'), result)
+
+    def test_set_lun_space_reservation(self):
+        path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
+
+        with mock.patch.object(netapp_api.NaElement,
+                               'create_node_with_children',
+                               ) as mock_set_space_reservation:
+            self.client.set_lun_space_reservation(path, True)
+
+            mock_set_space_reservation.assert_called_once_with(
+                'lun-set-space-reservation-info',
+                **{'path': path,
+                   'enable': 'True'})
             self.connection.invoke_successfully.assert_called_once_with(
                 mock.ANY, True)
 
-    def test_create_lun_raises_on_failure(self):
+    @ddt.data('9.4', '9.6')
+    def test_create_lun_raises_on_failure(self, ontap_version):
         self.connection.invoke_successfully = mock.Mock(
             side_effect=netapp_api.NaApiError)
+        self.mock_object(self.client, 'get_ontap_version',
+                         return_value=ontap_version)
 
         self.assertRaises(netapp_api.NaApiError,
                           self.client.create_lun,
@@ -515,7 +661,7 @@ class NetAppBaseClientTestCase(test.TestCase):
             'volume': fake.SNAPSHOT['volume_id'],
             'snapshot': fake.SNAPSHOT['name'],
         }
-        self.mock_object(self.client, 'send_request')
+        self.mock_object(self.client.connection, 'send_request')
 
         self.client.delete_snapshot(api_args['volume'],
                                     api_args['snapshot'])
@@ -524,12 +670,12 @@ class NetAppBaseClientTestCase(test.TestCase):
             'volume': api_args['volume'],
             'snapshot': api_args['snapshot'],
         }
-        self.client.send_request.assert_called_once_with('snapshot-delete',
-                                                         asserted_api_args)
+        self.client.connection.send_request.assert_called_once_with(
+            'snapshot-delete', asserted_api_args)
 
     def test_create_cg_snapshot(self):
-        self.mock_object(self.client, '_start_cg_snapshot', mock.Mock(
-            return_value=fake.CONSISTENCY_GROUP_ID))
+        self.mock_object(self.client, '_start_cg_snapshot',
+                         return_value=fake.CONSISTENCY_GROUP_ID)
         self.mock_object(self.client, '_commit_cg_snapshot')
 
         self.client.create_cg_snapshot([fake.CG_VOLUME_NAME],
@@ -539,8 +685,7 @@ class NetAppBaseClientTestCase(test.TestCase):
             fake.CONSISTENCY_GROUP_ID)
 
     def test_create_cg_snapshot_no_id(self):
-        self.mock_object(self.client, '_start_cg_snapshot', mock.Mock(
-            return_value=None))
+        self.mock_object(self.client, '_start_cg_snapshot', return_value=None)
 
         self.assertRaises(exception.VolumeBackendAPIException,
                           self.client.create_cg_snapshot,
@@ -553,21 +698,21 @@ class NetAppBaseClientTestCase(test.TestCase):
             'timeout': 'relaxed',
             'volumes': [{'volume-name': fake.CG_VOLUME_NAME}],
         }
-        self.mock_object(self.client, 'send_request')
+        self.mock_object(self.client.connection, 'send_request')
 
         self.client._start_cg_snapshot([fake.CG_VOLUME_NAME],
                                        snapshot_init['snapshot'])
 
-        self.client.send_request.assert_called_once_with('cg-start',
-                                                         snapshot_init)
+        self.client.connection.send_request.assert_called_once_with(
+            'cg-start', snapshot_init)
 
     def test_commit_cg_snapshot(self):
         snapshot_commit = {'cg-id': fake.CG_VOLUME_ID}
-        self.mock_object(self.client, 'send_request')
+        self.mock_object(self.client.connection, 'send_request')
 
         self.client._commit_cg_snapshot(snapshot_commit['cg-id'])
 
-        self.client.send_request.assert_called_once_with(
+        self.client.connection.send_request.assert_called_once_with(
             'cg-commit', {'cg-id': snapshot_commit['cg-id']})
 
     def test_wait_for_busy_snapshot_raise_exception(self):
@@ -576,10 +721,8 @@ class NetAppBaseClientTestCase(test.TestCase):
 
         # Need to mock sleep as it is called by @utils.retry
         self.mock_object(time, 'sleep')
-        mock_get_snapshot = self.mock_object(
-            self.client, 'get_snapshot',
-            mock.Mock(return_value=BUSY_SNAPSHOT)
-        )
+        mock_get_snapshot = self.mock_object(self.client, 'get_snapshot',
+                                             return_value=BUSY_SNAPSHOT)
 
         self.assertRaises(exception.SnapshotIsBusy,
                           self.client.wait_for_busy_snapshot,
@@ -593,7 +736,7 @@ class NetAppBaseClientTestCase(test.TestCase):
         mock_get_snapshot.assert_has_calls(calls)
 
     def test_rename_snapshot(self):
-        self.mock_object(self.client, 'send_request')
+        self.mock_object(self.client.connection, 'send_request')
 
         self.client.rename_snapshot(
             fake.SNAPSHOT['volume_id'], fake.SNAPSHOT_NAME,
@@ -606,5 +749,5 @@ class NetAppBaseClientTestCase(test.TestCase):
                 client_base.DELETED_PREFIX + fake.SNAPSHOT_NAME,
         }
 
-        self.client.send_request.assert_called_once_with(
+        self.client.connection.send_request.assert_called_once_with(
             'snapshot-rename', api_args)
