@@ -18,15 +18,13 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-Mock unit tests for the NetApp block storage library
-"""
+"""Mock unit tests for the NetApp block storage library"""
 
 import copy
+from unittest import mock
 import uuid
 
 import ddt
-import mock
 from oslo_log import versionutils
 from oslo_utils import units
 import six
@@ -34,15 +32,15 @@ import six
 from cinder import context
 from cinder import exception
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.netapp.dataontap import fakes as fake
 import cinder.tests.unit.volume.drivers.netapp.fakes as na_fakes
 from cinder.volume.drivers.netapp.dataontap import block_base
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
 from cinder.volume.drivers.netapp import utils as na_utils
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 
 @ddt.ddt
@@ -119,7 +117,8 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
     def test_create_volume(self):
         volume_size_in_bytes = int(fake.SIZE) * units.Gi
-        self.mock_object(na_utils, 'get_volume_extra_specs')
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         return_value={})
         self.mock_object(na_utils, 'log_extra_spec_warnings')
         self.mock_object(block_base, 'LOG')
         self.mock_object(volume_utils, 'extract_host',
@@ -136,7 +135,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
         self.library._create_lun.assert_called_once_with(
             fake.POOL_NAME, fake.LUN_NAME, volume_size_in_bytes,
-            fake.LUN_METADATA, None)
+            fake.LUN_METADATA, None, False)
         self.library._get_volume_model_update.assert_called_once_with(
             fake.VOLUME)
         self.assertEqual(
@@ -260,6 +259,33 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                           'fake_volume', fake.FC_FORMATTED_INITIATORS,
                           protocol, None)
 
+    def test__is_multiattached_true(self):
+        volume = copy.deepcopy(fake.test_volume)
+        volume.multiattach = True
+        volume.volume_attachment = [
+            fake.test_iscsi_attachment,
+            fake.test_iscsi_attachment,
+        ]
+
+        self.assertTrue(self.library._is_multiattached(
+            volume,
+            fake.ISCSI_CONNECTOR))
+
+    def test__is_multiattached_false(self):
+        volume1 = copy.deepcopy(fake.test_volume)
+        volume1.multiattach = True
+        volume1.volume_attachment = []
+        volume2 = copy.deepcopy(fake.test_volume)
+        volume2.multiattach = False
+        volume2.volume_attachment = []
+
+        self.assertFalse(self.library._is_multiattached(
+            volume1,
+            fake.ISCSI_CONNECTOR))
+        self.assertFalse(self.library._is_multiattached(
+            volume2,
+            fake.ISCSI_CONNECTOR))
+
     @mock.patch.object(block_base.NetAppBlockStorageLibrary,
                        '_find_mapped_lun_igroup')
     def test_unmap_lun_empty(self, mock_find_mapped_lun_igroup):
@@ -302,6 +328,50 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                            fake.ISCSI_MULTI_MAP_LIST[1]['initiator-group'])]
         self.zapi_client.unmap_lun.assert_has_calls(calls)
 
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, '_unmap_lun')
+    def test_terminate_connection_iscsi_multiattach(self, mock_unmap_lun):
+        volume = copy.deepcopy(fake.test_volume)
+        volume.multiattach = True
+        volume.volume_attachment = [
+            fake.test_iscsi_attachment,
+            fake.test_iscsi_attachment,
+        ]
+
+        self.library.terminate_connection_iscsi(volume, fake.ISCSI_CONNECTOR)
+
+        mock_unmap_lun.assert_not_called()
+
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, '_unmap_lun')
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, '_get_lun_attr')
+    def test_terminate_connection_iscsi_last_attachment(self,
+                                                        mock_get_lun_attr,
+                                                        mock_unmap_lun):
+        mock_get_lun_attr.return_value = {'Path': fake.PATH}
+        volume = copy.deepcopy(fake.test_volume)
+        volume.multiattach = True
+        volume.volume_attachment = [fake.test_iscsi_attachment]
+
+        self.library.terminate_connection_iscsi(volume, fake.ISCSI_CONNECTOR)
+
+        mock_unmap_lun.assert_called_once_with(
+            fake.PATH, [fake.ISCSI_CONNECTOR['initiator']])
+
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, '_unmap_lun')
+    @mock.patch.object(block_base.NetAppBlockStorageLibrary, '_get_lun_attr')
+    def test_terminate_connection_iscsi_all_initiators(self, mock_get_lun_attr,
+                                                       mock_unmap_lun):
+        mock_get_lun_attr.return_value = {'Path': fake.PATH}
+        volume = copy.deepcopy(fake.test_volume)
+        volume.multiattach = True
+        volume.volume_attachment = [
+            fake.test_iscsi_attachment,
+            fake.test_iscsi_attachment,
+        ]
+
+        self.library.terminate_connection_iscsi(volume, None)
+
+        mock_unmap_lun.assert_called_once_with(fake.PATH, [])
+
     def test_find_mapped_lun_igroup(self):
         self.assertRaises(NotImplementedError,
                           self.library._find_mapped_lun_igroup,
@@ -844,7 +914,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.library.configuration.netapp_lun_ostype = 'unknown'
         self.library.do_setup(mock.Mock())
 
-        self.assertRaises(exception.NetAppDriverException,
+        self.assertRaises(na_utils.NetAppDriverException,
                           self.library.check_for_setup_error)
 
         block_base.LOG.error.assert_called_once_with(mock.ANY)
@@ -856,7 +926,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.library.configuration.netapp_host_type = 'future_os'
         self.library.do_setup(mock.Mock())
 
-        self.assertRaises(exception.NetAppDriverException,
+        self.assertRaises(na_utils.NetAppDriverException,
                           self.library.check_for_setup_error)
 
         block_base.LOG.error.assert_called_once_with(mock.ANY)
@@ -955,7 +1025,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.mock_object(self.library.zapi_client, 'destroy_lun',
                          side_effect=netapp_api.NaApiError)
 
-        self.assertRaises(exception.NetAppDriverException,
+        self.assertRaises(na_utils.NetAppDriverException,
                           self.library._delete_lun,
                           fake.LUN_NAME)
 
@@ -993,7 +1063,8 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.library._clone_lun.assert_called_once_with(
             fake.CLONE_SOURCE_NAME, fake.CLONE_DESTINATION_NAME,
             space_reserved='false',
-            qos_policy_group_name=fake.QOS_POLICY_GROUP_NAME)
+            qos_policy_group_name=fake.QOS_POLICY_GROUP_NAME,
+            qos_policy_group_is_adaptive=False)
         self.library._extend_volume.assert_called_once_with(
             fake.CLONE_DESTINATION, fake.CLONE_DESTINATION_SIZE,
             fake.QOS_POLICY_GROUP_NAME)
@@ -1023,7 +1094,8 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.library._clone_lun.assert_called_once_with(
             fake.CLONE_SOURCE_NAME, fake.CLONE_DESTINATION_NAME,
             space_reserved='true',
-            qos_policy_group_name=fake.QOS_POLICY_GROUP_NAME)
+            qos_policy_group_name=fake.QOS_POLICY_GROUP_NAME,
+            qos_policy_group_is_adaptive=False)
         self.library._extend_volume.assert_called_once_with(
             fake.CLONE_DESTINATION, fake.CLONE_DESTINATION_SIZE,
             fake.QOS_POLICY_GROUP_NAME)
@@ -1134,7 +1206,8 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         mock_extend_volume.assert_called_once_with(
             fake.VOLUME, new_size, fake.QOS_POLICY_GROUP_NAME)
 
-    def test__extend_volume_direct(self):
+    @ddt.data('9.4', '9.6')
+    def test__extend_volume_direct(self, ontap_version):
 
         current_size = fake.LUN_SIZE
         current_size_bytes = current_size * units.Gi
@@ -1143,6 +1216,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         max_size = fake.LUN_SIZE * 10
         max_size_bytes = max_size * units.Gi
 
+        mock_get_ontap_version = self.mock_object(
+            self.library.zapi_client, 'get_ontap_version',
+            return_value=ontap_version)
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
                                         current_size_bytes,
@@ -1161,16 +1237,23 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
         self.library._extend_volume(fake.VOLUME, new_size, 'fake_qos_policy')
 
+        mock_get_ontap_version.assert_called_once_with(cached=True)
         mock_get_lun_from_table.assert_called_once_with(fake.VOLUME['name'])
-        mock_get_lun_geometry.assert_called_once_with(
-            fake.LUN_METADATA['Path'])
+
+        if ontap_version < '9.5':
+            mock_get_lun_geometry.assert_called_once_with(
+                fake.LUN_METADATA['Path'])
+        else:
+            mock_get_lun_geometry.assert_not_called()
+
         mock_do_direct_resize.assert_called_once_with(
             fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
         self.assertFalse(mock_do_sub_clone_resize.called)
         self.assertEqual(six.text_type(new_size_bytes),
                          self.library.lun_table[fake.VOLUME['name']].size)
 
-    def test__extend_attached_volume_direct(self):
+    @ddt.data('9.4', '9.6')
+    def test__extend_attached_volume_direct(self, ontap_version):
 
         current_size = fake.LUN_SIZE
         current_size_bytes = current_size * units.Gi
@@ -1182,6 +1265,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         volume_copy['size'] = new_size
         volume_copy['attach_status'] = fake.ATTACHED
 
+        mock_get_ontap_version = self.mock_object(
+            self.library.zapi_client, 'get_ontap_version',
+            return_value=ontap_version)
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
                                         current_size_bytes,
@@ -1196,20 +1282,27 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                                  'do_direct_resize')
         mock_do_sub_clone_resize = self.mock_object(self.library,
                                                     '_do_sub_clone_resize')
-        self.library.lun_table = {volume_copy['name']: fake_lun}
 
+        self.library.lun_table = {volume_copy['name']: fake_lun}
         self.library._extend_volume(volume_copy, new_size, 'fake_qos_policy')
 
         mock_get_lun_from_table.assert_called_once_with(volume_copy['name'])
-        mock_get_lun_geometry.assert_called_once_with(
-            fake.LUN_METADATA['Path'])
+        mock_get_ontap_version.assert_called_once_with(cached=True)
+
+        if ontap_version < '9.5':
+            mock_get_lun_geometry.assert_called_once_with(
+                fake.LUN_METADATA['Path'])
+        else:
+            mock_get_lun_geometry.assert_not_called()
+
         mock_do_direct_resize.assert_called_once_with(
             fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
         self.assertFalse(mock_do_sub_clone_resize.called)
         self.assertEqual(six.text_type(new_size_bytes),
                          self.library.lun_table[volume_copy['name']].size)
 
-    def test__extend_volume_clone(self):
+    @ddt.data('9.4', '9.6')
+    def test__extend_volume_clone(self, ontap_version):
 
         current_size = fake.LUN_SIZE
         current_size_bytes = current_size * units.Gi
@@ -1218,6 +1311,9 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         max_size = fake.LUN_SIZE * 10
         max_size_bytes = max_size * units.Gi
 
+        mock_get_ontap_version = self.mock_object(
+            self.library.zapi_client, 'get_ontap_version',
+            return_value=ontap_version)
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
                                         current_size_bytes,
@@ -1236,29 +1332,43 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
 
         self.library._extend_volume(fake.VOLUME, new_size, 'fake_qos_policy')
 
+        mock_get_ontap_version.assert_called_once_with(cached=True)
         mock_get_lun_from_table.assert_called_once_with(fake.VOLUME['name'])
-        mock_get_lun_geometry.assert_called_once_with(
-            fake.LUN_METADATA['Path'])
-        self.assertFalse(mock_do_direct_resize.called)
-        mock_do_sub_clone_resize.assert_called_once_with(
-            fake.LUN_METADATA['Path'], six.text_type(new_size_bytes),
-            qos_policy_group_name='fake_qos_policy')
+
+        if ontap_version < '9.5':
+            self.assertFalse(mock_do_direct_resize.called)
+            mock_get_lun_geometry.assert_called_once_with(
+                fake.LUN_METADATA['Path'])
+            mock_do_sub_clone_resize.assert_called_once_with(
+                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes),
+                qos_policy_group_name='fake_qos_policy')
+        else:
+            mock_get_lun_geometry.assert_not_called()
+            mock_do_sub_clone_resize.assert_not_called()
+            mock_do_direct_resize.assert_called_once_with(
+                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+
         self.assertEqual(six.text_type(new_size_bytes),
                          self.library.lun_table[fake.VOLUME['name']].size)
 
-    def test__extend_attached_volume_clone_error(self):
+    @ddt.data('9.4', '9.6')
+    def test__extend_attached_volume_clone_error(self, ontap_version):
 
         current_size = fake.LUN_SIZE
         current_size_bytes = current_size * units.Gi
         new_size = fake.LUN_SIZE * 20
+        new_size_bytes = new_size * units.Gi
         max_size = fake.LUN_SIZE * 10
         max_size_bytes = max_size * units.Gi
         volume_copy = copy.copy(fake.VOLUME)
         volume_copy['attach_status'] = fake.ATTACHED
 
+        mock_get_ontap_version = self.mock_object(
+            self.library.zapi_client, 'get_ontap_version',
+            return_value=ontap_version)
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
-                                        current_size_bytes,
+                                        six.text_type(current_size_bytes),
                                         fake.LUN_METADATA)
         mock_get_lun_from_table = self.mock_object(
             self.library, '_get_lun_from_table', return_value=fake_lun)
@@ -1272,21 +1382,35 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
                                                     '_do_sub_clone_resize')
         self.library.lun_table = {volume_copy['name']: fake_lun}
 
-        self.assertRaises(exception.VolumeBackendAPIException,
-                          self.library._extend_volume,
-                          volume_copy,
-                          new_size,
-                          fake.QOS_POLICY_GROUP_NAME)
+        # (throne82) This error occurs only with versions older than 9.5
+        if ontap_version < '9.5':
+            self.assertRaises(exception.VolumeBackendAPIException,
+                              self.library._extend_volume,
+                              volume_copy,
+                              new_size,
+                              fake.QOS_POLICY_GROUP_NAME)
+            self.assertFalse(mock_do_direct_resize.called)
+            self.assertFalse(mock_do_sub_clone_resize.called)
+            mock_get_lun_geometry.assert_called_once_with(
+                fake.LUN_METADATA['Path'])
+            self.assertEqual(six.text_type(current_size_bytes),
+                             self.library.lun_table[volume_copy['name']].size)
+        else:
+            self.library._extend_volume(volume_copy,
+                                        new_size, fake.QOS_POLICY_GROUP_NAME)
+            mock_do_direct_resize.assert_called_once_with(
+                fake.LUN_METADATA['Path'], six.text_type(new_size_bytes))
+            mock_do_sub_clone_resize.assert_not_called()
+            mock_get_lun_geometry.assert_not_called()
+            self.assertEqual(six.text_type(new_size_bytes),
+                             self.library.lun_table[volume_copy['name']].size)
 
-        mock_get_lun_from_table.assert_called_once_with(volume_copy['name'])
-        mock_get_lun_geometry.assert_called_once_with(
-            fake.LUN_METADATA['Path'])
-        self.assertFalse(mock_do_direct_resize.called)
-        self.assertFalse(mock_do_sub_clone_resize.called)
-        self.assertEqual(current_size_bytes,
-                         self.library.lun_table[volume_copy['name']].size)
+        mock_get_ontap_version.assert_called_once_with(cached=True)
+        mock_get_lun_from_table.assert_called_once_with(
+            volume_copy['name'])
 
-    def test__extend_volume_no_change(self):
+    @ddt.data('9.4', '9.6')
+    def test__extend_volume_no_change(self, ontap_version):
 
         current_size = fake.LUN_SIZE
         current_size_bytes = current_size * units.Gi
@@ -1296,6 +1420,8 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         volume_copy = copy.copy(fake.VOLUME)
         volume_copy['size'] = new_size
 
+        mock_get_ontap_version = self.mock_object(
+            self.library.zapi_client, 'get_ontap_version')
         fake_lun = block_base.NetAppLun(fake.LUN_HANDLE,
                                         fake.LUN_ID,
                                         current_size_bytes,
@@ -1318,6 +1444,7 @@ class NetAppBlockStorageLibraryTestCase(test.TestCase):
         self.assertFalse(mock_get_lun_geometry.called)
         self.assertFalse(mock_do_direct_resize.called)
         self.assertFalse(mock_do_sub_clone_resize.called)
+        self.assertFalse(mock_get_ontap_version.called)
 
     def test_do_sub_clone_resize(self):
 
