@@ -14,15 +14,15 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-Mock unit tests for the NetApp block storage C-mode library
-"""
+"""Mock unit tests for the NetApp block storage C-mode library."""
+
+from unittest import mock
 
 import ddt
-import mock
 
 from cinder import exception
-from cinder import test
+from cinder.objects import fields
+from cinder.tests.unit import test
 import cinder.tests.unit.volume.drivers.netapp.dataontap.fakes as fake
 from cinder.tests.unit.volume.drivers.netapp.dataontap.utils import fakes as\
     fake_utils
@@ -32,10 +32,12 @@ from cinder.volume.drivers.netapp.dataontap import block_cmode
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.client import client_base
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
+from cinder.volume.drivers.netapp.dataontap.utils import capabilities
 from cinder.volume.drivers.netapp.dataontap.utils import data_motion
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
-from cinder.volume.drivers.netapp.dataontap.utils import utils as config_utils
+from cinder.volume.drivers.netapp.dataontap.utils import utils as dot_utils
 from cinder.volume.drivers.netapp import utils as na_utils
+from cinder.volume import volume_utils
 
 
 @ddt.ddt
@@ -68,9 +70,6 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         }
         self.mock_object(block_base.NetAppBlockStorageLibrary, 'delete_volume')
 
-    def tearDown(self):
-        super(NetAppBlockStorageCmodeLibraryTestCase, self).tearDown()
-
     def get_config_cmode(self):
         config = na_fakes.create_configuration_cmode()
         config.netapp_storage_protocol = 'iscsi'
@@ -80,42 +79,47 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         config.netapp_transport_type = 'https'
         config.netapp_server_port = '443'
         config.netapp_vserver = 'openstack'
+        config.netapp_api_trace_pattern = 'fake_regex'
         return config
 
     @mock.patch.object(perf_cmode, 'PerformanceCmodeLibrary', mock.Mock())
     @mock.patch.object(client_base.Client, 'get_ontapi_version',
                        mock.MagicMock(return_value=(1, 20)))
+    @mock.patch.object(capabilities.CapabilitiesLibrary,
+                       'cluster_user_supported')
+    @mock.patch.object(capabilities.CapabilitiesLibrary,
+                       'check_api_permissions')
     @mock.patch.object(na_utils, 'check_flags')
     @mock.patch.object(block_base.NetAppBlockStorageLibrary, 'do_setup')
-    def test_do_setup(self, super_do_setup, mock_check_flags):
+    @mock.patch.object(client_base.Client, 'get_ontap_version',
+                       mock.MagicMock(return_value='9.6'))
+    def test_do_setup(self, super_do_setup, mock_check_flags,
+                      mock_check_api_permissions, mock_cluster_user_supported):
         self.mock_object(client_base.Client, '_init_ssh_client')
         self.mock_object(
-            config_utils, 'get_backend_configuration',
-            mock.Mock(return_value=self.get_config_cmode()))
+            dot_utils, 'get_backend_configuration',
+            return_value=self.get_config_cmode())
         context = mock.Mock()
 
         self.library.do_setup(context)
 
         super_do_setup.assert_called_once_with(context)
         self.assertEqual(1, mock_check_flags.call_count)
+        mock_check_api_permissions.assert_called_once_with()
+        mock_cluster_user_supported.assert_called_once_with()
 
     def test_check_for_setup_error(self):
         super_check_for_setup_error = self.mock_object(
             block_base.NetAppBlockStorageLibrary, 'check_for_setup_error')
-        mock_check_api_permissions = self.mock_object(
-            self.library.ssc_library, 'check_api_permissions')
-        mock_add_looping_tasks = self.mock_object(
-            self.library, '_add_looping_tasks')
         mock_get_pool_map = self.mock_object(
             self.library, '_get_flexvol_to_pool_map',
-            mock.Mock(return_value={'fake_map': None}))
+            return_value={'fake_map': None})
         mock_add_looping_tasks = self.mock_object(
             self.library, '_add_looping_tasks')
 
         self.library.check_for_setup_error()
 
         self.assertEqual(1, super_check_for_setup_error.call_count)
-        mock_check_api_permissions.assert_called_once_with()
         self.assertEqual(1, mock_add_looping_tasks.call_count)
         mock_get_pool_map.assert_called_once_with()
         mock_add_looping_tasks.assert_called_once_with()
@@ -123,34 +127,39 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
     def test_check_for_setup_error_no_filtered_pools(self):
         self.mock_object(block_base.NetAppBlockStorageLibrary,
                          'check_for_setup_error')
-        mock_check_api_permissions = self.mock_object(
-            self.library.ssc_library, 'check_api_permissions')
         self.mock_object(self.library, '_add_looping_tasks')
         self.mock_object(
-            self.library, '_get_flexvol_to_pool_map',
-            mock.Mock(return_value={}))
+            self.library, '_get_flexvol_to_pool_map', return_value={})
 
-        self.assertRaises(exception.NetAppDriverException,
+        self.assertRaises(na_utils.NetAppDriverException,
                           self.library.check_for_setup_error)
 
-        mock_check_api_permissions.assert_called_once_with()
-
-    @ddt.data({'replication_enabled': True, 'failed_over': False},
-              {'replication_enabled': True, 'failed_over': True},
-              {'replication_enabled': False, 'failed_over': False})
+    @ddt.data({'replication_enabled': True, 'failed_over': False,
+               'cluster_credentials': True},
+              {'replication_enabled': True, 'failed_over': True,
+               'cluster_credentials': True},
+              {'replication_enabled': False, 'failed_over': False,
+               'cluster_credentials': False})
     @ddt.unpack
-    def test_handle_housekeeping_tasks(self, replication_enabled, failed_over):
+    def test_handle_housekeeping_tasks(
+            self, replication_enabled, failed_over, cluster_credentials):
+        self.library.using_cluster_credentials = cluster_credentials
         ensure_mirrors = self.mock_object(data_motion.DataMotionMixin,
                                           'ensure_snapmirrors')
         self.mock_object(self.library.ssc_library, 'get_ssc_flexvol_names',
-                         mock.Mock(return_value=fake_utils.SSC.keys()))
+                         return_value=fake_utils.SSC.keys())
+        mock_remove_unused_qos_policy_groups = self.mock_object(
+            self.zapi_client, 'remove_unused_qos_policy_groups')
         self.library.replication_enabled = replication_enabled
         self.library.failed_over = failed_over
 
         self.library._handle_housekeeping_tasks()
 
-        (self.zapi_client.remove_unused_qos_policy_groups.
-         assert_called_once_with())
+        if self.library.using_cluster_credentials:
+            mock_remove_unused_qos_policy_groups.assert_called_once_with()
+        else:
+            mock_remove_unused_qos_policy_groups.assert_not_called()
+
         if replication_enabled and not failed_over:
             ensure_mirrors.assert_called_once_with(
                 self.library.configuration, self.library.backend_name,
@@ -158,6 +167,32 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         else:
             self.assertFalse(ensure_mirrors.called)
 
+    def test_handle_ems_logging(self):
+        volume_list = ['vol0', 'vol1', 'vol2']
+        self.mock_object(
+            self.library.ssc_library, 'get_ssc_flexvol_names',
+            return_value=volume_list)
+        self.mock_object(
+            dot_utils, 'build_ems_log_message_0',
+            return_value='fake_base_ems_log_message')
+        self.mock_object(
+            dot_utils, 'build_ems_log_message_1',
+            return_value='fake_pool_ems_log_message')
+        mock_send_ems_log_message = self.mock_object(
+            self.zapi_client, 'send_ems_log_message')
+
+        self.library._handle_ems_logging()
+
+        mock_send_ems_log_message.assert_has_calls([
+            mock.call('fake_base_ems_log_message'),
+            mock.call('fake_pool_ems_log_message'),
+        ])
+        dot_utils.build_ems_log_message_0.assert_called_once_with(
+            self.library.driver_name, self.library.app_version)
+        dot_utils.build_ems_log_message_1.assert_called_once_with(
+            self.library.driver_name, self.library.app_version,
+            self.library.vserver, volume_list, [])
+
     def test_find_mapped_lun_igroup(self):
         igroups = [fake.IGROUP1]
         self.zapi_client.get_igroup_by_initiators.return_value = igroups
@@ -232,14 +267,14 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         lun = fake.FAKE_LUN
         self.library._get_lun_by_args = mock.Mock(return_value=[lun])
         self.library._add_lun_to_table = mock.Mock()
-        self.library._update_stale_vols = mock.Mock()
 
         self.library._clone_lun('fakeLUN', 'newFakeLUN', 'false')
 
         self.library.zapi_client.clone_lun.assert_called_once_with(
             'fakeLUN', 'fakeLUN', 'newFakeLUN', 'false', block_count=0,
             dest_block=0, src_block=0, qos_policy_group_name=None,
-            source_snapshot=None, is_snapshot=False)
+            qos_policy_group_is_adaptive=False, source_snapshot=None,
+            is_snapshot=False)
 
     def test_clone_lun_blocks(self):
         """Test for when clone lun is passed block information."""
@@ -255,7 +290,6 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         lun = fake.FAKE_LUN
         self.library._get_lun_by_args = mock.Mock(return_value=[lun])
         self.library._add_lun_to_table = mock.Mock()
-        self.library._update_stale_vols = mock.Mock()
 
         self.library._clone_lun('fakeLUN', 'newFakeLUN', 'false',
                                 block_count=block_count, src_block=src_block,
@@ -265,6 +299,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             'fakeLUN', 'fakeLUN', 'newFakeLUN', 'false',
             block_count=block_count, dest_block=dest_block,
             src_block=src_block, qos_policy_group_name=None,
+            qos_policy_group_is_adaptive=False,
             source_snapshot=None, is_snapshot=False)
 
     def test_clone_lun_no_space_reservation(self):
@@ -279,13 +314,13 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         lun = fake.FAKE_LUN
         self.library._get_lun_by_args = mock.Mock(return_value=[lun])
         self.library._add_lun_to_table = mock.Mock()
-        self.library._update_stale_vols = mock.Mock()
 
         self.library._clone_lun('fakeLUN', 'newFakeLUN', is_snapshot=True)
 
         self.library.zapi_client.clone_lun.assert_called_once_with(
             'fakeLUN', 'fakeLUN', 'newFakeLUN', 'false', block_count=0,
             dest_block=0, src_block=0, qos_policy_group_name=None,
+            qos_policy_group_is_adaptive=False,
             source_snapshot=None, is_snapshot=True)
 
     def test_get_fc_target_wwpns(self):
@@ -297,39 +332,20 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
         self.assertSetEqual(set(ports), set(result))
 
-    @mock.patch.object(block_cmode.NetAppBlockStorageCmodeLibrary,
-                       '_get_pool_stats', mock.Mock())
-    def test_vol_stats_calls_provide_ems(self):
-        self.library.zapi_client.provide_ems = mock.Mock()
-
-        self.library.get_volume_stats(refresh=True)
-
-        self.assertEqual(1, self.library.zapi_client.provide_ems.call_count)
-
     def test_create_lun(self):
         self.library._create_lun(
             fake.VOLUME_ID, fake.LUN_ID, fake.LUN_SIZE, fake.LUN_METADATA)
 
         self.library.zapi_client.create_lun.assert_called_once_with(
             fake.VOLUME_ID, fake.LUN_ID, fake.LUN_SIZE, fake.LUN_METADATA,
-            None)
-
-    def test_get_preferred_target_from_list(self):
-        target_details_list = fake.ISCSI_TARGET_DETAILS_LIST
-        operational_addresses = [
-            target['address']
-            for target in target_details_list[2:]]
-        self.zapi_client.get_operational_lif_addresses = (
-            mock.Mock(return_value=operational_addresses))
-
-        result = self.library._get_preferred_target_from_list(
-            target_details_list)
-
-        self.assertEqual(target_details_list[2], result)
-
-    @ddt.data([], ['target_1', 'target_2'])
-    def test_get_pool_stats(self, replication_backends):
+            None, False)
 
+    @ddt.data({'replication_backends': [], 'cluster_credentials': False},
+              {'replication_backends': ['target_1', 'target_2'],
+               'cluster_credentials': True})
+    @ddt.unpack
+    def test_get_pool_stats(self, replication_backends, cluster_credentials):
+        self.library.using_cluster_credentials = cluster_credentials
         ssc = {
             'vola': {
                 'pool_name': 'vola',
@@ -346,12 +362,12 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         }
         mock_get_ssc = self.mock_object(self.library.ssc_library,
                                         'get_ssc',
-                                        mock.Mock(return_value=ssc))
+                                        return_value=ssc)
         mock_get_aggrs = self.mock_object(self.library.ssc_library,
                                           'get_ssc_aggregates',
-                                          mock.Mock(return_value=['aggr1']))
+                                          return_value=['aggr1'])
         self.mock_object(self.library, 'get_replication_backend_names',
-                         mock.Mock(return_value=replication_backends))
+                         return_value=replication_backends)
 
         self.library.reserved_percentage = 5
         self.library.max_over_subscription_ratio = 10
@@ -361,9 +377,12 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             'size-total': 10737418240.0,
             'size-available': 2147483648.0,
         }
-        self.mock_object(
-            self.zapi_client, 'get_flexvol_capacity',
-            mock.Mock(return_value=mock_capacities))
+        self.mock_object(self.zapi_client,
+                         'get_flexvol_capacity',
+                         return_value=mock_capacities)
+        self.mock_object(self.zapi_client,
+                         'get_flexvol_dedupe_used_percent',
+                         return_value=55.0)
 
         aggr_capacities = {
             'aggr1': {
@@ -374,7 +393,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         }
         mock_get_aggr_capacities = self.mock_object(
             self.zapi_client, 'get_aggregate_capacities',
-            mock.Mock(return_value=aggr_capacities))
+            return_value=aggr_capacities)
 
         result = self.library._get_pool_stats(filter_function='filter',
                                               goodness_function='goodness')
@@ -383,12 +402,13 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             'pool_name': 'vola',
             'QoS_support': True,
             'consistencygroup_support': True,
+            'consistent_group_snapshot_enabled': True,
             'reserved_percentage': 5,
             'max_over_subscription_ratio': 10.0,
             'multiattach': True,
             'total_capacity_gb': 10.0,
             'free_capacity_gb': 2.0,
-            'provisioned_capacity_gb': 8.0,
+            'netapp_dedupe_used_percent': 55.0,
             'netapp_aggregate_used_percent': 45,
             'utilization': 30.0,
             'filter_function': 'filter',
@@ -403,7 +423,16 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             'netapp_raid_type': 'raid_dp',
             'netapp_disk_type': 'SSD',
             'replication_enabled': False,
+            'online_extend_support': True,
         }]
+
+        expected[0].update({'QoS_support': cluster_credentials})
+        if not cluster_credentials:
+            expected[0].update({
+                'netapp_aggregate_used_percent': 0,
+                'netapp_dedupe_used_percent': 0
+            })
+
         if replication_backends:
             expected[0].update({
                 'replication_enabled': True,
@@ -414,29 +443,30 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
         self.assertEqual(expected, result)
         mock_get_ssc.assert_called_once_with()
-        mock_get_aggrs.assert_called_once_with()
-        mock_get_aggr_capacities.assert_called_once_with(['aggr1'])
+        if cluster_credentials:
+            mock_get_aggrs.assert_called_once_with()
+            mock_get_aggr_capacities.assert_called_once_with(['aggr1'])
 
     @ddt.data({}, None)
     def test_get_pool_stats_no_ssc_vols(self, ssc):
 
         mock_get_ssc = self.mock_object(self.library.ssc_library,
                                         'get_ssc',
-                                        mock.Mock(return_value=ssc))
+                                        return_value=ssc)
 
         pools = self.library._get_pool_stats()
 
         self.assertListEqual([], pools)
         mock_get_ssc.assert_called_once_with()
 
-    @ddt.data('open+|demix+', 'open.+', '.+\d', '^((?!mix+).)*$',
+    @ddt.data(r'open+|demix+', 'open.+', r'.+\d', '^((?!mix+).)*$',
               'open123, open321')
     def test_get_pool_map_match_selected_pools(self, patterns):
 
         self.library.configuration.netapp_pool_name_search_pattern = patterns
         mock_list_flexvols = self.mock_object(
             self.zapi_client, 'list_flexvols',
-            mock.Mock(return_value=fake.FAKE_CMODE_VOLUMES))
+            return_value=fake.FAKE_CMODE_VOLUMES)
 
         result = self.library._get_flexvol_to_pool_map()
 
@@ -458,7 +488,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library.configuration.netapp_pool_name_search_pattern = patterns
         mock_list_flexvols = self.mock_object(
             self.zapi_client, 'list_flexvols',
-            mock.Mock(return_value=fake.FAKE_CMODE_VOLUMES))
+            return_value=fake.FAKE_CMODE_VOLUMES)
 
         result = self.library._get_flexvol_to_pool_map()
 
@@ -479,7 +509,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library.configuration.netapp_pool_name_search_pattern = patterns
         mock_list_flexvols = self.mock_object(
             self.zapi_client, 'list_flexvols',
-            mock.Mock(return_value=fake.FAKE_CMODE_VOLUMES))
+            return_value=fake.FAKE_CMODE_VOLUMES)
 
         result = self.library._get_flexvol_to_pool_map()
 
@@ -490,7 +520,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
         mock_get_pool_map = self.mock_object(
             self.library, '_get_flexvol_to_pool_map',
-            mock.Mock(return_value=fake.FAKE_CMODE_VOLUMES))
+            return_value=fake.FAKE_CMODE_VOLUMES)
 
         result = self.library._update_ssc()
 
@@ -501,8 +531,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_delete_volume(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(
-                             return_value=fake.QOS_POLICY_GROUP_INFO))
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
         self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
 
         self.library.delete_volume(fake.VOLUME)
@@ -516,7 +545,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_delete_volume_get_valid_qos_policy_group_info_exception(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(side_effect=exception.Invalid))
+                         side_effect=exception.Invalid)
         self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
 
         self.library.delete_volume(fake.VOLUME)
@@ -528,8 +557,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_setup_qos_for_volume(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(
-                             return_value=fake.QOS_POLICY_GROUP_INFO))
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
         self.mock_object(self.zapi_client, 'provision_qos_policy_group')
 
         result = self.library._setup_qos_for_volume(fake.VOLUME,
@@ -541,8 +569,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_setup_qos_for_volume_exception_path(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(
-                             side_effect=exception.Invalid))
+                         side_effect=exception.Invalid)
         self.mock_object(self.zapi_client, 'provision_qos_policy_group')
 
         self.assertRaises(exception.VolumeBackendAPIException,
@@ -565,7 +592,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_unmanage(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(return_value=fake.QOS_POLICY_GROUP_INFO))
+                         return_value=fake.QOS_POLICY_GROUP_INFO)
         self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
         self.mock_object(block_base.NetAppBlockStorageLibrary, 'unmanage')
 
@@ -580,7 +607,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_unmanage_w_invalid_qos_policy(self):
         self.mock_object(na_utils, 'get_valid_qos_policy_group_info',
-                         mock.Mock(side_effect=exception.Invalid))
+                         side_effect=exception.Invalid)
         self.mock_object(self.library, '_mark_qos_policy_group_for_deletion')
         self.mock_object(block_base.NetAppBlockStorageLibrary, 'unmanage')
 
@@ -603,7 +630,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library._check_volume_type_for_lun = mock.Mock()
         self.library._setup_qos_for_volume = mock.Mock()
         self.mock_object(na_utils, 'get_qos_policy_group_name_from_info',
-                         mock.Mock(return_value=fake.QOS_POLICY_GROUP_NAME))
+                         return_value=fake.QOS_POLICY_GROUP_NAME)
         self.library._add_lun_to_table = mock.Mock()
         self.zapi_client.move_lun = mock.Mock()
         mock_set_lun_qos_policy_group = self.mock_object(
@@ -649,7 +676,7 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library.backend_name = 'dev0'
         self.mock_object(data_motion.DataMotionMixin,
                          'get_replication_backend_names',
-                         mock.Mock(return_value=configured_targets))
+                         return_value=configured_targets)
         complete_failover_call = self.mock_object(
             data_motion.DataMotionMixin, '_complete_failover')
 
@@ -663,12 +690,12 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library.backend_name = 'dev0'
         self.mock_object(
             data_motion.DataMotionMixin, '_complete_failover',
-            mock.Mock(side_effect=exception.NetAppDriverException))
+            side_effect=na_utils.NetAppDriverException)
         self.mock_object(data_motion.DataMotionMixin,
                          'get_replication_backend_names',
-                         mock.Mock(return_value=['dev1', 'dev2']))
+                         return_value=['dev1', 'dev2'])
         self.mock_object(self.library.ssc_library, 'get_ssc_flexvol_names',
-                         mock.Mock(return_value=fake_utils.SSC.keys()))
+                         return_value=fake_utils.SSC.keys())
         self.mock_object(self.library, '_update_zapi_client')
 
         self.assertRaises(exception.UnableToFailOver,
@@ -683,16 +710,16 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         """This tests executes a method in the DataMotionMixin."""
         self.library.backend_name = 'dev0'
         self.mock_object(data_motion.DataMotionMixin, '_complete_failover',
-                         mock.Mock(return_value=('dev1', [])))
+                         return_value=('dev1', []))
         self.mock_object(data_motion.DataMotionMixin,
                          'get_replication_backend_names',
-                         mock.Mock(return_value=['dev1', 'dev2']))
+                         return_value=['dev1', 'dev2'])
         self.mock_object(self.library.ssc_library, 'get_ssc_flexvol_names',
-                         mock.Mock(return_value=fake_utils.SSC.keys()))
+                         return_value=fake_utils.SSC.keys())
         self.mock_object(self.library, '_update_zapi_client')
 
-        actual_active, vol_updates = self.library.failover_host(
-            'fake_context', [], secondary_id='dev1')
+        actual_active, vol_updates, __ = self.library.failover_host(
+            'fake_context', [], secondary_id='dev1', groups=[])
 
         data_motion.DataMotionMixin._complete_failover.assert_called_once_with(
             'dev0', ['dev1', 'dev2'], fake_utils.SSC.keys(), [],
@@ -705,8 +732,8 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
 
     def test_add_looping_tasks(self):
         mock_update_ssc = self.mock_object(self.library, '_update_ssc')
-        mock_remove_unused_qos_policy_groups = self.mock_object(
-            self.zapi_client, 'remove_unused_qos_policy_groups')
+        mock_handle_housekeeping = self.mock_object(
+            self.library, '_handle_housekeeping_tasks')
         mock_add_task = self.mock_object(self.library.loopingcalls, 'add_task')
         mock_super_add_looping_tasks = self.mock_object(
             block_base.NetAppBlockStorageLibrary, '_add_looping_tasks')
@@ -718,9 +745,9 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
             mock.call(mock_update_ssc,
                       loopingcalls.ONE_HOUR,
                       loopingcalls.ONE_HOUR),
-            mock.call(mock_remove_unused_qos_policy_groups,
-                      loopingcalls.ONE_MINUTE,
-                      loopingcalls.ONE_MINUTE)])
+            mock.call(mock_handle_housekeeping,
+                      loopingcalls.TEN_MINUTES,
+                      0)])
         mock_super_add_looping_tasks.assert_called_once_with()
 
     def test_get_backing_flexvol_names(self):
@@ -730,3 +757,178 @@ class NetAppBlockStorageCmodeLibraryTestCase(test.TestCase):
         self.library._get_backing_flexvol_names()
 
         mock_ssc_library.assert_called_once_with()
+
+    def test_create_group(self):
+
+        model_update = self.library.create_group(
+            fake.VOLUME_GROUP)
+
+        self.assertEqual('available', model_update['status'])
+
+    def test_delete_group_volume_delete_failure(self):
+        self.mock_object(block_cmode, 'LOG')
+        self.mock_object(self.library, '_delete_lun', side_effect=Exception)
+
+        model_update, volumes = self.library.delete_group(
+            fake.VOLUME_GROUP, [fake.VG_VOLUME])
+
+        self.assertEqual('deleted', model_update['status'])
+        self.assertEqual('error_deleting', volumes[0]['status'])
+        self.assertEqual(1, block_cmode.LOG.exception.call_count)
+
+    def test_update_group(self):
+
+        model_update, add_volumes_update, remove_volumes_update = (
+            self.library.update_group(fake.VOLUME_GROUP))
+
+        self.assertIsNone(model_update)
+        self.assertIsNone(add_volumes_update)
+        self.assertIsNone(remove_volumes_update)
+
+    def test_delete_group_not_found(self):
+        self.mock_object(block_cmode, 'LOG')
+        self.mock_object(self.library, '_get_lun_attr', return_value=None)
+
+        model_update, volumes = self.library.delete_group(
+            fake.VOLUME_GROUP, [fake.VG_VOLUME])
+
+        self.assertEqual(0, block_cmode.LOG.error.call_count)
+        self.assertEqual(0, block_cmode.LOG.info.call_count)
+
+        self.assertEqual('deleted', model_update['status'])
+        self.assertEqual('deleted', volumes[0]['status'])
+
+    def test_create_group_snapshot_raise_exception(self):
+        self.mock_object(volume_utils, 'is_group_a_cg_snapshot_type',
+                         return_value=True)
+
+        mock_extract_host = self.mock_object(
+            volume_utils, 'extract_host', return_value=fake.POOL_NAME)
+
+        self.mock_object(self.zapi_client, 'create_cg_snapshot',
+                         side_effect=netapp_api.NaApiError)
+
+        self.assertRaises(na_utils.NetAppDriverException,
+                          self.library.create_group_snapshot,
+                          fake.VOLUME_GROUP,
+                          [fake.VG_SNAPSHOT])
+
+        mock_extract_host.assert_called_once_with(
+            fake.VG_SNAPSHOT['volume']['host'], level='pool')
+
+    def test_create_group_snapshot(self):
+        self.mock_object(volume_utils, 'is_group_a_cg_snapshot_type',
+                         return_value=False)
+
+        fake_lun = block_base.NetAppLun(fake.LUN_HANDLE, fake.LUN_ID,
+                                        fake.LUN_SIZE, fake.LUN_METADATA)
+        self.mock_object(self.library, '_get_lun_from_table',
+                         return_value=fake_lun)
+        mock__clone_lun = self.mock_object(self.library, '_clone_lun')
+
+        model_update, snapshots_model_update = (
+            self.library.create_group_snapshot(fake.VOLUME_GROUP,
+                                               [fake.SNAPSHOT]))
+
+        self.assertIsNone(model_update)
+        self.assertIsNone(snapshots_model_update)
+        mock__clone_lun.assert_called_once_with(fake_lun.name,
+                                                fake.SNAPSHOT['name'],
+                                                space_reserved='false',
+                                                is_snapshot=True)
+
+    def test_create_consistent_group_snapshot(self):
+        self.mock_object(volume_utils, 'is_group_a_cg_snapshot_type',
+                         return_value=True)
+
+        self.mock_object(volume_utils, 'extract_host',
+                         return_value=fake.POOL_NAME)
+        mock_create_cg_snapshot = self.mock_object(
+            self.zapi_client, 'create_cg_snapshot')
+        mock__clone_lun = self.mock_object(self.library, '_clone_lun')
+        mock_wait_for_busy_snapshot = self.mock_object(
+            self.zapi_client, 'wait_for_busy_snapshot')
+        mock_delete_snapshot = self.mock_object(
+            self.zapi_client, 'delete_snapshot')
+
+        model_update, snapshots_model_update = (
+            self.library.create_group_snapshot(fake.VOLUME_GROUP,
+                                               [fake.VG_SNAPSHOT]))
+
+        self.assertIsNone(model_update)
+        self.assertIsNone(snapshots_model_update)
+
+        mock_create_cg_snapshot.assert_called_once_with(
+            set([fake.POOL_NAME]), fake.VOLUME_GROUP['id'])
+        mock__clone_lun.assert_called_once_with(
+            fake.VG_SNAPSHOT['volume']['name'],
+            fake.VG_SNAPSHOT['name'],
+            source_snapshot=fake.VOLUME_GROUP['id'])
+        mock_wait_for_busy_snapshot.assert_called_once_with(
+            fake.POOL_NAME, fake.VOLUME_GROUP['id'])
+        mock_delete_snapshot.assert_called_once_with(
+            fake.POOL_NAME, fake.VOLUME_GROUP['id'])
+
+    @ddt.data(None,
+              {'replication_status': fields.ReplicationStatus.ENABLED})
+    def test_create_group_from_src_snapshot(self, volume_model_update):
+        mock_clone_source_to_destination = self.mock_object(
+            self.library, '_clone_source_to_destination',
+            return_value=volume_model_update)
+
+        actual_return_value = self.library.create_group_from_src(
+            fake.VOLUME_GROUP, [fake.VOLUME], group_snapshot=fake.VG_SNAPSHOT,
+            snapshots=[fake.VG_VOLUME_SNAPSHOT])
+
+        clone_source_to_destination_args = {
+            'name': fake.VG_SNAPSHOT['name'],
+            'size': fake.VG_SNAPSHOT['volume_size'],
+        }
+        mock_clone_source_to_destination.assert_called_once_with(
+            clone_source_to_destination_args, fake.VOLUME)
+        if volume_model_update:
+            volume_model_update['id'] = fake.VOLUME['id']
+        expected_return_value = ((None, [volume_model_update])
+                                 if volume_model_update else (None, []))
+        self.assertEqual(expected_return_value, actual_return_value)
+
+    @ddt.data(None,
+              {'replication_status': fields.ReplicationStatus.ENABLED})
+    def test_create_group_from_src_group(self, volume_model_update):
+        lun_name = fake.SOURCE_VG_VOLUME['name']
+        mock_lun = block_base.NetAppLun(
+            lun_name, lun_name, '3', {'UUID': 'fake_uuid'})
+        self.mock_object(self.library, '_get_lun_from_table',
+                         return_value=mock_lun)
+        mock_clone_source_to_destination = self.mock_object(
+            self.library, '_clone_source_to_destination',
+            return_value=volume_model_update)
+
+        actual_return_value = self.library.create_group_from_src(
+            fake.VOLUME_GROUP, [fake.VOLUME],
+            source_group=fake.SOURCE_VOLUME_GROUP,
+            source_vols=[fake.SOURCE_VG_VOLUME])
+
+        clone_source_to_destination_args = {
+            'name': fake.SOURCE_VG_VOLUME['name'],
+            'size': fake.SOURCE_VG_VOLUME['size'],
+        }
+        if volume_model_update:
+            volume_model_update['id'] = fake.VOLUME['id']
+        expected_return_value = ((None, [volume_model_update])
+                                 if volume_model_update else (None, []))
+        mock_clone_source_to_destination.assert_called_once_with(
+            clone_source_to_destination_args, fake.VOLUME)
+        self.assertEqual(expected_return_value, actual_return_value)
+
+    def test_delete_group_snapshot(self):
+        mock__delete_lun = self.mock_object(self.library, '_delete_lun')
+
+        model_update, snapshots_model_update = (
+            self.library.delete_group_snapshot(fake.VOLUME_GROUP,
+                                               [fake.VG_SNAPSHOT]))
+
+        self.assertIsNone(model_update)
+        self.assertIsNone(snapshots_model_update)
+
+        mock__delete_lun.assert_called_once_with(fake.VG_SNAPSHOT['name'])
