@@ -14,27 +14,26 @@
 #    under the License.
 """ Tests for create_volume TaskFlow """
 
-import six
 import sys
+from unittest import mock
 import uuid
 
-import ddt
-import mock
-
 from castellan.common import exception as castellan_exc
 from castellan.tests.unit.key_manager import mock_key_manager
+import ddt
 from oslo_utils import imageutils
+import six
 
 from cinder import context
 from cinder import exception
 from cinder.message import message_field
-from cinder import test
 from cinder.tests.unit.backup import fake_backup
 from cinder.tests.unit.consistencygroup import fake_consistencygroup
 from cinder.tests.unit import fake_constants as fakes
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit.image import fake as fake_image
+from cinder.tests.unit import test
 from cinder.tests.unit import utils
 from cinder.tests.unit.volume.flows import fake_volume_api
 from cinder.volume.flows.api import create_volume
@@ -60,19 +59,25 @@ class CreateVolumeFlowTestCase(test.TestCase):
             return_value={})
 
     @mock.patch('cinder.objects.Volume.get_by_id')
-    @mock.patch('cinder.volume.utils.extract_host')
+    @mock.patch('cinder.volume.volume_utils.extract_host')
     @mock.patch('time.time')
     @mock.patch('cinder.objects.Snapshot.get_by_id')
     def test_cast_create_volume_from_resource(self, mock_snapshot_get,
                                               mock_time, mock_extract_host,
                                               volume_get_by_id):
         mock_time.side_effect = self.time_inc
-        volume = fake_volume.fake_volume_obj(self.ctxt,
-                                             host='host@backend#pool')
+        volume = fake_volume.fake_volume_obj(
+            self.ctxt,
+            host='host@backend#pool',
+            cluster_name='cluster@backend#pool')
         volume_get_by_id.return_value = volume
 
+        # This is the spec for a volume created from another resource. It
+        # includes the 'resource_backend'. When the volume is associated
+        # with a cluster the 'resource_backend' should use the cluster name.
         spec = {'volume_id': volume.id,
                 'volume': volume,
+                'resource_backend': 'cluster@backend#pool',
                 'source_volid': volume.id,
                 'snapshot_id': None,
                 'image_id': 4,
@@ -87,7 +92,13 @@ class CreateVolumeFlowTestCase(test.TestCase):
             fake_volume_api.FakeVolumeAPI(spec, self),
             fake_volume_api.FakeDb())
 
+        # Remove 'resource_backend' prior to calling task._cast_create_volume
+        # (the point of the test is to confirm that it adds it to the spec
+        # sent to the scheduler).
+        spec.pop('resource_backend')
+
         task._cast_create_volume(self.ctxt, spec, {})
+        mock_snapshot_get.assert_not_called()
         mock_extract_host.assert_not_called()
 
         snapshot = fake_snapshot.fake_snapshot_obj(self.ctxt,
@@ -96,6 +107,7 @@ class CreateVolumeFlowTestCase(test.TestCase):
 
         spec = {'volume_id': volume.id,
                 'volume': volume,
+                'resource_backend': 'cluster@backend#pool',
                 'source_volid': None,
                 'snapshot_id': snapshot.id,
                 'image_id': 4,
@@ -110,12 +122,13 @@ class CreateVolumeFlowTestCase(test.TestCase):
             fake_volume_api.FakeVolumeAPI(spec, self),
             fake_volume_api.FakeDb())
 
+        spec.pop('resource_backend')
         task._cast_create_volume(self.ctxt, spec, {})
         mock_snapshot_get.assert_called_once_with(self.ctxt, snapshot.id)
         mock_extract_host.assert_not_called()
 
     @mock.patch('cinder.objects.Volume.get_by_id')
-    @mock.patch('cinder.volume.utils.extract_host')
+    @mock.patch('cinder.volume.volume_utils.extract_host')
     @mock.patch('time.time')
     @mock.patch('cinder.objects.ConsistencyGroup.get_by_id')
     def test_cast_create_volume(self, consistencygroup_get_by_id, mock_time,
@@ -124,10 +137,14 @@ class CreateVolumeFlowTestCase(test.TestCase):
         volume = fake_volume.fake_volume_obj(self.ctxt)
         volume_get_by_id.return_value = volume
         props = {}
-        cg_obj = (fake_consistencygroup.
-                  fake_consistencyobject_obj(self.ctxt, consistencygroup_id=1,
-                                             host='host@backend#pool'))
+        cg_obj = fake_consistencygroup.fake_consistencyobject_obj(
+            self.ctxt,
+            consistencygroup_id=1,
+            host='host@backend#pool',
+            cluster_name='cluster@backend#pool')
         consistencygroup_get_by_id.return_value = cg_obj
+        mock_extract_host.return_value = 'cluster@backend'
+
         spec = {'volume_id': None,
                 'volume': None,
                 'source_volid': None,
@@ -145,9 +162,14 @@ class CreateVolumeFlowTestCase(test.TestCase):
             fake_volume_api.FakeDb())
 
         task._cast_create_volume(self.ctxt, spec, props)
+        consistencygroup_get_by_id.assert_not_called()
+        mock_extract_host.assert_not_called()
 
+        # This is the spec for a volume created from a consistency group. It
+        # includes the 'resource_backend'.
         spec = {'volume_id': volume.id,
                 'volume': volume,
+                'resource_backend': 'cluster@backend',
                 'source_volid': 2,
                 'snapshot_id': 3,
                 'image_id': 4,
@@ -162,9 +184,14 @@ class CreateVolumeFlowTestCase(test.TestCase):
             fake_volume_api.FakeVolumeAPI(spec, self),
             fake_volume_api.FakeDb())
 
+        # Remove 'resource_backend' prior to calling task._cast_create_volume
+        # (the point of the test is to confirm that it adds it to the spec
+        # sent to the scheduler).
+        spec.pop('resource_backend')
+
         task._cast_create_volume(self.ctxt, spec, props)
         consistencygroup_get_by_id.assert_called_once_with(self.ctxt, 5)
-        mock_extract_host.assert_called_once_with('host@backend#pool')
+        mock_extract_host.assert_called_once_with('cluster@backend#pool')
 
     @mock.patch('cinder.db.volume_create')
     @mock.patch('cinder.objects.Volume.get_by_id')
@@ -214,6 +241,34 @@ class CreateVolumeFlowTestCase(test.TestCase):
                               multiattach=None)
         self.assertFalse(result['volume_properties']['bootable'])
 
+    @ddt.data({'bootable': True},
+              {'bootable': False})
+    @mock.patch('cinder.db.volume_create')
+    @mock.patch('cinder.objects.Volume.get_by_id')
+    @ddt.unpack
+    def test_create_from_source_volid_bootable(self,
+                                               volume_get_by_id,
+                                               volume_create,
+                                               bootable):
+
+        volume_db = {'bootable': bootable}
+        volume_obj = fake_volume.fake_volume_obj(self.ctxt, **volume_db)
+        volume_get_by_id.return_value = volume_obj
+        volume_create.return_value = {'id': '123456'}
+        task = create_volume.EntryCreateTask()
+
+        result = task.execute(self.ctxt,
+                              optional_args=None,
+                              source_volid=volume_obj.id,
+                              snapshot_id=None,
+                              availability_zones=['nova'],
+                              size=1,
+                              encryption_key_id=None,
+                              description='123',
+                              name='123',
+                              multiattach=None)
+        self.assertEqual(bootable, result['volume_properties']['bootable'])
+
     @ddt.data(('enabled', {'replication_enabled': '<is> True'}),
               ('disabled', {'replication_enabled': '<is> False'}),
               ('disabled', {}))
@@ -790,20 +845,13 @@ class CreateVolumeFlowTestCase(test.TestCase):
                            'backup_id': None}
         self.assertEqual(expected_result, result)
 
-    @mock.patch('cinder.db.volume_type_get_by_name')
-    @mock.patch('cinder.volume.volume_types.is_encrypted')
-    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
-    @mock.patch('cinder.volume.volume_types.get_default_volume_type')
     @mock.patch('cinder.objects.volume_type.VolumeType.get_by_name_or_id')
     def test_extract_image_volume_type_from_image_invalid_type(
             self,
-            fake_get_type,
-            fake_get_def_vol_type,
-            fake_get_qos,
-            fake_is_encrypted,
-            fake_db_get_vol_type):
-
-        image_volume_type = 'invalid'
+            fake_get_type):
+        # Expected behavior: if the cinder_img_volume_type image property
+        # specifies an invalid type, it should raise an exception
+        image_volume_type = 'an_invalid_type'
         fake_image_service = fake_image.FakeImageService()
         image_id = 7
         image_meta = {}
@@ -819,13 +867,14 @@ class CreateVolumeFlowTestCase(test.TestCase):
             fake_image_service,
             {'nova'})
 
-        fake_is_encrypted.return_value = False
-        fake_get_def_vol_type.return_value = {'name': 'fake_vol_type'}
-        fake_get_type.return_value = {'name': 'fake_vol_type'}
-        fake_db_get_vol_type.side_effect = (
-            exception.VolumeTypeNotFoundByName(volume_type_name='invalid'))
-        fake_get_qos.return_value = {'qos_specs': None}
-        result = task.execute(self.ctxt,
+        def raise_with_id(stuff, id):
+            raise exception.VolumeTypeNotFoundByName(volume_type_name=id)
+
+        fake_get_type.side_effect = raise_with_id
+
+        e = self.assertRaises(exception.VolumeTypeNotFoundByName,
+                              task.execute,
+                              self.ctxt,
                               size=1,
                               snapshot=None,
                               image_id=image_id,
@@ -839,39 +888,26 @@ class CreateVolumeFlowTestCase(test.TestCase):
                               group=None,
                               group_snapshot=None,
                               backup=None)
-        expected_result = {'size': 1,
-                           'snapshot_id': None,
-                           'source_volid': None,
-                           'availability_zones': ['nova'],
-                           'volume_type': {'name': 'fake_vol_type'},
-                           'volume_type_id': None,
-                           'encryption_key_id': None,
-                           'qos_specs': None,
-                           'consistencygroup_id': None,
-                           'cgsnapshot_id': None,
-                           'group_id': None,
-                           'multiattach': False,
-                           'refresh_az': False,
-                           'replication_status': 'disabled',
-                           'backup_id': None}
-        self.assertEqual(expected_result, result)
+        self.assertIn(image_volume_type, str(e))
 
-    @mock.patch('cinder.db.volume_type_get_by_name')
     @mock.patch('cinder.volume.volume_types.is_encrypted')
     @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
     @mock.patch('cinder.objects.volume_type.VolumeType.get_by_name_or_id')
+    @mock.patch('cinder.volume.volume_types.get_default_volume_type')
     @ddt.data((8, None), (9, {'cinder_img_volume_type': None}))
     @ddt.unpack
     def test_extract_image_volume_type_from_image_properties_error(
             self,
             image_id,
             fake_img_properties,
-            fake_get_type,
+            fake_get_default_vol_type,
+            fake_get_by_name_or_id,
             fake_get_qos,
-            fake_is_encrypted,
-            fake_db_get_vol_type):
-
-        self.flags(default_volume_type='fake_volume_type')
+            fake_is_encrypted):
+        # Expected behavior: if the image has no properties
+        # or the cinder_img_volume_type is present but has no
+        # value, the default volume type should be used
+        self.flags(default_volume_type='fake_default_volume_type')
         fake_image_service = fake_image.FakeImageService()
         image_meta = {}
         image_meta['id'] = image_id
@@ -886,8 +922,19 @@ class CreateVolumeFlowTestCase(test.TestCase):
             {'nova'})
 
         fake_is_encrypted.return_value = False
-        fake_get_type.return_value = None
         fake_get_qos.return_value = {'qos_specs': None}
+
+        fake_volume_type = {'name': 'fake_default_volume_type',
+                            'id': fakes.VOLUME_TYPE_ID}
+        fake_get_default_vol_type.return_value = fake_volume_type
+
+        # yeah, I don't like this either, but until someone figures
+        # out why we re-get the volume_type object in the execute
+        # function, we have to do this.  At least I will check later
+        # and make sure we called it with the correct vol_type_id, so
+        # I'm not completely cheating
+        fake_get_by_name_or_id.return_value = fake_volume_type
+
         result = task.execute(self.ctxt,
                               size=1,
                               snapshot=None,
@@ -902,12 +949,17 @@ class CreateVolumeFlowTestCase(test.TestCase):
                               group=None,
                               group_snapshot=None,
                               backup=None)
+
+        fake_get_default_vol_type.assert_called_once()
+        fake_get_by_name_or_id.assert_called_once_with(
+            self.ctxt, fakes.VOLUME_TYPE_ID)
+
         expected_result = {'size': 1,
                            'snapshot_id': None,
                            'source_volid': None,
                            'availability_zones': ['nova'],
-                           'volume_type': None,
-                           'volume_type_id': None,
+                           'volume_type': fake_volume_type,
+                           'volume_type_id': fakes.VOLUME_TYPE_ID,
                            'encryption_key_id': None,
                            'qos_specs': None,
                            'consistencygroup_id': None,
@@ -919,46 +971,43 @@ class CreateVolumeFlowTestCase(test.TestCase):
                            'backup_id': None}
         self.assertEqual(expected_result, result)
 
-    @mock.patch('cinder.db.volume_type_get_by_name')
-    @mock.patch('cinder.volume.volume_types.is_encrypted')
-    @mock.patch('cinder.volume.volume_types.get_volume_type_qos_specs')
-    def test_extract_image_volume_type_from_image_invalid_input(
-            self,
-            fake_get_qos,
-            fake_is_encrypted,
-            fake_db_get_vol_type):
+    glance_nonactive_statuses = ('queued', 'saving', 'deleted', 'deactivated',
+                                 'uploading', 'importing', 'not_a_vaild_state',
+                                 'error')
 
+    @ddt.data(*glance_nonactive_statuses)
+    def test_extract_image_volume_type_from_image_invalid_input(
+            self, status):
+        # Expected behavior: an image must be in 'active' status
+        # or we should not create an image from it
         fake_image_service = fake_image.FakeImageService()
-        image_id = 10
-        image_meta = {}
-        image_meta['id'] = image_id
-        image_meta['status'] = 'inactive'
-        fake_image_service.create(self.ctxt, image_meta)
+        image_meta = {'status': status}
+        image_id = fake_image_service.create(self.ctxt, image_meta)['id']
         fake_key_manager = mock_key_manager.MockKeyManager()
 
         task = create_volume.ExtractVolumeRequestTask(
             fake_image_service,
             {'nova'})
 
-        fake_is_encrypted.return_value = False
-        fake_get_qos.return_value = {'qos_specs': None}
-
-        self.assertRaises(exception.InvalidInput,
-                          task.execute,
-                          self.ctxt,
-                          size=1,
-                          snapshot=None,
-                          image_id=image_id,
-                          source_volume=None,
-                          availability_zone='nova',
-                          volume_type=None,
-                          metadata=None,
-                          key_manager=fake_key_manager,
-                          consistencygroup=None,
-                          cgsnapshot=None,
-                          group=None,
-                          group_snapshot=None,
-                          backup=None)
+        e = self.assertRaises(exception.InvalidInput,
+                              task.execute,
+                              self.ctxt,
+                              size=1,
+                              snapshot=None,
+                              image_id=image_id,
+                              source_volume=None,
+                              availability_zone='nova',
+                              volume_type=None,
+                              metadata=None,
+                              key_manager=fake_key_manager,
+                              consistencygroup=None,
+                              cgsnapshot=None,
+                              group=None,
+                              group_snapshot=None,
+                              backup=None)
+        self.assertIn("Invalid input received", str(e))
+        self.assertIn("Image {} is not active".format(image_id), str(e))
+        fake_image_service.delete(self.ctxt, image_id)
 
 
 @ddt.ddt
@@ -1121,32 +1170,6 @@ class CreateVolumeFlowManagerTestCase(test.TestCase):
                                                      image_meta=image_meta)
         mock_cleanup_cg.assert_called_once_with(volume)
 
-    @ddt.data(True, False)
-    def test__copy_image_to_volume(self, is_encrypted):
-        fake_db = mock.MagicMock()
-        fake_driver = mock.MagicMock()
-        fake_volume_manager = mock.MagicMock()
-        fake_manager = create_volume_manager.CreateVolumeFromSpecTask(
-            fake_volume_manager, fake_db, fake_driver)
-        key = fakes.ENCRYPTION_KEY_ID if is_encrypted else None
-        volume = fake_volume.fake_volume_obj(
-            self.ctxt,
-            encryption_key_id=key)
-
-        fake_image_service = fake_image.FakeImageService()
-        image_id = fakes.IMAGE_ID
-        image_meta = {'id': image_id}
-        image_location = 'abc'
-
-        fake_manager._copy_image_to_volume(self.ctxt, volume, image_meta,
-                                           image_location, fake_image_service)
-        if is_encrypted:
-            fake_driver.copy_image_to_encrypted_volume.assert_called_once_with(
-                self.ctxt, volume, fake_image_service, image_id)
-        else:
-            fake_driver.copy_image_to_volume.assert_called_once_with(
-                self.ctxt, volume, fake_image_service, image_id)
-
     @ddt.data({'driver_error': True},
               {'driver_error': False})
     @mock.patch('cinder.backup.api.API.get_available_backup_service_host')
@@ -1197,7 +1220,7 @@ class CreateVolumeFlowManagerTestCase(test.TestCase):
         fake_driver.create_volume_from_backup.assert_called_once_with(
             volume_obj, backup_obj)
         if driver_error:
-            mock_create_volume.assert_called_once_with(volume_obj)
+            mock_create_volume.assert_called_once_with(self.ctxt, volume_obj)
             mock_get_backup_host.assert_called_once_with(
                 backup_obj.host, backup_obj.availability_zone)
             mock_restore_backup.assert_called_once_with(self.ctxt,
@@ -1208,6 +1231,29 @@ class CreateVolumeFlowManagerTestCase(test.TestCase):
             fake_driver.create_volume_from_backup.assert_called_once_with(
                 volume_obj, backup_obj)
 
+    @mock.patch('cinder.message.api.API.create')
+    def test_create_drive_error(self, mock_message_create):
+        fake_db = mock.MagicMock()
+        fake_driver = mock.MagicMock()
+        fake_volume_manager = mock.MagicMock()
+        fake_manager = create_volume_manager.CreateVolumeFromSpecTask(
+            fake_volume_manager, fake_db, fake_driver)
+        volume_obj = fake_volume.fake_volume_obj(self.ctxt)
+        err = NotImplementedError()
+
+        fake_driver.create_volume.side_effect = [err]
+        self.assertRaises(
+            NotImplementedError,
+            fake_manager._create_raw_volume,
+            self.ctxt,
+            volume_obj)
+        mock_message_create.assert_called_once_with(
+            self.ctxt,
+            message_field.Action.CREATE_VOLUME_FROM_BACKEND,
+            resource_uuid=volume_obj.id,
+            detail=message_field.Detail.DRIVER_FAILED_CREATE,
+            exception=err)
+
 
 class CreateVolumeFlowManagerGlanceCinderBackendCase(test.TestCase):
 
@@ -1266,7 +1312,7 @@ class CreateVolumeFlowManagerGlanceCinderBackendCase(test.TestCase):
                                         image_id,
                                         image_meta,
                                         fake_image_service)
-        if format is 'raw' and not owner and location:
+        if format == 'raw' and not owner and location:
             fake_driver.create_cloned_volume.assert_called_once_with(
                 volume, image_volume)
             handle_bootable.assert_called_once_with(self.ctxt, volume,
@@ -1331,7 +1377,7 @@ class CreateVolumeFlowManagerGlanceCinderBackendCase(test.TestCase):
                                         image_id,
                                         image_meta,
                                         fake_image_service)
-        if format is 'raw' and not owner and location:
+        if format == 'raw' and not owner and location:
             fake_driver.create_cloned_volume.assert_called_once_with(
                 volume, image_volume)
             handle_bootable.assert_called_once_with(self.ctxt, volume,
@@ -1506,53 +1552,52 @@ class CreateVolumeFlowManagerImageCacheTestCase(test.TestCase):
         image_location = 'someImageLocationStr'
         image_id = fakes.IMAGE_ID
         image_meta = mock.MagicMock()
-        image_info = imageutils.QemuImgInfo()
-        image_info.virtual_size = '1073741824'
-        mock_qemu_info.return_value = image_info
-
         volume = fake_volume.fake_volume_obj(self.ctxt, size=1,
                                              host='foo@bar#pool')
-        image_volume = fake_volume.fake_db_volume(size=2)
-        self.mock_db.volume_create.return_value = image_volume
-
+        self.mock_driver.clone_image.return_value = (None, False)
         self.flags(verify_glance_signatures='disabled')
 
-        if cloning_supported:
-            mock_create_from_src.side_effect = exception.CinderException(
-                'Error during cloning')
-        else:
-            mock_create_from_src.side_effect = NotImplementedError(
-                'Driver does not support clone')
-
         manager = create_volume_manager.CreateVolumeFromSpecTask(
             self.mock_volume_manager,
             self.mock_db,
             self.mock_driver,
             image_volume_cache=self.mock_cache
         )
-
-        model_update = manager._create_from_image_cache_or_download(
-            self.ctxt,
-            volume,
-            image_location,
-            image_id,
-            image_meta,
-            self.mock_image_service,
-            update_cache=False)
+        if cloning_supported:
+            mock_create_from_src.side_effect = exception.SnapshotLimitReached(
+                'Error during cloning')
+            self.assertRaises(
+                exception.SnapshotLimitReached,
+                manager._create_from_image,
+                self.ctxt,
+                volume,
+                image_location,
+                image_id,
+                image_meta,
+                self.mock_image_service)
+        else:
+            mock_create_from_src.side_effect = NotImplementedError(
+                'Driver does not support clone')
+            model_update = manager._create_from_image(
+                self.ctxt,
+                volume,
+                image_location,
+                image_id,
+                image_meta,
+                self.mock_image_service)
+            mock_create_from_img_dl.assert_called_once()
+            self.assertEqual(mock_create_from_img_dl.return_value,
+                             model_update)
 
         # Ensure cloning was attempted and that it failed
         mock_create_from_src.assert_called_once()
-        mock_create_from_img_dl.assert_called_once()
-        self.assertEqual(mock_create_from_img_dl.return_value, model_update)
-
-        # Ensure a new cache entry is created when cloning fails, but
-        # only when the driver supports cloning.
-        if cloning_supported:
-            (self.mock_volume_manager.
-             _create_image_cache_volume_entry.assert_called_once())
-        else:
-            (self.mock_volume_manager.
-             _create_image_cache_volume_entry.assert_not_called())
+        with mock.patch(
+                'cinder.volume.flows.manager.create_volume.'
+                'CreateVolumeFromSpecTask') as volume_manager:
+            (volume_manager.CreateVolumeFromSpecTask.
+             _create_from_image_cache_or_download.called_once())
+            (volume_manager.CreateVolumeFromSpecTask.
+             _create_from_image_cache.called_once())
 
     @mock.patch('cinder.volume.flows.manager.create_volume.'
                 'CreateVolumeFromSpecTask.'
