@@ -16,9 +16,9 @@
 """Tests for volume and images."""
 
 import datetime
-import mock
 import os
 import tempfile
+from unittest import mock
 
 from oslo_utils import imageutils
 from oslo_utils import units
@@ -30,6 +30,7 @@ from cinder import objects
 from cinder.objects import fields
 from cinder import quota
 from cinder.tests import fake_driver
+from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit.image import fake as fake_image
 from cinder.tests.unit import utils as tests_utils
@@ -79,12 +80,20 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
             'display_description': 'Test Desc',
             'size': 20,
             'status': 'uploading',
-            'host': 'dummy'
+            'host': 'dummy',
+            'volume_type_id': fake.VOLUME_TYPE_ID
         }
+        self.mock_object(db.sqlalchemy.api, 'volume_type_get',
+                         v2_fakes.fake_volume_type_get)
 
     def test_copy_volume_to_image_status_available(self):
         # creating volume testdata
         self.volume_attrs['instance_uuid'] = None
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
         db.volume_create(self.context, self.volume_attrs)
 
         # start test
@@ -125,6 +134,11 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
         # Creating volume testdata
         self.volume_attrs['instance_uuid'] = 'b21f957d-a72f-4b93-b5a5-' \
                                              '45b1161abb02'
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
         db.volume_create(self.context, self.volume_attrs)
 
         method = 'volume_update_status_based_on_attachment'
@@ -146,6 +160,11 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
     def test_copy_volume_to_image_status_use(self):
         self.image_meta['id'] = 'a440c04b-79fa-479c-bed1-0b816eaec379'
         # creating volume testdata
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
         db.volume_create(self.context, self.volume_attrs)
 
         # start test
@@ -159,6 +178,11 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
     def test_copy_volume_to_image_exception(self):
         self.image_meta['id'] = NON_EXISTENT_IMAGE_ID
         # creating volume testdata
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
         self.volume_attrs['status'] = 'in-use'
         db.volume_create(self.context, self.volume_attrs)
 
@@ -292,6 +316,11 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
         # creating volume testdata
         self.volume_attrs['instance_uuid'] = None
         self.volume_attrs['snapshot_id'] = fake.SNAPSHOT_ID
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
         db.volume_create(self.context, self.volume_attrs)
 
         def fake_create(context, volume, **kwargs):
@@ -437,6 +466,14 @@ class ImageVolumeCacheTestCase(base.BaseVolumeTestCase):
 
 class ImageVolumeTestCases(base.BaseVolumeTestCase):
 
+    def setUp(self):
+        super(ImageVolumeTestCases, self).setUp()
+        db.volume_type_create(self.context,
+                              v2_fakes.fake_default_type_get(
+                                  fake.VOLUME_TYPE2_ID))
+        self.vol_type = db.volume_type_get_by_name(self.context,
+                                                   'vol_type_name')
+
     @mock.patch('cinder.volume.drivers.lvm.LVMVolumeDriver.'
                 'create_cloned_volume')
     @mock.patch('cinder.quota.QUOTAS.rollback')
@@ -453,7 +490,9 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
 
         self.assertNotEqual(False, result)
         mock_reserve.assert_called_once_with(self.context, volumes=1,
-                                             gigabytes=vol.size)
+                                             volumes_vol_type_name=1,
+                                             gigabytes=vol.size,
+                                             gigabytes_vol_type_name=vol.size)
         mock_commit.assert_called_once_with(self.context, ["RESERVATION"],
                                             project_id=vol.project_id)
 
@@ -468,7 +507,9 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
                 self.context, vol, {'id': fake.VOLUME_ID}))
 
         mock_reserve.assert_called_once_with(self.context, volumes=1,
-                                             gigabytes=vol.size)
+                                             volumes_vol_type_name=1,
+                                             gigabytes=vol.size,
+                                             gigabytes_vol_type_name=vol.size)
         mock_rollback.assert_called_once_with(self.context, ["RESERVATION"])
 
     @mock.patch('cinder.image.image_utils.qemu_img_info')
@@ -669,7 +710,8 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
             volume_api = cinder.volume.api.API(
                 image_service=FakeImageService())
             volume = volume_api.create(self.context, 2, 'name', 'description',
-                                       image_id=self.FAKE_UUID)
+                                       image_id=self.FAKE_UUID,
+                                       volume_type=self.vol_type)
             volume_id = volume['id']
             self.assertEqual('creating', volume['status'])
 
@@ -720,7 +762,8 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
                         'disk_format': 'raw',
                         'container_format': 'bare',
                         'min_disk': 5,
-                        'status': 'deleted'}
+                        'status': 'deleted',
+                        'id': image_id}
 
         volume_api = cinder.volume.api.API(
             image_service=_ModifiedFakeImageService())
@@ -743,3 +786,38 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
                           volume,
                           test_meta1,
                           force=True)
+
+
+class CopyVolumeToImagePrivateFunctionsTestCase(
+        cinder.tests.unit.test.TestCase):
+
+    @mock.patch('cinder.volume.api.API.get_volume_image_metadata',
+                return_value={'some_key': 'some_value',
+                              'cinder_encryption_key_id': 'stale_value'})
+    def test_merge_volume_image_meta(self, mock_get_img_meta):
+        # this is what we're passing to copy_volume_to_image
+        image_meta = {
+            'container_format': 'bare',
+            'disk_format': 'raw',
+            'cinder_encryption_key_id': 'correct_value'
+        }
+        self.assertNotIn('properties', image_meta)
+
+        volume_api = cinder.volume.api.API()
+        volume_api._merge_volume_image_meta(None, None, image_meta)
+        # we've got 'properties' now
+        self.assertIn('properties', image_meta)
+        # verify the key_id is what we expect
+        self.assertEqual(image_meta['cinder_encryption_key_id'],
+                         'correct_value')
+
+        translate = cinder.image.glance.GlanceImageService._translate_to_glance
+        sent_to_glance = translate(image_meta)
+
+        # this is correct, glance gets a "flat" dict of properties
+        self.assertNotIn('properties', sent_to_glance)
+
+        # make sure the image would be created in Glance with the
+        # correct key_id
+        self.assertEqual(image_meta['cinder_encryption_key_id'],
+                         sent_to_glance['cinder_encryption_key_id'])
