@@ -16,17 +16,16 @@
 
 """Unit tests for brcd fc zone client http(s)."""
 import time
+from unittest import mock
+from unittest.mock import patch
 
 from oslo_utils import encodeutils
-
-import mock
-from mock import patch
 import six
 
-from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.zonemanager.drivers.brocade import (brcd_http_fc_zone_client
                                                 as client)
+from cinder.zonemanager.drivers.brocade import exception as b_exception
 import cinder.zonemanager.drivers.brocade.fc_zone_constants as zone_constant
 
 
@@ -47,6 +46,7 @@ qlps = {}
 ifas = {}
 parsed_raw_zoneinfo = ""
 random_no = ''
+auth_version = ''
 session = None
 active_cfg = 'openstack_cfg'
 activate = True
@@ -62,15 +62,14 @@ nameserver_info = """
 </HEAD>
 <BODY>
 <PRE>
---BEGIN NS INFO
-
-2;8;020800;N    ;10:00:00:05:1e:7c:64:96;20:00:00:05:1e:7c:64:96;[89]""" \
-"""Brocade-825 | 3.0.4.09 | DCM-X3650-94 | Microsoft Windows Server 2003 R2"""\
-    """| Service Pack 2";FCP ;      3;20:08:00:05:1e:89:54:a0;"""\
-    """0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0;000000;port8"""\
-    """
---END NS INFO
-
+--BEGIN DEVICEPORT 10:00:00:05:1e:7c:64:96
+node.wwn=20:00:00:05:1e:7c:64:96
+deviceport.portnum=9
+deviceport.portid=300900
+deviceport.portIndex=9
+deviceport.porttype=N
+deviceport.portwwn=10:00:00:05:1e:7c:64:96
+--END DEVICEPORT 10:00:00:05:1e:7c:64:96
 </PRE>
 </BODY>
 </HTML>
@@ -104,7 +103,8 @@ zone_string_to_post_no_activate = "zonecfginfo=openstack_cfg "\
     "&saveonly=true"
 zone_string_to_post_invalid_request = "zonecfginfo=openstack_cfg "\
     "openstack50060b0000c26604201900051ee8e32900000000000000000000000000;"\
-    "zone1;zone2 openstack50060b0000c26604201900051ee8e329000000000000000000000"\
+    "zone1;zone2 "\
+    "openstack50060b0000c26604201900051ee8e329000000000000000000000"\
     "00000 50:06:0b:00:00:c2:66:04;20:19:00:05:1e:e8:e3:29 "\
     "zone1 20:01:00:05:33:0e:96:15;20:00:00:05:33:0e:93:11 "\
     "zone2 20:01:00:05:33:0e:96:14;20:00:00:05:33:0e:93:11 &saveonly=true"
@@ -472,6 +472,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         self.ifas = {}
         self.parsed_raw_zoneinfo = ""
         self.random_no = ''
+        self.auth_version = ''
         self.session = None
         super(TestBrcdHttpFCZoneClient, self).setUp()
 
@@ -495,7 +496,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
     def test_authenticate_failed(self, connect_mock):
         connect_mock.return_value = un_authenticate_resp
         self.assertRaises(
-            exception.BrocadeZoningHttpException, self.authenticate)
+            b_exception.BrocadeZoningHttpException, self.authenticate)
 
     def test_get_parsed_data(self):
         valid_delimiter1 = zone_constant.SWITCHINFO_BEGIN
@@ -503,12 +504,12 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         invalid_delimiter = "--END SWITCH INFORMATION1"
         self.assertEqual(parsed_value, self.get_parsed_data(
             switch_page_resp, valid_delimiter1, valid_delimiter2))
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.get_parsed_data,
                           switch_page_resp,
                           valid_delimiter1,
                           invalid_delimiter)
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.get_parsed_data,
                           switch_page_resp,
                           invalid_delimiter,
@@ -520,7 +521,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         self.assertEqual(
             "v7.3.0b_rc1_bld06", self.get_nvp_value(parsed_value,
                                                     valid_keyname))
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.get_nvp_value,
                           parsed_value,
                           invalid_keyname)
@@ -529,7 +530,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         manageable_list = ['2', '128']
         self.assertEqual(
             manageable_list, self.get_managable_vf_list(session_info_vf))
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.get_managable_vf_list, session_info_AD)
 
     @mock.patch.object(client.BrcdHTTPFCZoneClient, 'is_vf_enabled')
@@ -537,7 +538,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         is_vf_enabled_mock.return_value = (True, session_info_vf)
         self.vfid = None
         self.assertRaises(
-            exception.BrocadeZoningHttpException,
+            b_exception.BrocadeZoningHttpException,
             self.check_change_vf_context)
         self.vfid = "2"
         with mock.patch.object(self, 'change_vf_context') \
@@ -551,7 +552,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         is_vf_enabled_mock.return_value = (False, session_info_AD)
         self.vfid = "128"
         self.assertRaises(
-            exception.BrocadeZoningHttpException,
+            b_exception.BrocadeZoningHttpException,
             self.check_change_vf_context)
 
     @mock.patch.object(client.BrcdHTTPFCZoneClient, 'get_managable_vf_list')
@@ -574,7 +575,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
                                               get_managable_vf_list_mock):
         get_managable_vf_list_mock.return_value = ['2', '128']
         connect_mock.return_value = session_info_vf_not_changed
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.change_vf_context, "2", session_info_vf)
         data = zone_constant.CHANGE_VF.format(vfid="2")
         headers = {zone_constant.AUTH_HEADER: self.auth_header}
@@ -586,7 +587,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
     def test_change_vf_context_vfid_not_managaed(self,
                                                  get_managable_vf_list_mock):
         get_managable_vf_list_mock.return_value = ['2', '128']
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.change_vf_context, "12", session_info_vf)
 
     @patch.object(client.BrcdHTTPFCZoneClient, 'connect')
@@ -654,7 +655,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
                               '20:19:00:05:1e:e8:e3:29']
                           }
         self.assertRaises(
-            exception.BrocadeZoningHttpException,
+            b_exception.BrocadeZoningHttpException,
             self.add_zones, add_zones_info, False)
 
     @patch.object(client.BrcdHTTPFCZoneClient, 'post_zone_data')
@@ -713,7 +714,7 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
         self.ifas = ifas.copy()
         self.active_cfg = active_cfg
         delete_zones_info = 'openstack50060b0000c26604201900051ee8e32'
-        self.assertRaises(exception.BrocadeZoningHttpException,
+        self.assertRaises(b_exception.BrocadeZoningHttpException,
                           self.delete_zones, delete_zones_info, False)
 
     @patch.object(time, 'sleep')
@@ -749,6 +750,13 @@ class TestBrcdHttpFCZoneClient(client.BrcdHTTPFCZoneClient, test.TestCase):
                 delete_zones_info,
                 active_cfg))
 
+        cfgs = {'openstack_cfg': 'openstack50060b0000c26604201900051ee8e329'}
+        res = self.delete_zones_cfgs(cfgs,
+                                     zones_to_delete.copy(),
+                                     delete_zones_info,
+                                     active_cfg)
+        self.assertEqual((zones, {}, ''), res)
+
         cfgs = {'openstack_cfg': 'zone2'}
         zones = {'zone2': '20:01:00:05:33:0e:96:14;20:00:00:05:33:0e:93:11'}
         delete_zones_info = valid_zone_name + ";zone1"
