@@ -12,16 +12,14 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-INFINIDAT InfiniBox Volume Driver
-"""
+"""INFINIDAT InfiniBox Volume Driver."""
 
 from contextlib import contextmanager
 import functools
 import platform
 import socket
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import units
@@ -35,8 +33,8 @@ from cinder import utils
 from cinder import version
 from cinder.volume import configuration
 from cinder.volume.drivers.san import san
-from cinder.volume import utils as vol_utils
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 from cinder.zonemanager import utils as fczm_utils
 
 try:
@@ -132,6 +130,16 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         self.configuration.append_config_values(infinidat_opts)
         self._lookup_service = fczm_utils.create_lookup_service()
 
+    @classmethod
+    def get_driver_options(cls):
+        additional_opts = cls._get_oslo_driver_opts(
+            'san_ip', 'san_login', 'san_password', 'use_chap_auth',
+            'chap_username', 'chap_password', 'san_thin_provision',
+            'use_multipath_for_image_xfer', 'enforce_multipath_for_image_xfer',
+            'num_volume_device_scan_tries', 'volume_dd_blocksize',
+            'max_over_subscription_ratio')
+        return infinidat_opts + additional_opts
+
     def _setup_and_get_system_object(self, management_address, auth):
         system = infinisdk.InfiniBox(management_address, auth=auth)
         system.api.add_auto_retry(
@@ -374,9 +382,9 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         infinidat_host = self._get_or_create_host(port)
         if self.configuration.use_chap_auth:
             chap_username = (self.configuration.chap_username or
-                             vol_utils.generate_username())
+                             volume_utils.generate_username())
             chap_password = (self.configuration.chap_password or
-                             vol_utils.generate_password())
+                             volume_utils.generate_password())
             infinidat_host.update_fields(
                 security_method='CHAP',
                 security_chap_inbound_username=chap_username,
@@ -623,11 +631,11 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                 dst_ctx = self._device_connect_context(volume)
                 with src_ctx as src_dev, dst_ctx as dst_dev:
                     dd_block_size = self.configuration.volume_dd_blocksize
-                    vol_utils.copy_volume(src_dev['device']['path'],
-                                          dst_dev['device']['path'],
-                                          snapshot.volume.size * units.Ki,
-                                          dd_block_size,
-                                          sparse=True)
+                    volume_utils.copy_volume(src_dev['device']['path'],
+                                             dst_dev['device']['path'],
+                                             snapshot.volume.size * units.Ki,
+                                             dd_block_size,
+                                             sparse=True)
             except Exception:
                 infinidat_volume.delete()
                 raise
@@ -676,11 +684,11 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             dst_ctx = self._device_connect_context(volume)
             with src_ctx as src_dev, dst_ctx as dst_dev:
                 dd_block_size = self.configuration.volume_dd_blocksize
-                vol_utils.copy_volume(src_dev['device']['path'],
-                                      dst_dev['device']['path'],
-                                      src_vref.size * units.Ki,
-                                      dd_block_size,
-                                      sparse=True)
+                volume_utils.copy_volume(src_dev['device']['path'],
+                                         dst_dev['device']['path'],
+                                         src_vref.size * units.Ki,
+                                         dd_block_size,
+                                         sparse=True)
         except Exception:
             infinidat_volume.delete()
             raise
@@ -719,7 +727,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def create_group(self, context, group):
         """Creates a group."""
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group):
+        if not volume_utils.is_group_a_cg_snapshot_type(group):
             raise NotImplementedError()
         obj = self._system.cons_groups.create(name=self._make_cg_name(group),
                                               pool=self._get_infinidat_pool())
@@ -730,7 +738,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def delete_group(self, context, group, volumes):
         """Deletes a group."""
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group):
+        if not volume_utils.is_group_a_cg_snapshot_type(group):
             raise NotImplementedError()
         try:
             infinidat_cg = self._get_infinidat_cg(group)
@@ -747,7 +755,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                      add_volumes=None, remove_volumes=None):
         """Updates a group."""
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group):
+        if not volume_utils.is_group_a_cg_snapshot_type(group):
             raise NotImplementedError()
         add_volumes = add_volumes if add_volumes else []
         remove_volumes = remove_volumes if remove_volumes else []
@@ -766,12 +774,12 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                               source_group=None, source_vols=None):
         """Creates a group from source."""
         # The source is either group_snapshot+snapshots or
-        # source_group+source_vols. The target is group+voluems
+        # source_group+source_vols. The target is group+volumes
         # we assume the source (source_vols / snapshots) are in the same
         # order as the target (volumes)
 
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group):
+        if not volume_utils.is_group_a_cg_snapshot_type(group):
             raise NotImplementedError()
         self.create_group(context, group)
         new_infinidat_group = self._get_infinidat_cg(group)
@@ -791,7 +799,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def create_group_snapshot(self, context, group_snapshot, snapshots):
         """Creates a group_snapshot."""
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group_snapshot):
+        if not volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
             raise NotImplementedError()
         infinidat_cg = self._get_infinidat_cg(group_snapshot.group)
         group_snap_name = self._make_group_snapshot_name(group_snapshot)
@@ -810,7 +818,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def delete_group_snapshot(self, context, group_snapshot, snapshots):
         """Deletes a group_snapshot."""
         # let generic volume group support handle non-cgsnapshots
-        if not vol_utils.is_group_a_cg_snapshot_type(group_snapshot):
+        if not volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
             raise NotImplementedError()
         cgsnap_name = self._make_group_snapshot_name(group_snapshot)
         infinidat_cgsnap = self._system.cons_groups.safe_get(name=cgsnap_name)
