@@ -36,7 +36,7 @@ from cinder.objects import fields
 from cinder import utils
 from cinder.volume import configuration
 from cinder.volume.drivers.san import san
-from cinder.volume import utils as vol_utils
+from cinder.volume import volume_utils
 
 krest = importutils.try_import("krest")
 
@@ -51,14 +51,6 @@ kaminario_opts = [
                 default=False,
                 help="K2 driver will calculate max_oversubscription_ratio "
                      "on setting this option as True."),
-    cfg.BoolOpt('unique_fqdn_network',
-                default=True,
-                help="Whether or not our private network has unique FQDN on "
-                     "each initiator or not.  For example networks with QA "
-                     "systems usually have multiple servers/VMs with the same "
-                     "FQDN.  When true this will create host entries on K2 "
-                     "using the FQDN, when false it will use the reversed "
-                     "IQN/WWNN."),
     cfg.BoolOpt('disable_discovery',
                 default=False,
                 help="Disabling iSCSI discovery (sendtargets) for multipath "
@@ -72,6 +64,15 @@ K2HTTPError = requests.exceptions.HTTPError
 K2_RETRY_ERRORS = ("MC_ERR_BUSY", "MC_ERR_BUSY_SPECIFIC",
                    "MC_ERR_INPROGRESS", "MC_ERR_START_TIMEOUT")
 
+
+class KaminarioCinderDriverException(exception.VolumeDriverException):
+    message = _("KaminarioCinderDriver failure: %(reason)s")
+
+
+class KaminarioRetryableException(exception.VolumeDriverException):
+    message = _("Kaminario retryable exception: %(reason)s")
+
+
 if krest:
     class KrestWrap(krest.EndPoint):
         def __init__(self, *args, **kwargs):
@@ -87,7 +88,7 @@ if krest:
                         return True
             return False
 
-        @utils.retry(exception.KaminarioRetryableException,
+        @utils.retry(KaminarioRetryableException,
                      retries=MAX_K2_RETRY)
         def _request(self, method, *args, **kwargs):
             try:
@@ -98,7 +99,7 @@ if krest:
                 err_code = err.response.status_code
                 err_msg = err.response.text
                 if self._should_retry(err_code, err_msg):
-                    raise exception.KaminarioRetryableException(
+                    raise KaminarioRetryableException(
                         reason=six.text_type(err_msg))
                 raise
             finally:
@@ -126,12 +127,19 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
         k2_lock_sfx = self.configuration.safe_get('san_ip')
         self.k2_lock_name = "%s-%s" % (K2_LOCK_PREFIX, k2_lock_sfx)
 
+    @classmethod
+    def get_driver_options(cls):
+        additional_opts = cls._get_oslo_driver_opts(
+            'san_ip', 'san_login', 'san_password', 'replication_device',
+            'volume_dd_blocksize', 'unique_fqdn_network')
+        return kaminario_opts + additional_opts
+
     @utils.trace
     def check_for_setup_error(self):
         if krest is None:
             msg = _("Unable to import 'krest' python module.")
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
         else:
             conf = self.configuration
             self.client = KrestWrap(conf.san_ip,
@@ -152,12 +160,12 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                     msg = _("K2 REST API version should be "
                             ">= %s.") % K2_MIN_VERSION
                     LOG.error(msg)
-                    raise exception.KaminarioCinderDriverException(reason=msg)
+                    raise KaminarioCinderDriverException(reason=msg)
 
             else:
                 msg = _("K2 REST API version search failed.")
                 LOG.error(msg)
-                raise exception.KaminarioCinderDriverException(reason=msg)
+                raise KaminarioCinderDriverException(reason=msg)
 
     def _check_ops(self):
         """Ensure that the options we care about are set."""
@@ -209,7 +217,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                 LOG.debug("Deleting vg: %s for failed volume in K2.", vg_name)
                 vg_rs.hits[0].delete()
             LOG.exception("Creation of volume %s failed.", vol_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
         if self._get_is_replica(volume.volume_type) and self.replica:
             self._create_volume_replica(volume, vg, vol, self.replica.rpo)
@@ -235,7 +243,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
         else:
             msg = _("Unable to find K2peer in source K2:")
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
         try:
             LOG.debug("Creating source session with name: %(sname)s and "
                       " target session name: %(tname)s",
@@ -270,7 +278,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             self._delete_by_ref(self.target, "volume_groups",
                                 rvg_name, "remote vg")
             self._delete_by_ref(self.client, "volume_groups", vg.name, "vg")
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def _create_failover_volume_replica(self, volume, vg_name, vol_name):
@@ -294,7 +302,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
         else:
             msg = _("Unable to find K2peer in source K2:")
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
         try:
             LOG.debug("Creating source session with name: %(sname)s and "
                       " target session name: %(tname)s",
@@ -325,7 +333,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                                 session_name, 'remote session')
             self._delete_by_ref(self.client, "volumes", vol_name, "volume")
             self._delete_by_ref(self.client, "volume_groups", vg_name, "vg")
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def _delete_by_ref(self, device, url, name, msg):
@@ -352,7 +360,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
         """Failover to replication target."""
         volume_updates = []
         back_end_ip = None
-        svc_host = vol_utils.extract_host(self.host, 'backend')
+        svc_host = volume_utils.extract_host(self.host, 'backend')
         service = objects.Service.get_by_args(context, svc_host,
                                               'cinder-volume')
 
@@ -512,10 +520,10 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
     def _is_user_snap_sync_finished(self):
         # waiting for user snapshot to be synced
         while len(self.snap_updates) > 0:
-            for l in self.snap_updates:
-                sess = l.get('tgt_ssn')
-                gno = l.get('gno')
-                stime = l.get('stime')
+            for update in self.snap_updates:
+                sess = update.get('tgt_ssn')
+                gno = update.get('gno')
+                stime = update.get('stime')
                 sess.refresh()
                 if (sess.generation_number == gno and
                    sess.current_snapshot_progress == 100
@@ -527,7 +535,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                         self.snap_updates.append({'tgt_ssn': sess,
                                                   'gno': gen_no,
                                                   'stime': time.time()})
-                    self.snap_updates.remove(l)
+                    self.snap_updates.remove(update)
                 eventlet.sleep(1)
 
     @utils.trace
@@ -561,12 +569,12 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                 LOG.exception("Creating a view: %(view)s from snapshot: "
                               "%(snap)s failed", {"view": view_name,
                                                   "snap": snap_name})
-                raise exception.KaminarioCinderDriverException(reason=ex)
+                raise KaminarioCinderDriverException(reason=ex)
 
         else:
             msg = _("Snapshot: %s search failed in K2.") % snap_name
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
 
         try:
             conn = self.initialize_connection(cview, properties)
@@ -574,11 +582,11 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             self.create_volume(volume)
             conn = self.initialize_connection(volume, properties)
             dest_attach_info = self._connect_device(conn)
-            vol_utils.copy_volume(src_attach_info['device']['path'],
-                                  dest_attach_info['device']['path'],
-                                  snapshot.volume.size * units.Ki,
-                                  self.configuration.volume_dd_blocksize,
-                                  sparse=True)
+            volume_utils.copy_volume(src_attach_info['device']['path'],
+                                     dest_attach_info['device']['path'],
+                                     snapshot.volume.size * units.Ki,
+                                     self.configuration.volume_dd_blocksize,
+                                     sparse=True)
             self._kaminario_disconnect_volume(src_attach_info,
                                               dest_attach_info)
             self.terminate_connection(volume, properties)
@@ -593,7 +601,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             self.delete_volume(volume)
             LOG.exception("Copy to volume: %(vol)s from view: %(view)s "
                           "failed", {"vol": vol_name, "view": view_name})
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def create_cloned_volume(self, volume, src_vref):
@@ -614,7 +622,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                     "To get this done, create a snapshot from the attached "
                     "volume and then create a volume from the snapshot.")
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
         try:
             properties = utils.brick_get_connector_properties()
             conn = self.initialize_connection(src_vref, properties)
@@ -622,11 +630,11 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             self.create_volume(volume)
             conn = self.initialize_connection(volume, properties)
             dest_attach_info = self._connect_device(conn)
-            vol_utils.copy_volume(src_attach_info['device']['path'],
-                                  dest_attach_info['device']['path'],
-                                  src_vref.size * units.Ki,
-                                  self.configuration.volume_dd_blocksize,
-                                  sparse=True)
+            volume_utils.copy_volume(src_attach_info['device']['path'],
+                                     dest_attach_info['device']['path'],
+                                     src_vref.size * units.Ki,
+                                     self.configuration.volume_dd_blocksize,
+                                     sparse=True)
             self._kaminario_disconnect_volume(src_attach_info,
                                               dest_attach_info)
             self.terminate_connection(volume, properties)
@@ -638,7 +646,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             self.terminate_connection(volume, properties)
             self.delete_volume(volume)
             LOG.exception("Create a clone: %s failed.", clone_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def delete_volume(self, volume):
@@ -663,7 +671,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                 vg_rs.hits[0].delete()
         except Exception as ex:
             LOG.exception("Deletion of volume %s failed.", vol_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def _delete_volume_replica(self, volume, vg_name, vol_name):
@@ -762,7 +770,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                             is_auto_deleteable=False).save()
         except Exception as ex:
             LOG.exception("Creation of snapshot: %s failed.", snap_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def delete_snapshot(self, snapshot):
@@ -775,7 +783,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                 snap_rs.hits[0].delete()
         except Exception as ex:
             LOG.exception("Deletion of snapshot: %s failed.", snap_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     @utils.trace
     def extend_volume(self, volume, new_size):
@@ -789,7 +797,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             vol.save()
         except Exception as ex:
             LOG.exception("Extending volume: %s failed.", vol_name)
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     def update_volume_stats(self):
         conf = self.configuration
@@ -803,7 +811,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                 and (cap.total - cap.free) != 0):
             ratio = provisioned_vol / float(cap.total - cap.free)
         else:
-            ratio = vol_utils.get_max_over_subscription_ratio(
+            ratio = volume_utils.get_max_over_subscription_ratio(
                 conf.max_over_subscription_ratio, supports_auto=True)
 
         self.stats = {'QoS_support': False,
@@ -875,7 +883,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
                                       name="Best_Effort_Retention").hits[0]
         except Exception as ex:
             LOG.exception("Retention policy search failed in K2.")
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
 
     def _get_volume_object(self, volume):
         vol_name = self.get_volume_name(volume.id)
@@ -886,7 +894,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
         if not hasattr(vol_rs, 'hits') or vol_rs.total == 0:
             msg = _("Unable to find volume: %s from K2.") % vol_name
             LOG.error(msg)
-            raise exception.KaminarioCinderDriverException(reason=msg)
+            raise KaminarioCinderDriverException(reason=msg)
         return vol_rs.hits[0]
 
     def _get_lun_number(self, vol, host):
@@ -959,7 +967,7 @@ class KaminarioCinderDriver(cinder.volume.driver.ISCSIDriver):
             LOG.exception("Unable to map volume: %(vol)s to host: "
                           "%(host)s", {'host': host_name,
                                        'vol': vol.name})
-            raise exception.KaminarioCinderDriverException(reason=ex)
+            raise KaminarioCinderDriverException(reason=ex)
         # Get lun number.
         if type(volume).__name__ == 'RestObject':
             return self._get_lun_number(vol, host)
