@@ -35,7 +35,7 @@ from cinder import interface
 from cinder import utils
 from cinder.volume import configuration
 from cinder.volume import driver
-from cinder.volume import utils as volutils
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -53,7 +53,9 @@ volume_opts = [
                     'this requires lvm_mirrors + 2 PVs with available space'),
     cfg.StrOpt('lvm_type',
                default='auto',
-               choices=['default', 'thin', 'auto'],
+               choices=[('default', 'Thick-provisioned LVM.'),
+                        ('thin', 'Thin-provisioned LVM.'),
+                        ('auto', 'Defaults to thin when supported.')],
                help='Type of LVM volumes to deploy; (default, thin, or auto). '
                     'Auto defaults to thin if thin is supported.'),
     cfg.StrOpt('lvm_conf_file',
@@ -110,6 +112,26 @@ class LVMVolumeDriver(driver.VolumeDriver):
         self.protocol = self.target_driver.protocol
         self._sparse_copy_volume = False
 
+    @classmethod
+    def get_driver_options(cls):
+        # Imports required to have config options
+        from cinder.volume.targets import spdknvmf  # noqa
+
+        additional_opts = cls._get_oslo_driver_opts(
+            'target_ip_address', 'target_helper', 'target_protocol',
+            'volume_clear', 'volume_clear_size', 'reserved_percentage',
+            'max_over_subscription_ratio', 'volume_dd_blocksize',
+            'target_prefix', 'volumes_dir', 'iscsi_secondary_ip_addresses',
+            'target_port',
+            'iscsi_write_cache', 'iscsi_target_flags',  # TGT
+            'iet_conf', 'iscsi_iotype',  # IET
+            'nvmet_port_id',  # NVMET
+            'scst_target_iqn_name', 'scst_target_driver',  # SCST
+            'spdk_rpc_ip', 'spdk_rpc_port', 'spdk_rpc_username',   # SPDKNVMF
+            'spdk_rpc_password', 'spdk_max_queue_depth',  # SPDKNVMF
+        )
+        return volume_opts + additional_opts
+
     def _sizestr(self, size_in_g):
         return '%sg' % size_in_g
 
@@ -161,7 +183,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
         # be sure to convert before passing in
         vol_sz_in_meg = size_in_g * units.Ki
 
-        volutils.clear_volume(
+        volume_utils.clear_volume(
             vol_sz_in_meg, dev_path,
             volume_clear=self.configuration.volume_clear,
             volume_clear_size=self.configuration.volume_clear_size)
@@ -283,7 +305,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
             try:
                 lvm_type = self.configuration.lvm_type
                 if lvm_type == 'auto':
-                    if volutils.supports_thin_provisioning():
+                    if volume_utils.supports_thin_provisioning():
                         lvm_type = 'thin'
                     else:
                         lvm_type = 'default'
@@ -301,10 +323,9 @@ class LVMVolumeDriver(driver.VolumeDriver):
                            self.configuration.volume_group)
                 raise exception.VolumeBackendAPIException(data=message)
 
-        vg_list = volutils.get_all_volume_groups(
+        vg_list = volume_utils.get_all_volume_groups(
             self.configuration.volume_group)
-        vg_dict = \
-            next(vg for vg in vg_list if vg['name'] == self.vg.vg_name)
+        vg_dict = next(vg for vg in vg_list if vg['name'] == self.vg.vg_name)
         if vg_dict is None:
             message = (_("Volume Group %s does not exist") %
                        self.configuration.volume_group)
@@ -320,7 +341,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
             self.configuration.lvm_type = 'default'
 
-            if volutils.supports_thin_provisioning():
+            if volume_utils.supports_thin_provisioning():
                 if self.vg.get_volume(pool_name) is not None:
                     LOG.info('Enabling LVM thin provisioning by default '
                              'because a thin pool exists.')
@@ -332,7 +353,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
         if self.configuration.lvm_type == 'thin':
             # Specific checks for using Thin provisioned LV's
-            if not volutils.supports_thin_provisioning():
+            if not volume_utils.supports_thin_provisioning():
                 message = _("Thin provisioning not supported "
                             "on this version of LVM.")
                 raise exception.VolumeBackendAPIException(data=message)
@@ -422,12 +443,12 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
         # copy_volume expects sizes in MiB, we store integer GiB
         # be sure to convert before passing in
-        volutils.copy_volume(self.local_path(snapshot),
-                             self.local_path(volume),
-                             snapshot['volume_size'] * units.Ki,
-                             self.configuration.volume_dd_blocksize,
-                             execute=self._execute,
-                             sparse=self._sparse_copy_volume)
+        volume_utils.copy_volume(self.local_path(snapshot),
+                                 self.local_path(volume),
+                                 snapshot['volume_size'] * units.Ki,
+                                 self.configuration.volume_dd_blocksize,
+                                 execute=self._execute,
+                                 sparse=self._sparse_copy_volume)
 
     def delete_volume(self, volume):
         """Deletes a logical volume."""
@@ -503,10 +524,11 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
     def copy_volume_to_image(self, context, volume, image_service, image_meta):
         """Copy the volume to the specified image."""
-        image_utils.upload_volume(context,
-                                  image_service,
-                                  image_meta,
-                                  self.local_path(volume))
+        volume_utils.upload_volume(context,
+                                   image_service,
+                                   image_meta,
+                                   self.local_path(volume),
+                                   volume)
 
     def create_cloned_volume(self, volume, src_vref):
         """Creates a clone of the specified volume."""
@@ -544,7 +566,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
                                 mirror_count)
 
             self.vg.activate_lv(temp_snapshot['name'], is_snapshot=True)
-            volutils.copy_volume(
+            volume_utils.copy_volume(
                 self.local_path(temp_snapshot),
                 self.local_path(volume),
                 src_vref['size'] * units.Ki,
@@ -559,17 +581,6 @@ class LVMVolumeDriver(driver.VolumeDriver):
                     image_service):
         return None, False
 
-    def get_volume_stats(self, refresh=False):
-        """Get volume status.
-
-        If 'refresh' is True, run update the stats first.
-        """
-
-        if refresh:
-            self._update_volume_stats()
-
-        return self._stats
-
     def extend_volume(self, volume, new_size):
         """Extend an existing volume's size."""
         self.vg.extend_volume(volume['name'],
@@ -589,8 +600,8 @@ class LVMVolumeDriver(driver.VolumeDriver):
         lv_name = existing_ref['source-name']
         self.vg.get_volume(lv_name)
 
-        vol_id = volutils.extract_id_from_volume_name(lv_name)
-        if volutils.check_already_managed_volume(vol_id):
+        vol_id = volume_utils.extract_id_from_volume_name(lv_name)
+        if volume_utils.check_already_managed_volume(vol_id):
             raise exception.ManageExistingAlreadyManaged(volume_ref=lv_name)
 
         # Attempt to rename the LV to match the OpenStack internal name.
@@ -672,10 +683,12 @@ class LVMVolumeDriver(driver.VolumeDriver):
                 continue
 
             if resource_type == 'volume':
-                potential_id = volutils.extract_id_from_volume_name(lv['name'])
+                potential_id = volume_utils.extract_id_from_volume_name(
+                    lv['name'])
             else:
                 unescape = self._unescape_snapshot(lv['name'])
-                potential_id = volutils.extract_id_from_snapshot_name(unescape)
+                potential_id = volume_utils.extract_id_from_snapshot_name(
+                    unescape)
             lv_info = {'reference': {'source-name': lv['name']},
                        'size': int(math.ceil(float(lv['size']))),
                        'cinder_id': None,
@@ -698,8 +711,8 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
             entries.append(lv_info)
 
-        return volutils.paginate_entries_list(entries, marker, limit, offset,
-                                              sort_keys, sort_dirs)
+        return volume_utils.paginate_entries_list(entries, marker, limit,
+                                                  offset, sort_keys, sort_dirs)
 
     def get_manageable_volumes(self, cinder_volumes, marker, limit, offset,
                                sort_keys, sort_dirs):
@@ -752,7 +765,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
             LOG.error(message)
             raise exception.VolumeBackendAPIException(data=message)
 
-        vg_list = volutils.get_all_volume_groups()
+        vg_list = volume_utils.get_all_volume_groups()
         try:
             next(vg for vg in vg_list if vg['name'] == dest_vg)
         except StopIteration:
@@ -780,12 +793,12 @@ class LVMVolumeDriver(driver.VolumeDriver):
         # be sure to convert before passing in
         size_in_mb = int(volume['size']) * units.Ki
         try:
-            volutils.copy_volume(self.local_path(volume),
-                                 self.local_path(volume, vg=dest_vg),
-                                 size_in_mb,
-                                 self.configuration.volume_dd_blocksize,
-                                 execute=self._execute,
-                                 sparse=self._sparse_copy_volume)
+            volume_utils.copy_volume(self.local_path(volume),
+                                     self.local_path(volume, vg=dest_vg),
+                                     size_in_mb,
+                                     self.configuration.volume_dd_blocksize,
+                                     execute=self._execute,
+                                     sparse=self._sparse_copy_volume)
         except Exception as e:
             with excutils.save_and_reraise_exception():
                 LOG.error("Volume migration failed due to "
