@@ -30,15 +30,17 @@ import six
 
 from cinder import exception
 from cinder.i18n import _
+from cinder.objects import fields
 from cinder import utils
 from cinder.volume.drivers.netapp.dataontap import block_base
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
 from cinder.volume.drivers.netapp.dataontap.utils import capabilities
 from cinder.volume.drivers.netapp.dataontap.utils import data_motion
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
-from cinder.volume.drivers.netapp.dataontap.utils import utils as cmode_utils
+from cinder.volume.drivers.netapp.dataontap.utils import utils as dot_utils
 from cinder.volume.drivers.netapp import options as na_opts
 from cinder.volume.drivers.netapp import utils as na_utils
+from cinder.volume import volume_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -68,23 +70,28 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         na_utils.check_flags(self.REQUIRED_CMODE_FLAGS, self.configuration)
 
         # cDOT API client
-        self.zapi_client = cmode_utils.get_client_for_backend(
+        self.zapi_client = dot_utils.get_client_for_backend(
             self.failed_over_backend_name or self.backend_name)
         self.vserver = self.zapi_client.vserver
 
-        # Performance monitoring library
-        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
-            self.zapi_client)
-
         # Storage service catalog
         self.ssc_library = capabilities.CapabilitiesLibrary(
             self.driver_protocol, self.vserver, self.zapi_client,
             self.configuration)
 
+        self.ssc_library.check_api_permissions()
+
+        self.using_cluster_credentials = (
+            self.ssc_library.cluster_user_supported())
+
+        # Performance monitoring library
+        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
+            self.zapi_client)
+
     def _update_zapi_client(self, backend_name):
         """Set cDOT API client for the specified config backend stanza name."""
 
-        self.zapi_client = cmode_utils.get_client_for_backend(backend_name)
+        self.zapi_client = dot_utils.get_client_for_backend(backend_name)
         self.vserver = self.zapi_client.vserver
         self.ssc_library._update_for_failover(self.zapi_client,
                                               self._get_flexvol_to_pool_map())
@@ -95,13 +102,11 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
 
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate."""
-        self.ssc_library.check_api_permissions()
-
         if not self._get_flexvol_to_pool_map():
             msg = _('No pools are available for provisioning volumes. '
                     'Ensure that the configuration option '
                     'netapp_pool_name_search_pattern is set correctly.')
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
         self._add_looping_tasks()
         super(NetAppBlockStorageCmodeLibrary, self).check_for_setup_error()
 
@@ -117,12 +122,6 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
                                    loopingcalls.ONE_HOUR,
                                    loopingcalls.ONE_HOUR)
 
-        # Add the task that harvests soft-deleted QoS policy groups.
-        self.loopingcalls.add_task(
-            self.zapi_client.remove_unused_qos_policy_groups,
-            loopingcalls.ONE_MINUTE,
-            loopingcalls.ONE_MINUTE)
-
         self.loopingcalls.add_task(
             self._handle_housekeeping_tasks,
             loopingcalls.TEN_MINUTES,
@@ -132,12 +131,12 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
 
     def _handle_housekeeping_tasks(self):
         """Handle various cleanup activities."""
-
-        # Harvest soft-deleted QoS policy groups
-        self.zapi_client.remove_unused_qos_policy_groups()
-
         active_backend = self.failed_over_backend_name or self.backend_name
 
+        # Add the task that harvests soft-deleted QoS policy groups.
+        if self.using_cluster_credentials:
+            self.zapi_client.remove_unused_qos_policy_groups()
+
         LOG.debug("Current service state: Replication enabled: %("
                   "replication)s. Failed-Over: %(failed)s. Active Backend "
                   "ID: %(active)s",
@@ -153,12 +152,26 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
                 self.configuration, self.backend_name,
                 self.ssc_library.get_ssc_flexvol_names())
 
+    def _handle_ems_logging(self):
+        """Log autosupport messages."""
+
+        base_ems_message = dot_utils.build_ems_log_message_0(
+            self.driver_name, self.app_version)
+        self.zapi_client.send_ems_log_message(base_ems_message)
+
+        pool_ems_message = dot_utils.build_ems_log_message_1(
+            self.driver_name, self.app_version, self.vserver,
+            self.ssc_library.get_ssc_flexvol_names(), [])
+        self.zapi_client.send_ems_log_message(pool_ems_message)
+
     def _create_lun(self, volume_name, lun_name, size,
-                    metadata, qos_policy_group_name=None):
+                    metadata, qos_policy_group_name=None,
+                    qos_policy_group_is_adaptive=False):
         """Creates a LUN, handling Data ONTAP differences as needed."""
 
         self.zapi_client.create_lun(
-            volume_name, lun_name, size, metadata, qos_policy_group_name)
+            volume_name, lun_name, size, metadata, qos_policy_group_name,
+            qos_policy_group_is_adaptive)
 
     def _create_lun_handle(self, metadata, vserver=None):
         """Returns LUN handle based on filer type."""
@@ -181,19 +194,22 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
 
     def _clone_lun(self, name, new_name, space_reserved=None,
                    qos_policy_group_name=None, src_block=0, dest_block=0,
-                   block_count=0, source_snapshot=None, is_snapshot=False):
+                   block_count=0, source_snapshot=None, is_snapshot=False,
+                   qos_policy_group_is_adaptive=False):
         """Clone LUN with the given handle to the new name."""
         if not space_reserved:
             space_reserved = self.lun_space_reservation
         metadata = self._get_lun_attr(name, 'metadata')
         volume = metadata['Volume']
 
-        self.zapi_client.clone_lun(volume, name, new_name, space_reserved,
-                                   qos_policy_group_name=qos_policy_group_name,
-                                   src_block=src_block, dest_block=dest_block,
-                                   block_count=block_count,
-                                   source_snapshot=source_snapshot,
-                                   is_snapshot=is_snapshot)
+        self.zapi_client.clone_lun(
+            volume, name, new_name, space_reserved,
+            qos_policy_group_name=qos_policy_group_name,
+            src_block=src_block, dest_block=dest_block,
+            block_count=block_count,
+            source_snapshot=source_snapshot,
+            is_snapshot=is_snapshot,
+            qos_policy_group_is_adaptive=qos_policy_group_is_adaptive)
 
         LOG.debug("Cloned LUN with new name %s", new_name)
         lun = self.zapi_client.get_lun_by_args(vserver=self.vserver,
@@ -246,7 +262,6 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         # Used for service state report
         data['replication_enabled'] = self.replication_enabled
 
-        self.zapi_client.provide_ems(self, self.driver_name, self.app_version)
         self._stats = data
 
     def _get_pool_stats(self, filter_function=None, goodness_function=None):
@@ -263,12 +278,18 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         if not ssc:
             return pools
 
-        # Get up-to-date node utilization metrics just once
-        self.perf_library.update_performance_cache(ssc)
+        # Utilization and performance metrics require cluster-scoped
+        # credentials
+        if self.using_cluster_credentials:
+            # Get up-to-date node utilization metrics just once
+            self.perf_library.update_performance_cache(ssc)
 
-        # Get up-to-date aggregate capacities just once
-        aggregates = self.ssc_library.get_ssc_aggregates()
-        aggr_capacities = self.zapi_client.get_aggregate_capacities(aggregates)
+            # Get up-to-date aggregate capacities just once
+            aggregates = self.ssc_library.get_ssc_aggregates()
+            aggr_capacities = self.zapi_client.get_aggregate_capacities(
+                aggregates)
+        else:
+            aggr_capacities = {}
 
         for ssc_vol_name, ssc_vol_info in ssc.items():
 
@@ -278,9 +299,11 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
             pool.update(ssc_vol_info)
 
             # Add driver capabilities and config info
-            pool['QoS_support'] = True
+            pool['QoS_support'] = self.using_cluster_credentials
             pool['multiattach'] = True
+            pool['online_extend_support'] = True
             pool['consistencygroup_support'] = True
+            pool['consistent_group_snapshot_enabled'] = True
             pool['reserved_percentage'] = self.reserved_percentage
             pool['max_over_subscription_ratio'] = (
                 self.max_over_subscription_ratio)
@@ -295,8 +318,13 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
             size_available_gb = capacity['size-available'] / units.Gi
             pool['free_capacity_gb'] = na_utils.round_down(size_available_gb)
 
-            pool['provisioned_capacity_gb'] = round(
-                pool['total_capacity_gb'] - pool['free_capacity_gb'], 2)
+            if self.using_cluster_credentials:
+                dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
+                    ssc_vol_name)
+            else:
+                dedupe_used = 0.0
+            pool['netapp_dedupe_used_percent'] = na_utils.round_down(
+                dedupe_used)
 
             aggregate_name = ssc_vol_info.get('netapp_aggregate')
             aggr_capacity = aggr_capacities.get(aggregate_name, {})
@@ -367,29 +395,6 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         msg = 'Deleted LUN with name %(name)s and QoS info %(qos)s'
         LOG.debug(msg, {'name': volume['name'], 'qos': qos_policy_group_info})
 
-    def _get_preferred_target_from_list(self, target_details_list,
-                                        filter=None):
-        # cDOT iSCSI LIFs do not migrate from controller to controller
-        # in failover.  Rather, an iSCSI LIF must be configured on each
-        # controller and the initiator has to take responsibility for
-        # using a LIF that is UP.  In failover, the iSCSI LIF on the
-        # downed controller goes DOWN until the controller comes back up.
-        #
-        # Currently Nova only accepts a single target when obtaining
-        # target details from Cinder, so we pass back the first portal
-        # with an UP iSCSI LIF.  There are plans to have Nova accept
-        # and try multiple targets.  When that happens, we can and should
-        # remove this filter and return all targets since their operational
-        # state could change between the time we test here and the time
-        # Nova uses the target.
-
-        operational_addresses = (
-            self.zapi_client.get_operational_lif_addresses())
-
-        return (super(NetAppBlockStorageCmodeLibrary, self)
-                ._get_preferred_target_from_list(target_details_list,
-                                                 filter=operational_addresses))
-
     def _setup_qos_for_volume(self, volume, extra_specs):
         try:
             qos_policy_group_info = na_utils.get_valid_qos_policy_group_info(
@@ -401,6 +406,11 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         self.zapi_client.provision_qos_policy_group(qos_policy_group_info)
         return qos_policy_group_info
 
+    def _get_volume_model_update(self, volume):
+        """Provide any updates necessary for a volume being created/managed."""
+        if self.replication_enabled:
+            return {'replication_status': fields.ReplicationStatus.ENABLED}
+
     def _mark_qos_policy_group_for_deletion(self, qos_policy_group_info):
         self.zapi_client.mark_qos_policy_group_for_deletion(
             qos_policy_group_info)
@@ -420,11 +430,162 @@ class NetAppBlockStorageCmodeLibrary(block_base.NetAppBlockStorageLibrary,
         self._mark_qos_policy_group_for_deletion(qos_policy_group_info)
         super(NetAppBlockStorageCmodeLibrary, self).unmanage(volume)
 
-    def failover_host(self, context, volumes, secondary_id=None):
+    def failover_host(self, context, volumes, secondary_id=None, groups=None):
         """Failover a backend to a secondary replication target."""
 
         return self._failover_host(volumes, secondary_id=secondary_id)
 
     def _get_backing_flexvol_names(self):
         """Returns a list of backing flexvol names."""
-        return self.ssc_library.get_ssc().keys()
+
+        ssc = self.ssc_library.get_ssc()
+        return list(ssc.keys())
+
+    def create_group(self, group):
+        """Driver entry point for creating a generic volume group.
+
+        ONTAP does not maintain an actual Group construct. As a result, no
+        communication to the backend is necessary for generic volume group
+        creation.
+
+        :returns: Hard-coded model update for generic volume group model.
+        """
+        model_update = {'status': fields.GroupStatus.AVAILABLE}
+        return model_update
+
+    def delete_group(self, group, volumes):
+        """Driver entry point for deleting a group.
+
+        :returns: Updated group model and list of volume models
+                 for the volumes that were deleted.
+        """
+        model_update = {'status': fields.GroupStatus.DELETED}
+        volumes_model_update = []
+        for volume in volumes:
+            try:
+                self._delete_lun(volume['name'])
+                volumes_model_update.append(
+                    {'id': volume['id'], 'status': 'deleted'})
+            except Exception:
+                volumes_model_update.append(
+                    {'id': volume['id'],
+                     'status': 'error_deleting'})
+                LOG.exception("Volume %(vol)s in the group could not be "
+                              "deleted.", {'vol': volume})
+        return model_update, volumes_model_update
+
+    def update_group(self, group, add_volumes=None, remove_volumes=None):
+        """Driver entry point for updating a generic volume group.
+
+        Since no actual group construct is ever created in ONTAP, it is not
+        necessary to update any metadata on the backend. Since this is a NO-OP,
+        there is guaranteed to be no change in any of the volumes' statuses.
+        """
+        return None, None, None
+
+    def create_group_snapshot(self, group_snapshot, snapshots):
+        """Creates a Cinder group snapshot object.
+
+        The Cinder group snapshot object is created by making use of an
+        ephemeral ONTAP consistency group snapshot in order to provide
+        write-order consistency for a set of flexvol snapshots. First, a list
+        of the flexvols backing the given Cinder group must be gathered. An
+        ONTAP group-snapshot of these flexvols will create a snapshot copy of
+        all the Cinder volumes in the generic volume group. For each Cinder
+        volume in the group, it is then necessary to clone its backing LUN from
+        the ONTAP cg-snapshot. The naming convention used for the clones is
+        what indicates the clone's role as a Cinder snapshot and its inclusion
+        in a Cinder group. The ONTAP cg-snapshot of the flexvols is no longer
+        required after having cloned the LUNs backing the Cinder volumes in
+        the Cinder group.
+
+        :returns: An implicit update for group snapshot and snapshots models
+                 that is interpreted by the manager to set their models to
+                 available.
+        """
+        try:
+            if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
+                self._create_consistent_group_snapshot(group_snapshot,
+                                                       snapshots)
+            else:
+                for snapshot in snapshots:
+                    self._create_snapshot(snapshot)
+        except Exception as ex:
+            err_msg = (_("Create group snapshot failed (%s).") % ex)
+            LOG.exception(err_msg, resource=group_snapshot)
+            raise na_utils.NetAppDriverException(err_msg)
+
+        return None, None
+
+    def _create_consistent_group_snapshot(self, group_snapshot, snapshots):
+        flexvols = set()
+        for snapshot in snapshots:
+            flexvols.add(volume_utils.extract_host(
+                snapshot['volume']['host'], level='pool'))
+
+        self.zapi_client.create_cg_snapshot(flexvols, group_snapshot['id'])
+
+        for snapshot in snapshots:
+            self._clone_lun(snapshot['volume']['name'], snapshot['name'],
+                            source_snapshot=group_snapshot['id'])
+
+        for flexvol in flexvols:
+            try:
+                self.zapi_client.wait_for_busy_snapshot(
+                    flexvol, group_snapshot['id'])
+                self.zapi_client.delete_snapshot(
+                    flexvol, group_snapshot['id'])
+            except exception.SnapshotIsBusy:
+                self.zapi_client.mark_snapshot_for_deletion(
+                    flexvol, group_snapshot['id'])
+
+    def delete_group_snapshot(self, group_snapshot, snapshots):
+        """Delete LUNs backing each snapshot in the group snapshot.
+
+        :returns: An implicit update for snapshots models that is interpreted
+                 by the manager to set their models to deleted.
+        """
+        for snapshot in snapshots:
+            self._delete_lun(snapshot['name'])
+            LOG.debug("Snapshot %s deletion successful", snapshot['name'])
+
+        return None, None
+
+    def create_group_from_src(self, group, volumes, group_snapshot=None,
+                              snapshots=None, source_group=None,
+                              source_vols=None):
+        """Creates a group from a group snapshot or a group of cinder vols.
+
+        :returns: An implicit update for the volumes model that is
+                 interpreted by the manager as a successful operation.
+        """
+        LOG.debug("VOLUMES %s ", ', '.join([vol['id'] for vol in volumes]))
+        volume_model_updates = []
+
+        if group_snapshot:
+            vols = zip(volumes, snapshots)
+
+            for volume, snapshot in vols:
+                source = {
+                    'name': snapshot['name'],
+                    'size': snapshot['volume_size'],
+                }
+                volume_model_update = self._clone_source_to_destination(
+                    source, volume)
+                if volume_model_update is not None:
+                    volume_model_update['id'] = volume['id']
+                    volume_model_updates.append(volume_model_update)
+
+        else:
+            vols = zip(volumes, source_vols)
+
+            for volume, old_src_vref in vols:
+                src_lun = self._get_lun_from_table(old_src_vref['name'])
+                source = {'name': src_lun.name, 'size': old_src_vref['size']}
+                volume_model_update = self._clone_source_to_destination(
+                    source, volume)
+                if volume_model_update is not None:
+                    volume_model_update['id'] = volume['id']
+                    volume_model_updates.append(volume_model_update)
+
+        return None, volume_model_updates
