@@ -2,6 +2,7 @@
 # Copyright (c) 2014 Clinton Knight.  All rights reserved.
 # Copyright (c) 2015 Tom Barron.  All rights reserved.
 # Copyright (c) 2016 Mike Rooney. All rights reserved.
+# Copyright (c) 2017 Jose Porrua. All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -20,18 +21,17 @@ import math
 import re
 
 from oslo_log import log as logging
+from oslo_utils import strutils
 from oslo_utils import units
 import six
 
 from cinder import exception
-from cinder.i18n import _, _LW, _LE
+from cinder.i18n import _
 from cinder import utils
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.client import client_base
 from cinder.volume.drivers.netapp import utils as na_utils
-
-from oslo_utils import strutils
-
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 DEFAULT_MAX_PAGE_LENGTH = 50
@@ -50,6 +50,8 @@ class Client(client_base.Client):
         (major, minor) = self.get_ontapi_version(cached=False)
         self.connection.set_api_version(major, minor)
         self._init_features()
+        ontap_version = self.get_ontap_version(cached=False)
+        self.connection.set_ontap_version(ontap_version)
 
     def _init_features(self):
         super(Client, self)._init_features()
@@ -60,11 +62,13 @@ class Client(client_base.Client):
         ontapi_1_2x = (1, 20) <= ontapi_version < (1, 30)
         ontapi_1_30 = ontapi_version >= (1, 30)
         ontapi_1_100 = ontapi_version >= (1, 100)
+        ontapi_1_1xx = (1, 100) <= ontapi_version < (1, 200)
 
         self.features.add_feature('SNAPMIRROR_V2', supported=ontapi_1_20)
         self.features.add_feature('USER_CAPABILITY_LIST',
                                   supported=ontapi_1_20)
         self.features.add_feature('SYSTEM_METRICS', supported=ontapi_1_2x)
+        self.features.add_feature('CLONE_SPLIT_STATUS', supported=ontapi_1_30)
         self.features.add_feature('FAST_CLONE_DELETE', supported=ontapi_1_30)
         self.features.add_feature('SYSTEM_CONSTITUENT_METRICS',
                                   supported=ontapi_1_30)
@@ -72,6 +76,10 @@ class Client(client_base.Client):
                                   supported=ontapi_1_30)
         self.features.add_feature('BACKUP_CLONE_PARAM', supported=ontapi_1_100)
         self.features.add_feature('CLUSTER_PEER_POLICY', supported=ontapi_1_30)
+        self.features.add_feature('FLEXVOL_ENCRYPTION', supported=ontapi_1_1xx)
+
+        LOG.info('Reported ONTAPI Version: %(major)s.%(minor)s',
+                 {'major': ontapi_version[0], 'minor': ontapi_version[1]})
 
     def _invoke_vserver_api(self, na_element, vserver):
         server = copy.copy(self.connection)
@@ -88,9 +96,10 @@ class Client(client_base.Client):
             return int(api_result_element.get_child_content('num-records'))
         except TypeError:
             msg = _('Missing record count for NetApp iterator API invocation.')
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
 
     def set_vserver(self, vserver):
+        self.vserver = vserver
         self.connection.set_vserver(vserver)
 
     def send_iter_request(self, api_name, api_args=None, enable_tunneling=True,
@@ -103,7 +112,7 @@ class Client(client_base.Client):
         api_args['max-records'] = max_page_length
 
         # Get first page
-        result = self.send_request(
+        result = self.connection.send_request(
             api_name, api_args, enable_tunneling=enable_tunneling)
 
         # Most commonly, we can just return here if there is no more data
@@ -116,13 +125,13 @@ class Client(client_base.Client):
         attributes_list = result.get_child_by_name('attributes-list')
         if not attributes_list:
             msg = _('Missing attributes list for API %s.') % api_name
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
 
         # Get remaining pages, saving data into first page
         while next_tag is not None:
             next_api_args = copy.deepcopy(api_args)
             next_api_args['tag'] = next_tag
-            next_result = self.send_request(
+            next_result = self.connection.send_request(
                 api_name, next_api_args, enable_tunneling=enable_tunneling)
 
             next_attributes_list = next_result.get_child_by_name(
@@ -139,6 +148,69 @@ class Client(client_base.Client):
         result.get_child_by_name('next-tag').set_content('')
         return result
 
+    def list_vservers(self, vserver_type='data'):
+        """Get the names of vservers present, optionally filtered by type."""
+        query = {
+            'vserver-info': {
+                'vserver-type': vserver_type,
+            }
+        } if vserver_type else None
+
+        api_args = {
+            'desired-attributes': {
+                'vserver-info': {
+                    'vserver-name': None,
+                },
+            },
+        }
+        if query:
+            api_args['query'] = query
+
+        result = self.send_iter_request('vserver-get-iter', api_args,
+                                        enable_tunneling=False)
+        vserver_info_list = result.get_child_by_name(
+            'attributes-list') or netapp_api.NaElement('none')
+        return [vserver_info.get_child_content('vserver-name')
+                for vserver_info in vserver_info_list.get_children()]
+
+    def _get_ems_log_destination_vserver(self):
+        """Returns the best vserver destination for EMS messages."""
+        major, minor = self.get_ontapi_version(cached=True)
+
+        if (major > 1) or (major == 1 and minor > 15):
+            # Prefer admin Vserver (requires cluster credentials).
+            admin_vservers = self.list_vservers(vserver_type='admin')
+            if admin_vservers:
+                return admin_vservers[0]
+
+            # Fall back to data Vserver.
+            data_vservers = self.list_vservers(vserver_type='data')
+            if data_vservers:
+                return data_vservers[0]
+
+        # If older API version, or no other Vservers found, use node Vserver.
+        node_vservers = self.list_vservers(vserver_type='node')
+        if node_vservers:
+            return node_vservers[0]
+
+        raise exception.NotFound("No Vserver found to receive EMS messages.")
+
+    def send_ems_log_message(self, message_dict):
+        """Sends a message to the Data ONTAP EMS log."""
+
+        # NOTE(cknight): Cannot use deepcopy on the connection context
+        node_client = copy.copy(self)
+        node_client.connection = copy.copy(self.connection)
+        node_client.connection.set_timeout(25)
+
+        try:
+            node_client.set_vserver(self._get_ems_log_destination_vserver())
+            node_client.connection.send_request('ems-autosupport-log',
+                                                message_dict)
+            LOG.debug('EMS executed successfully.')
+        except netapp_api.NaApiError as e:
+            LOG.warning('Failed to invoke EMS. %s', e)
+
     def get_iscsi_target_details(self):
         """Gets the iSCSI target portal details."""
         iscsi_if_iter = netapp_api.NaElement('iscsi-interface-get-iter')
@@ -381,20 +453,18 @@ class Client(client_base.Client):
 
     def clone_lun(self, volume, name, new_name, space_reserved='true',
                   qos_policy_group_name=None, src_block=0, dest_block=0,
-                  block_count=0, source_snapshot=None, is_snapshot=False):
-        # zAPI can only handle 2^24 blocks per range
-        bc_limit = 2 ** 24  # 8GB
-        # zAPI can only handle 32 block ranges per call
-        br_limit = 32
-        z_limit = br_limit * bc_limit  # 256 GB
-        z_calls = int(math.ceil(block_count / float(z_limit)))
+                  block_count=0, source_snapshot=None, is_snapshot=False,
+                  qos_policy_group_is_adaptive=False):
+        # ONTAP handles only 128 MB per call as of v9.1
+        bc_limit = 2 ** 18  # 2^18 blocks * 512 bytes/block = 128 MB
+        z_calls = int(math.ceil(block_count / float(bc_limit)))
         zbc = block_count
         if z_calls == 0:
             z_calls = 1
         for _call in range(0, z_calls):
-            if zbc > z_limit:
-                block_count = z_limit
-                zbc -= z_limit
+            if zbc > bc_limit:
+                block_count = bc_limit
+                zbc -= bc_limit
             else:
                 block_count = zbc
 
@@ -411,8 +481,13 @@ class Client(client_base.Client):
             clone_create = netapp_api.NaElement.create_node_with_children(
                 'clone-create', **zapi_args)
             if qos_policy_group_name is not None:
-                clone_create.add_new_child('qos-policy-group-name',
-                                           qos_policy_group_name)
+                if qos_policy_group_is_adaptive:
+                    clone_create.add_new_child(
+                        'qos-adaptive-policy-group-name',
+                        qos_policy_group_name)
+                else:
+                    clone_create.add_new_child('qos-policy-group-name',
+                                               qos_policy_group_name)
             if block_count > 0:
                 block_ranges = netapp_api.NaElement("block-ranges")
                 segments = int(math.ceil(block_count / float(bc_limit)))
@@ -431,7 +506,7 @@ class Client(client_base.Client):
                                'destination-block-number':
                                six.text_type(dest_block),
                                'block-count':
-                               six.text_type(block_count)})
+                               six.text_type(int(block_count))})
                     block_ranges.add_child_elem(block_range)
                     src_block += int(block_count)
                     dest_block += int(block_count)
@@ -451,15 +526,18 @@ class Client(client_base.Client):
             return []
         return attr_list.get_children()
 
-    def file_assign_qos(self, flex_vol, qos_policy_group_name, file_path):
+    def file_assign_qos(self, flex_vol, qos_policy_group_name,
+                        qos_policy_group_is_adaptive, file_path):
         """Assigns the named QoS policy-group to a file."""
+        qos_arg_name = "qos-%spolicy-group-name" % (
+            "adaptive-" if qos_policy_group_is_adaptive else "")
         api_args = {
             'volume': flex_vol,
-            'qos-policy-group-name': qos_policy_group_name,
+            qos_arg_name: qos_policy_group_name,
             'file': file_path,
             'vserver': self.vserver,
         }
-        return self.send_request('file-assign-qos', api_args, False)
+        return self.connection.send_request('file-assign-qos', api_args, False)
 
     def provision_qos_policy_group(self, qos_policy_group_info):
         """Create QOS policy group on the backend if appropriate."""
@@ -495,9 +573,9 @@ class Client(client_base.Client):
                 },
             },
         }
-        result = self.send_request('qos-policy-group-get-iter',
-                                   api_args,
-                                   False)
+        result = self.connection.send_request('qos-policy-group-get-iter',
+                                              api_args,
+                                              False)
         return self._has_records(result)
 
     def qos_policy_group_create(self, qos_policy_group_name, max_throughput):
@@ -507,7 +585,8 @@ class Client(client_base.Client):
             'max-throughput': max_throughput,
             'vserver': self.vserver,
         }
-        return self.send_request('qos-policy-group-create', api_args, False)
+        return self.connection.send_request(
+            'qos-policy-group-create', api_args, False)
 
     def qos_policy_group_modify(self, qos_policy_group_name, max_throughput):
         """Modifies a QOS policy group."""
@@ -515,12 +594,14 @@ class Client(client_base.Client):
             'policy-group': qos_policy_group_name,
             'max-throughput': max_throughput,
         }
-        return self.send_request('qos-policy-group-modify', api_args, False)
+        return self.connection.send_request(
+            'qos-policy-group-modify', api_args, False)
 
     def qos_policy_group_delete(self, qos_policy_group_name):
         """Attempts to delete a QOS policy group."""
         api_args = {'policy-group': qos_policy_group_name}
-        return self.send_request('qos-policy-group-delete', api_args, False)
+        return self.connection.send_request(
+            'qos-policy-group-delete', api_args, False)
 
     def qos_policy_group_rename(self, qos_policy_group_name, new_name):
         """Renames a QOS policy group."""
@@ -528,7 +609,8 @@ class Client(client_base.Client):
             'policy-group-name': qos_policy_group_name,
             'new-name': new_name,
         }
-        return self.send_request('qos-policy-group-rename', api_args, False)
+        return self.connection.send_request(
+            'qos-policy-group-rename', api_args, False)
 
     def mark_qos_policy_group_for_deletion(self, qos_policy_group_info):
         """Do (soft) delete of backing QOS policy group for a cinder volume."""
@@ -549,9 +631,9 @@ class Client(client_base.Client):
             try:
                 self.qos_policy_group_rename(current_name, new_name)
             except netapp_api.NaApiError as ex:
-                msg = _LW('Rename failure in cleanup of cDOT QOS policy group '
-                          '%(name)s: %(ex)s')
-                LOG.warning(msg, {'name': current_name, 'ex': ex})
+                LOG.warning('Rename failure in cleanup of cDOT QOS policy '
+                            'group %(name)s: %(ex)s',
+                            {'name': current_name, 'ex': ex})
 
         # Attempt to delete any QoS policies named "delete-openstack-*".
         self.remove_unused_qos_policy_groups()
@@ -572,11 +654,12 @@ class Client(client_base.Client):
         }
 
         try:
-            self.send_request('qos-policy-group-delete-iter', api_args, False)
+            self.connection.send_request(
+                'qos-policy-group-delete-iter', api_args, False)
         except netapp_api.NaApiError as ex:
             msg = 'Could not delete QOS policy groups. Details: %(ex)s'
             msg_args = {'ex': ex}
-            LOG.debug(msg % msg_args)
+            LOG.debug(msg, msg_args)
 
     def set_lun_qos_policy_group(self, path, qos_policy_group):
         """Sets qos_policy_group on a LUN."""
@@ -584,7 +667,8 @@ class Client(client_base.Client):
             'path': path,
             'qos-policy-group': qos_policy_group,
         }
-        return self.send_request('lun-set-qos-policy-group', api_args)
+        return self.connection.send_request(
+            'lun-set-qos-policy-group', api_args)
 
     def get_if_info_by_ip(self, ip):
         """Gets the network interface info by ip."""
@@ -594,7 +678,7 @@ class Client(client_base.Client):
         net_if_iter.add_child_elem(query)
         query.add_node_with_children(
             'net-interface-info',
-            **{'address': na_utils.resolve_hostname(ip)})
+            **{'address': volume_utils.resolve_hostname(ip)})
         result = self.connection.invoke_successfully(net_if_iter, True)
         num_records = result.get_child_content('num-records')
         if num_records and int(num_records) >= 1:
@@ -710,7 +794,7 @@ class Client(client_base.Client):
                 },
             },
         }
-        result = self.send_request(
+        result = self.connection.send_request(
             'system-user-capability-get-iter', api_args, False)
 
         if not self._has_records(result):
@@ -745,13 +829,29 @@ class Client(client_base.Client):
             raise ValueError(_('Non-getter API passed to API test method.'))
 
         try:
-            self.send_request(api, enable_tunneling=False)
+            self.connection.send_request(api, enable_tunneling=False)
         except netapp_api.NaApiError as ex:
             if ex.code in (netapp_api.EAPIPRIVILEGE, netapp_api.EAPINOTFOUND):
                 return False
 
         return True
 
+    def list_cluster_nodes(self):
+        """Get all available cluster nodes."""
+
+        api_args = {
+            'desired-attributes': {
+                'node-details-info': {
+                    'node': None,
+                },
+            },
+        }
+        result = self.send_iter_request('system-node-get-iter', api_args)
+        nodes_info_list = result.get_child_by_name(
+            'attributes-list') or netapp_api.NaElement('none')
+        return [node_info.get_child_content('node') for node_info
+                in nodes_info_list.get_children()]
+
     def get_operational_lif_addresses(self):
         """Gets the IP addresses of operational LIFs on the vserver."""
 
@@ -805,7 +905,7 @@ class Client(client_base.Client):
         if self._get_record_count(result) != 1:
             msg = _('Volume %s not found.')
             msg_args = flexvol_path or flexvol_name
-            raise exception.NetAppDriverException(msg % msg_args)
+            raise na_utils.NetAppDriverException(msg % msg_args)
 
         attributes_list = result.get_child_by_name('attributes-list')
         volume_attributes = attributes_list.get_child_by_name(
@@ -984,19 +1084,28 @@ class Client(client_base.Client):
                 'sis-status-info': {
                     'state': None,
                     'is-compression-enabled': None,
+                    'logical-data-size': None,
+                    'logical-data-limit': None,
                 },
             },
         }
 
+        no_dedupe_response = {
+            'compression': False,
+            'dedupe': False,
+            'logical-data-size': 0,
+            'logical-data-limit': 1,
+        }
+
         try:
             result = self.send_iter_request('sis-get-iter', api_args)
         except netapp_api.NaApiError:
-            msg = _LE('Failed to get dedupe info for volume %s.')
-            LOG.exception(msg, flexvol_name)
-            return {'compression': False, 'dedupe': False}
+            LOG.exception('Failed to get dedupe info for volume %s.',
+                          flexvol_name)
+            return no_dedupe_response
 
         if self._get_record_count(result) != 1:
-            return {'compression': False, 'dedupe': False}
+            return no_dedupe_response
 
         attributes_list = result.get_child_by_name(
             'attributes-list') or netapp_api.NaElement('none')
@@ -1004,15 +1113,63 @@ class Client(client_base.Client):
         sis_status_info = attributes_list.get_child_by_name(
             'sis-status-info') or netapp_api.NaElement('none')
 
+        logical_data_size = sis_status_info.get_child_content(
+            'logical-data-size') or 0
+        logical_data_limit = sis_status_info.get_child_content(
+            'logical-data-limit') or 1
+
         sis = {
             'compression': strutils.bool_from_string(
                 sis_status_info.get_child_content('is-compression-enabled')),
             'dedupe': na_utils.to_bool(
                 sis_status_info.get_child_content('state')),
+            'logical-data-size': int(logical_data_size),
+            'logical-data-limit': int(logical_data_limit),
         }
 
         return sis
 
+    def get_flexvol_dedupe_used_percent(self, flexvol_name):
+        """Determine how close a flexvol is to its shared block limit."""
+
+        # Note(cknight): The value returned by this method is computed from
+        # values returned by two different APIs, one of which was new in
+        # Data ONTAP 8.3.
+        if not self.features.CLONE_SPLIT_STATUS:
+            return 0.0
+
+        dedupe_info = self.get_flexvol_dedupe_info(flexvol_name)
+        clone_split_info = self.get_clone_split_info(flexvol_name)
+
+        total_dedupe_blocks = (dedupe_info.get('logical-data-size') +
+                               clone_split_info.get('unsplit-size'))
+        dedupe_used_percent = (100.0 * float(total_dedupe_blocks) /
+                               dedupe_info.get('logical-data-limit'))
+        return dedupe_used_percent
+
+    def get_clone_split_info(self, flexvol_name):
+        """Get the status of unsplit file/LUN clones in a flexvol."""
+
+        try:
+            result = self.connection.send_request(
+                'clone-split-status', {'volume-name': flexvol_name})
+        except netapp_api.NaApiError:
+            LOG.exception('Failed to get clone split info for volume %s.',
+                          flexvol_name)
+            return {'unsplit-size': 0, 'unsplit-clone-count': 0}
+
+        clone_split_info = result.get_child_by_name(
+            'clone-split-info') or netapp_api.NaElement('none')
+
+        unsplit_size = clone_split_info.get_child_content('unsplit-size') or 0
+        unsplit_clone_count = clone_split_info.get_child_content(
+            'unsplit-clone-count') or 0
+
+        return {
+            'unsplit-size': int(unsplit_size),
+            'unsplit-clone-count': int(unsplit_clone_count),
+        }
+
     def is_flexvol_mirrored(self, flexvol_name, vserver_name):
         """Check if flexvol is a SnapMirror source."""
 
@@ -1033,8 +1190,43 @@ class Client(client_base.Client):
         try:
             result = self.send_iter_request('snapmirror-get-iter', api_args)
         except netapp_api.NaApiError:
-            msg = _LE('Failed to get SnapMirror info for volume %s.')
-            LOG.exception(msg, flexvol_name)
+            LOG.exception('Failed to get SnapMirror info for volume %s.',
+                          flexvol_name)
+            return False
+
+        if not self._has_records(result):
+            return False
+
+        return True
+
+    def is_flexvol_encrypted(self, flexvol_name, vserver_name):
+        """Check if a flexvol is encrypted."""
+
+        if not self.features.FLEXVOL_ENCRYPTION:
+            return False
+
+        api_args = {
+            'query': {
+                'volume-attributes': {
+                    'encrypt': 'true',
+                    'volume-id-attributes': {
+                        'name': flexvol_name,
+                        'owning-vserver-name': vserver_name,
+                    },
+                },
+            },
+            'desired-attributes': {
+                'volume-attributes': {
+                    'encrypt': None,
+                },
+            },
+        }
+
+        try:
+            result = self.send_iter_request('volume-get-iter', api_args)
+        except netapp_api.NaApiError:
+            LOG.exception('Failed to get Encryption info for volume %s.',
+                          flexvol_name)
             return False
 
         if not self._has_records(result):
@@ -1068,7 +1260,7 @@ class Client(client_base.Client):
         if snapshot_reserve is not None:
             api_args['percentage-snapshot-reserve'] = six.text_type(
                 snapshot_reserve)
-        self.send_request('volume-create', api_args)
+        self.connection.send_request('volume-create', api_args)
 
         # cDOT compression requires that deduplication be enabled.
         if dedupe_enabled or compression_enabled:
@@ -1105,7 +1297,7 @@ class Client(client_base.Client):
             'volume': orig_flexvol_name,
             'new-volume-name': new_flexvol_name,
         }
-        self.send_request('volume-rename', api_args)
+        self.connection.send_request('volume-rename', api_args)
 
     def mount_flexvol(self, flexvol_name, junction_path=None):
         """Mounts a volume on a junction path."""
@@ -1114,17 +1306,17 @@ class Client(client_base.Client):
             'junction-path': (junction_path if junction_path
                               else '/%s' % flexvol_name)
         }
-        self.send_request('volume-mount', api_args)
+        self.connection.send_request('volume-mount', api_args)
 
     def enable_flexvol_dedupe(self, flexvol_name):
         """Enable deduplication on volume."""
         api_args = {'path': '/vol/%s' % flexvol_name}
-        self.send_request('sis-enable', api_args)
+        self.connection.send_request('sis-enable', api_args)
 
     def disable_flexvol_dedupe(self, flexvol_name):
         """Disable deduplication on volume."""
         api_args = {'path': '/vol/%s' % flexvol_name}
-        self.send_request('sis-disable', api_args)
+        self.connection.send_request('sis-disable', api_args)
 
     def enable_flexvol_compression(self, flexvol_name):
         """Enable compression on volume."""
@@ -1132,7 +1324,7 @@ class Client(client_base.Client):
             'path': '/vol/%s' % flexvol_name,
             'enable-compression': 'true'
         }
-        self.send_request('sis-set-config', api_args)
+        self.connection.send_request('sis-set-config', api_args)
 
     def disable_flexvol_compression(self, flexvol_name):
         """Disable compression on volume."""
@@ -1140,7 +1332,7 @@ class Client(client_base.Client):
             'path': '/vol/%s' % flexvol_name,
             'enable-compression': 'false'
         }
-        self.send_request('sis-set-config', api_args)
+        self.connection.send_request('sis-set-config', api_args)
 
     @utils.trace_method
     def delete_file(self, path_to_file):
@@ -1152,7 +1344,7 @@ class Client(client_base.Client):
         # Use fast clone deletion engine if it is supported.
         if self.features.FAST_CLONE_DELETE:
             api_args['is-clone-file'] = 'true'
-        self.send_request('file-delete-file', api_args, True)
+        self.connection.send_request('file-delete-file', api_args, True)
 
     def _get_aggregates(self, aggregate_names=None, desired_attributes=None):
 
@@ -1168,9 +1360,9 @@ class Client(client_base.Client):
         if desired_attributes:
             api_args['desired-attributes'] = desired_attributes
 
-        result = self.send_request('aggr-get-iter',
-                                   api_args,
-                                   enable_tunneling=False)
+        result = self.connection.send_request('aggr-get-iter',
+                                              api_args,
+                                              enable_tunneling=False)
         if not self._has_records(result):
             return []
         else:
@@ -1231,8 +1423,8 @@ class Client(client_base.Client):
             aggrs = self._get_aggregates(aggregate_names=[aggregate_name],
                                          desired_attributes=desired_attributes)
         except netapp_api.NaApiError:
-            msg = _LE('Failed to get info for aggregate %s.')
-            LOG.exception(msg, aggregate_name)
+            LOG.exception('Failed to get info for aggregate %s.',
+                          aggregate_name)
             return {}
 
         if len(aggrs) < 1:
@@ -1303,8 +1495,8 @@ class Client(client_base.Client):
             result = self.send_iter_request(
                 'storage-disk-get-iter', api_args, enable_tunneling=False)
         except netapp_api.NaApiError:
-            msg = _LE('Failed to get disk info for aggregate %s.')
-            LOG.exception(msg, aggregate_name)
+            LOG.exception('Failed to get disk info for aggregate %s.',
+                          aggregate_name)
             return disk_types
 
         attributes_list = result.get_child_by_name(
@@ -1350,9 +1542,13 @@ class Client(client_base.Client):
         try:
             aggrs = self._get_aggregates(aggregate_names=[aggregate_name],
                                          desired_attributes=desired_attributes)
-        except netapp_api.NaApiError:
-            msg = _LE('Failed to get info for aggregate %s.')
-            LOG.exception(msg, aggregate_name)
+        except netapp_api.NaApiError as e:
+            if e.code == netapp_api.EAPINOTFOUND:
+                LOG.debug('Aggregate capacity can only be collected with '
+                          'cluster scoped credentials.')
+            else:
+                LOG.exception('Failed to get info for aggregate %s.',
+                              aggregate_name)
             return {}
 
         if len(aggrs) < 1:
@@ -1387,9 +1583,9 @@ class Client(client_base.Client):
             }
         }
 
-        result = self.send_request('perf-object-instance-list-info-iter',
-                                   api_args,
-                                   enable_tunneling=False)
+        result = self.connection.send_request(
+            'perf-object-instance-list-info-iter', api_args,
+            enable_tunneling=False)
 
         uuids = []
 
@@ -1403,7 +1599,7 @@ class Client(client_base.Client):
 
     def get_performance_counters(self, object_name, instance_uuids,
                                  counter_names):
-        """Gets or or more cDOT performance counters."""
+        """Gets more cDOT performance counters."""
 
         api_args = {
             'objectname': object_name,
@@ -1416,9 +1612,8 @@ class Client(client_base.Client):
             ],
         }
 
-        result = self.send_request('perf-object-get-instances',
-                                   api_args,
-                                   enable_tunneling=False)
+        result = self.connection.send_request(
+            'perf-object-get-instances', api_args, enable_tunneling=False)
 
         counter_data = []
 
@@ -1449,12 +1644,8 @@ class Client(client_base.Client):
 
         return counter_data
 
-    def get_snapshots_marked_for_deletion(self, volume_list=None):
-        """Get a list of snapshots marked for deletion.
-
-        :param volume_list: placeholder parameter to match 7mode client method
-        signature.
-        """
+    def get_snapshots_marked_for_deletion(self):
+        """Get a list of snapshots marked for deletion."""
 
         api_args = {
             'query': {
@@ -1473,7 +1664,7 @@ class Client(client_base.Client):
             },
         }
 
-        result = self.send_request('snapshot-get-iter', api_args)
+        result = self.connection.send_request('snapshot-get-iter', api_args)
 
         snapshots = []
 
@@ -1514,7 +1705,7 @@ class Client(client_base.Client):
                 },
             },
         }
-        result = self.send_request('snapshot-get-iter', api_args)
+        result = self.connection.send_request('snapshot-get-iter', api_args)
 
         self._handle_get_snapshot_return_failure(result, snapshot_name)
 
@@ -1589,7 +1780,7 @@ class Client(client_base.Client):
         if passphrase:
             api_args['passphrase'] = passphrase
 
-        self.send_request('cluster-peer-create', api_args)
+        self.connection.send_request('cluster-peer-create', api_args)
 
     def get_cluster_peers(self, remote_cluster_name=None):
         """Gets one or more cluster peer relationships."""
@@ -1647,7 +1838,7 @@ class Client(client_base.Client):
         """Deletes a cluster peer relationship."""
 
         api_args = {'cluster-name': cluster_name}
-        self.send_request('cluster-peer-delete', api_args)
+        self.connection.send_request('cluster-peer-delete', api_args)
 
     def get_cluster_peer_policy(self):
         """Gets the cluster peering policy configuration."""
@@ -1655,7 +1846,7 @@ class Client(client_base.Client):
         if not self.features.CLUSTER_PEER_POLICY:
             return {}
 
-        result = self.send_request('cluster-peer-policy-get')
+        result = self.connection.send_request('cluster-peer-policy-get')
 
         attributes = result.get_child_by_name(
             'attributes') or netapp_api.NaElement('none')
@@ -1701,7 +1892,7 @@ class Client(client_base.Client):
             api_args['passphrase-minlength'] = six.text_type(
                 passphrase_minimum_length)
 
-        self.send_request('cluster-peer-policy-modify', api_args)
+        self.connection.send_request('cluster-peer-policy-modify', api_args)
 
     def create_vserver_peer(self, vserver_name, peer_vserver_name):
         """Creates a Vserver peer relationship for SnapMirrors."""
@@ -1712,19 +1903,19 @@ class Client(client_base.Client):
                 {'vserver-peer-application': 'snapmirror'},
             ],
         }
-        self.send_request('vserver-peer-create', api_args)
+        self.connection.send_request('vserver-peer-create', api_args)
 
     def delete_vserver_peer(self, vserver_name, peer_vserver_name):
         """Deletes a Vserver peer relationship."""
 
         api_args = {'vserver': vserver_name, 'peer-vserver': peer_vserver_name}
-        self.send_request('vserver-peer-delete', api_args)
+        self.connection.send_request('vserver-peer-delete', api_args)
 
     def accept_vserver_peer(self, vserver_name, peer_vserver_name):
         """Accepts a pending Vserver peer relationship."""
 
         api_args = {'vserver': vserver_name, 'peer-vserver': peer_vserver_name}
-        self.send_request('vserver-peer-accept', api_args)
+        self.connection.send_request('vserver-peer-accept', api_args)
 
     def get_vserver_peers(self, vserver_name=None, peer_vserver_name=None):
         """Gets one or more Vserver peer relationships."""
@@ -1765,7 +1956,7 @@ class Client(client_base.Client):
         """Verify support for SnapMirror control plane v2."""
         if not self.features.SNAPMIRROR_V2:
             msg = _('SnapMirror features require Data ONTAP 8.2 or later.')
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
 
     def create_snapmirror(self, source_vserver, source_volume,
                           destination_vserver, destination_volume,
@@ -1787,7 +1978,7 @@ class Client(client_base.Client):
             api_args['policy'] = policy
 
         try:
-            self.send_request('snapmirror-create', api_args)
+            self.connection.send_request('snapmirror-create', api_args)
         except netapp_api.NaApiError as e:
             if e.code != netapp_api.ERELATION_EXISTS:
                 raise
@@ -1809,7 +2000,8 @@ class Client(client_base.Client):
         if transfer_priority:
             api_args['transfer-priority'] = transfer_priority
 
-        result = self.send_request('snapmirror-initialize', api_args)
+        result = self.connection.send_request('snapmirror-initialize',
+                                              api_args)
 
         result_info = {}
         result_info['operation-id'] = result.get_child_content(
@@ -1841,7 +2033,7 @@ class Client(client_base.Client):
                 }
             }
         }
-        self.send_request('snapmirror-release-iter', api_args)
+        self.connection.send_request('snapmirror-release-iter', api_args)
 
     def quiesce_snapmirror(self, source_vserver, source_volume,
                            destination_vserver, destination_volume):
@@ -1854,7 +2046,7 @@ class Client(client_base.Client):
             'destination-volume': destination_volume,
             'destination-vserver': destination_vserver,
         }
-        self.send_request('snapmirror-quiesce', api_args)
+        self.connection.send_request('snapmirror-quiesce', api_args)
 
     def abort_snapmirror(self, source_vserver, source_volume,
                          destination_vserver, destination_volume,
@@ -1870,7 +2062,7 @@ class Client(client_base.Client):
             'clear-checkpoint': 'true' if clear_checkpoint else 'false',
         }
         try:
-            self.send_request('snapmirror-abort', api_args)
+            self.connection.send_request('snapmirror-abort', api_args)
         except netapp_api.NaApiError as e:
             if e.code != netapp_api.ENOTRANSFER_IN_PROGRESS:
                 raise
@@ -1886,7 +2078,7 @@ class Client(client_base.Client):
             'destination-volume': destination_volume,
             'destination-vserver': destination_vserver,
         }
-        self.send_request('snapmirror-break', api_args)
+        self.connection.send_request('snapmirror-break', api_args)
 
     def modify_snapmirror(self, source_vserver, source_volume,
                           destination_vserver, destination_volume,
@@ -1910,7 +2102,7 @@ class Client(client_base.Client):
         if max_transfer_rate is not None:
             api_args['max-transfer-rate'] = max_transfer_rate
 
-        self.send_request('snapmirror-modify', api_args)
+        self.connection.send_request('snapmirror-modify', api_args)
 
     def delete_snapmirror(self, source_vserver, source_volume,
                           destination_vserver, destination_volume):
@@ -1927,7 +2119,7 @@ class Client(client_base.Client):
                 }
             }
         }
-        self.send_request('snapmirror-destroy-iter', api_args)
+        self.connection.send_request('snapmirror-destroy-iter', api_args)
 
     def update_snapmirror(self, source_vserver, source_volume,
                           destination_vserver, destination_volume):
@@ -1941,7 +2133,7 @@ class Client(client_base.Client):
             'destination-vserver': destination_vserver,
         }
         try:
-            self.send_request('snapmirror-update', api_args)
+            self.connection.send_request('snapmirror-update', api_args)
         except netapp_api.NaApiError as e:
             if (e.code != netapp_api.ETRANSFER_IN_PROGRESS and
                     e.code != netapp_api.EANOTHER_OP_ACTIVE):
@@ -1959,7 +2151,7 @@ class Client(client_base.Client):
             'destination-vserver': destination_vserver,
         }
         try:
-            self.send_request('snapmirror-resume', api_args)
+            self.connection.send_request('snapmirror-resume', api_args)
         except netapp_api.NaApiError as e:
             if e.code != netapp_api.ERELATION_NOT_QUIESCED:
                 raise
@@ -1975,7 +2167,7 @@ class Client(client_base.Client):
             'destination-volume': destination_volume,
             'destination-vserver': destination_vserver,
         }
-        self.send_request('snapmirror-resync', api_args)
+        self.connection.send_request('snapmirror-resync', api_args)
 
     def _get_snapmirrors(self, source_vserver=None, source_volume=None,
                          destination_vserver=None, destination_volume=None,
