@@ -42,7 +42,7 @@ from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
 from cinder.volume.drivers.netapp.dataontap.utils import utils as dot_utils
 from cinder.volume.drivers.netapp import options as na_opts
 from cinder.volume.drivers.netapp import utils as na_utils
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -161,11 +161,14 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 self.ssc_library.get_ssc_flexvol_names())
 
     def _do_qos_for_volume(self, volume, extra_specs, cleanup=True):
+        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
+            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
         try:
             qos_policy_group_info = na_utils.get_valid_qos_policy_group_info(
                 volume, extra_specs)
             self.zapi_client.provision_qos_policy_group(qos_policy_group_info)
-            self._set_qos_policy_group_on_volume(volume, qos_policy_group_info)
+            self._set_qos_policy_group_on_volume(volume, qos_policy_group_info,
+                                                 qos_policy_group_is_adaptive)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.error("Setting QoS for %s failed", volume['id'])
@@ -178,7 +181,8 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         if self.replication_enabled:
             return {'replication_status': fields.ReplicationStatus.ENABLED}
 
-    def _set_qos_policy_group_on_volume(self, volume, qos_policy_group_info):
+    def _set_qos_policy_group_on_volume(self, volume, qos_policy_group_info,
+                                        qos_policy_group_is_adaptive):
         if qos_policy_group_info is None:
             return
         qos_policy_group_name = na_utils.get_qos_policy_group_name_from_info(
@@ -187,11 +191,12 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             return
         target_path = '%s' % (volume['name'])
         share = volume_utils.extract_host(volume['host'], level='pool')
-        export_path = share.split(':')[1]
+        __, export_path = na_utils.get_export_host_junction_path(share)
         flex_vol_name = self.zapi_client.get_vol_by_junc_vserver(self.vserver,
                                                                  export_path)
         self.zapi_client.file_assign_qos(flex_vol_name,
                                          qos_policy_group_name,
+                                         qos_policy_group_is_adaptive,
                                          target_path)
 
     def _clone_backing_file_for_volume(self, volume_name, clone_name,
@@ -325,10 +330,9 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         vserver_addresses = self.zapi_client.get_operational_lif_addresses()
 
         for share in self._mounted_shares:
+            host, junction_path = na_utils.get_export_host_junction_path(share)
 
-            host = share.split(':')[0]
-            junction_path = share.split(':')[1]
-            address = na_utils.resolve_hostname(host)
+            address = volume_utils.resolve_hostname(host)
 
             if address not in vserver_addresses:
                 LOG.warning('Address not found for NFS share %s.', share)
@@ -365,7 +369,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         ip_vserver = self._get_vserver_for_ip(ip)
         if ip_vserver and shares:
             for share in shares:
-                ip_sh = share.split(':')[0]
+                ip_sh, __ = na_utils.get_export_host_junction_path(share)
                 sh_vserver = self._get_vserver_for_ip(ip_sh)
                 if sh_vserver == ip_vserver:
                     LOG.debug('Share match found for ip %s', ip)
@@ -464,7 +468,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _get_ip_verify_on_cluster(self, host):
         """Verifies if host on same cluster and returns ip."""
-        ip = na_utils.resolve_hostname(host)
+        ip = volume_utils.resolve_hostname(host)
         vserver = self._get_vserver_for_ip(ip)
         if not vserver:
             raise exception.NotFound(_("Unable to locate an SVM that is "
@@ -541,15 +545,17 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                   volume['id'])
 
     def _get_source_ip_and_path(self, nfs_share, file_name):
-        src_ip = self._get_ip_verify_on_cluster(nfs_share.split(':')[0])
-        src_path = os.path.join(nfs_share.split(':')[1], file_name)
+        host, share_path = na_utils.get_export_host_junction_path(nfs_share)
+        src_ip = self._get_ip_verify_on_cluster(host)
+        src_path = os.path.join(share_path, file_name)
+
         return src_ip, src_path
 
     def _get_destination_ip_and_path(self, volume):
         share = volume_utils.extract_host(volume['host'], level='pool')
-        share_ip_and_path = share.split(":")
-        dest_ip = self._get_ip_verify_on_cluster(share_ip_and_path[0])
-        dest_path = os.path.join(share_ip_and_path[1], volume['name'])
+        share_ip, share_path = na_utils.get_export_host_junction_path(share)
+        dest_ip = self._get_ip_verify_on_cluster(share_ip)
+        dest_path = os.path.join(share_path, volume['name'])
 
         return dest_ip, dest_path
 
@@ -686,7 +692,9 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _get_backing_flexvol_names(self):
         """Returns a list of backing flexvol names."""
-        return self.ssc_library.get_ssc().keys()
+
+        ssc = self.ssc_library.get_ssc()
+        return list(ssc.keys())
 
     def _get_flexvol_names_from_hosts(self, hosts):
         """Returns a set of flexvol names."""
@@ -789,7 +797,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         except Exception as ex:
             err_msg = (_("Create group snapshot failed (%s).") % ex)
             LOG.exception(err_msg, resource=group_snapshot)
-            raise exception.NetAppDriverException(err_msg)
+            raise na_utils.NetAppDriverException(err_msg)
 
         return None, None
 
