@@ -26,7 +26,6 @@ NetApp drivers to achieve the desired functionality.
 import decimal
 import platform
 import re
-import socket
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
@@ -61,6 +60,10 @@ DEFAULT_CHAP_USER_NAME = 'NetApp_iSCSI_CHAP_Username'
 API_TRACE_PATTERN = '(.*)'
 
 
+class NetAppDriverException(exception.VolumeDriverException):
+    message = _("NetApp Cinder Driver exception.")
+
+
 def validate_instantiation(**kwargs):
     """Checks if a driver is instantiated other than by the unified driver.
 
@@ -136,7 +139,7 @@ def setup_api_trace_pattern(api_trace_pattern):
     except (re.error, TypeError):
         msg = _('Cannot parse the API trace pattern. %s is not a '
                 'valid python regular expression.') % api_trace_pattern
-        raise exception.InvalidConfigurationValue(reason=msg)
+        raise exception.InvalidConfigurationValue(msg)
     API_TRACE_PATTERN = api_trace_pattern
 
 
@@ -148,13 +151,6 @@ def trace_filter_func_api(all_args):
     return re.match(API_TRACE_PATTERN, api_name) is not None
 
 
-def resolve_hostname(hostname):
-    """Resolves host name to IP address."""
-    res = socket.getaddrinfo(hostname, None)[0]
-    family, socktype, proto, canonname, sockaddr = res
-    return sockaddr[0]
-
-
 def round_down(value, precision='0.00'):
     return float(decimal.Decimal(six.text_type(value)).quantize(
         decimal.Decimal(precision), rounding=decimal.ROUND_DOWN))
@@ -380,12 +376,11 @@ def get_export_host_junction_path(share):
     if '[' in share and ']' in share:
         try:
             # ipv6
-            host = re.search('\[(.*)\]', share).group(1)
+            host = re.search(r'\[(.*)\]', share).group(1)
             junction_path = share.split(':')[-1]
         except AttributeError:
-            raise exception.NetAppDriverException(_("Share '%s' is "
-                                                    "not in a valid "
-                                                    "format.") % share)
+            raise NetAppDriverException(_("Share '%s' is not in a valid "
+                                          "format.") % share)
     else:
         # ipv4
         path = share.split(':')
@@ -393,9 +388,8 @@ def get_export_host_junction_path(share):
             host = path[0]
             junction_path = path[1]
         else:
-            raise exception.NetAppDriverException(_("Share '%s' is "
-                                                    "not in a valid "
-                                                    "format.") % share)
+            raise NetAppDriverException(_("Share '%s' is not in a valid "
+                                          "format.") % share)
 
     return host, junction_path
 
