@@ -14,10 +14,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import binascii
 import errno
 import os
+import tempfile
 import time
 
+from castellan import key_manager
 from os_brick.remotefs import remotefs as remotefs_brick
 from oslo_concurrency import processutils as putils
 from oslo_config import cfg
@@ -33,7 +36,7 @@ from cinder import interface
 from cinder import utils
 from cinder.volume import configuration
 from cinder.volume.drivers import remotefs
-from cinder.volume import utils as vutils
+from cinder.volume import volume_utils
 
 VERSION = '1.4.0'
 
@@ -56,8 +59,8 @@ nfs_opts = [
                default='$state_path/mnt',
                help='Base dir containing mount points for NFS shares.'),
     cfg.StrOpt('nfs_mount_options',
-               help='Mount options passed to the NFS client. See section '
-                    'of the NFS man page for details.'),
+               help='Mount options passed to the NFS client. See the NFS(5) '
+                    'man page for details.'),
     cfg.IntOpt('nfs_mount_attempts',
                default=3,
                help='The number of attempts to mount NFS shares before '
@@ -118,10 +121,16 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
         self._sparse_copy_volume_data = True
         self.reserved_percentage = self.configuration.reserved_percentage
         self.max_over_subscription_ratio = (
-            vutils.get_max_over_subscription_ratio(
+            volume_utils.get_max_over_subscription_ratio(
                 self.configuration.max_over_subscription_ratio,
                 supports_auto=supports_auto_mosr))
 
+        self._supports_encryption = True
+
+    @staticmethod
+    def get_driver_options():
+        return nfs_opts + remotefs.nas_opts
+
     def initialize_connection(self, volume, connector):
 
         LOG.debug('Initializing connection to volume %(vol)s. '
@@ -359,6 +368,12 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
 
     def extend_volume(self, volume, new_size):
         """Extend an existing volume to the new size."""
+        if self._is_volume_attached(volume):
+            # NOTE(kaisers): no attached extensions until #1870367 is fixed
+            msg = (_("Cannot extend volume %s while it is attached.")
+                   % volume['id'])
+            raise exception.ExtendVolumeError(msg)
+
         LOG.info('Extending volume %s.', volume.id)
         extend_by = int(new_size) - volume.size
         if not self._is_share_eligible(volume.provider_location,
@@ -468,8 +483,8 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
             try:
                 os.rename(current_path, original_path)
             except OSError:
-                LOG.error('Unable to rename the logical volume '
-                          'for volume: %s', volume.id)
+                LOG.exception('Unable to rename the logical volume '
+                              'for volume: %s', volume.id)
                 # If the rename fails, _name_id should be set to the new
                 # volume id and provider_location should be set to the
                 # one from the new volume as well.
@@ -573,7 +588,9 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
         self._check_snapshot_support()
         return self._delete_snapshot(snapshot)
 
-    def _copy_volume_from_snapshot(self, snapshot, volume, volume_size):
+    def _copy_volume_from_snapshot(self, snapshot, volume, volume_size,
+                                   src_encryption_key_id=None,
+                                   new_encryption_key_id=None):
         """Copy data from snapshot to destination volume.
 
         This is done with a qemu-img convert to raw/qcow2 from the snapshot
@@ -606,9 +623,46 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
         else:
             out_format = 'raw'
 
-        image_utils.convert_image(path_to_snap_img,
-                                  path_to_new_vol,
-                                  out_format,
-                                  run_as_root=self._execute_as_root)
+        if new_encryption_key_id is not None:
+            if src_encryption_key_id is None:
+                message = _("Can't create an encrypted volume %(format)s "
+                            "from an unencrypted source."
+                            ) % {'format': out_format}
+                LOG.error(message)
+                # TODO(enriquetaso): handle unencrypted snap->encrypted vol
+                raise exception.NfsException(message)
+            keymgr = key_manager.API(CONF)
+            new_key = keymgr.get(volume.obj_context, new_encryption_key_id)
+            new_passphrase = \
+                binascii.hexlify(new_key.get_encoded()).decode('utf-8')
+
+            # volume.obj_context is the owner of this request
+            src_key = keymgr.get(volume.obj_context, src_encryption_key_id)
+            src_passphrase = \
+                binascii.hexlify(src_key.get_encoded()).decode('utf-8')
+
+            tmp_dir = volume_utils.image_conversion_dir()
+            with tempfile.NamedTemporaryFile(prefix='luks_',
+                                             dir=tmp_dir) as src_pass_file:
+                with open(src_pass_file.name, 'w') as f:
+                    f.write(src_passphrase)
+
+                with tempfile.NamedTemporaryFile(prefix='luks_',
+                                                 dir=tmp_dir) as new_pass_file:
+                    with open(new_pass_file.name, 'w') as f:
+                        f.write(new_passphrase)
+
+                    image_utils.convert_image(
+                        path_to_snap_img,
+                        path_to_new_vol,
+                        'luks',
+                        passphrase_file=new_pass_file.name,
+                        src_passphrase_file=src_pass_file.name,
+                        run_as_root=self._execute_as_root)
+        else:
+            image_utils.convert_image(path_to_snap_img,
+                                      path_to_new_vol,
+                                      out_format,
+                                      run_as_root=self._execute_as_root)
 
         self._set_rw_permissions_for_all(path_to_new_vol)
