@@ -10,12 +10,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
-import six
-
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import units
+import six
 import taskflow.engines
 from taskflow.patterns import linear_flow
 from taskflow.types import failure as ft
@@ -30,14 +27,13 @@ from cinder import quota
 from cinder import quota_utils
 from cinder import utils
 from cinder.volume.flows import common
-from cinder.volume import utils as vol_utils
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
 ACTION = 'volume:create'
 CONF = cfg.CONF
-GB = units.Gi
 QUOTAS = quota.QUOTAS
 
 # Only in these 'sources' status can we attempt to create a volume from a
@@ -225,28 +221,7 @@ class ExtractVolumeRequestTask(flow_utils.CinderTask):
         # exist, this is expected as it signals that the image_id is missing.
         image_meta = self.image_service.show(context, image_id)
 
-        # check whether image is active
-        if image_meta['status'] != 'active':
-            msg = _('Image %(image_id)s is not active.')\
-                % {'image_id': image_id}
-            raise exception.InvalidInput(reason=msg)
-
-        # Check image size is not larger than volume size.
-        image_size = utils.as_int(image_meta['size'], quiet=False)
-        image_size_in_gb = (image_size + GB - 1) // GB
-        if image_size_in_gb > size:
-            msg = _('Size of specified image %(image_size)sGB'
-                    ' is larger than volume size %(volume_size)sGB.')
-            msg = msg % {'image_size': image_size_in_gb, 'volume_size': size}
-            raise exception.InvalidInput(reason=msg)
-
-        # Check image min_disk requirement is met for the particular volume
-        min_disk = image_meta.get('min_disk', 0)
-        if size < min_disk:
-            msg = _('Volume size %(volume_size)sGB cannot be smaller'
-                    ' than the image minDisk size %(min_disk)sGB.')
-            msg = msg % {'volume_size': size, 'min_disk': min_disk}
-            raise exception.InvalidInput(reason=msg)
+        volume_utils.check_image_metadata(image_meta, size)
 
         return image_meta
 
@@ -260,7 +235,7 @@ class ExtractVolumeRequestTask(flow_utils.CinderTask):
         the validated availability zone.
         """
         refresh_az = False
-        type_azs = vol_utils.extract_availability_zones_from_volume_type(
+        type_azs = volume_utils.extract_availability_zones_from_volume_type(
             volume_type)
         type_az_configured = type_azs is not None
         if type_az_configured:
@@ -362,45 +337,49 @@ class ExtractVolumeRequestTask(flow_utils.CinderTask):
             # Clone the existing key and associate a separate -- but
             # identical -- key with each volume.
             if encryption_key_id is not None:
-                encryption_key_id = vol_utils.clone_encryption_key(
+                encryption_key_id = volume_utils.clone_encryption_key(
                     context,
                     key_manager,
                     encryption_key_id)
             else:
-                encryption_key_id = vol_utils.create_encryption_key(
+                encryption_key_id = volume_utils.create_encryption_key(
                     context,
                     key_manager,
                     volume_type_id)
 
         return encryption_key_id
 
-    def _get_volume_type(self, context, volume_type,
+    @staticmethod
+    def _get_volume_type(context, volume_type,
                          source_volume, snapshot, image_volume_type_id):
+        """Returns a volume_type object or raises.  Never returns None."""
         if volume_type:
             return volume_type
-        identifier = collections.defaultdict(str)
-        try:
-            if source_volume:
-                identifier = {'source': 'volume',
-                              'id': source_volume['volume_type_id']}
-            elif snapshot:
-                identifier = {'source': 'snapshot',
-                              'id': snapshot['volume_type_id']}
-            elif image_volume_type_id:
-                identifier = {'source': 'image',
-                              'id': image_volume_type_id}
-            elif CONF.default_volume_type:
-                identifier = {'source': 'default volume type config',
-                              'id': CONF.default_volume_type}
-            if identifier:
+
+        identifier = None
+        if source_volume:
+            identifier = {'source': 'volume',
+                          'id': source_volume['volume_type_id']}
+        elif snapshot:
+            identifier = {'source': 'snapshot',
+                          'id': snapshot['volume_type_id']}
+        elif image_volume_type_id:
+            identifier = {'source': 'image',
+                          'id': image_volume_type_id}
+        if identifier:
+            try:
                 return objects.VolumeType.get_by_name_or_id(
                     context, identifier['id'])
-        except (exception.VolumeTypeNotFound,
-                exception.VolumeTypeNotFoundByName,
-                exception.InvalidVolumeType):
-            LOG.exception("Failed to find volume type from "
-                          "source %(source)s, identifier %(id)s", identifier)
-        return None
+            except (exception.VolumeTypeNotFound,
+                    exception.VolumeTypeNotFoundByName,
+                    exception.InvalidVolumeType):
+                LOG.exception("Failed to find volume type from "
+                              "source %(source)s, identifier %(id)s",
+                              identifier)
+                raise
+
+        # otherwise, use the default volume type
+        return volume_types.get_default_volume_type(context)
 
     def execute(self, context, size, snapshot, image_id, source_volume,
                 availability_zone, volume_type, metadata, key_manager,
@@ -447,7 +426,7 @@ class ExtractVolumeRequestTask(flow_utils.CinderTask):
             volume_type_id,
             snapshot,
             source_volume,
-            image_meta)
+            image_meta)   # new key id that's been cloned already
 
         if volume_type_id:
             volume_type = objects.VolumeType.get_by_name_or_id(
@@ -479,7 +458,7 @@ class ExtractVolumeRequestTask(flow_utils.CinderTask):
             specs = None
             extra_specs = None
 
-        if vol_utils.is_replicated_spec(extra_specs):
+        if volume_utils.is_replicated_spec(extra_specs):
             replication_status = fields.ReplicationStatus.ENABLED
         else:
             replication_status = fields.ReplicationStatus.DISABLED
@@ -763,14 +742,14 @@ class VolumeCastTask(flow_utils.CinderTask):
             # If cgroup_id existed, we should cast volume to the scheduler
             # to choose a proper pool whose backend is same as CG's backend.
             cgroup = objects.ConsistencyGroup.get_by_id(context, cgroup_id)
-            request_spec['resource_backend'] = vol_utils.extract_host(
-                cgroup.host)
+            request_spec['resource_backend'] = volume_utils.extract_host(
+                cgroup.resource_backend)
         elif group_id:
             # If group_id exists, we should cast volume to the scheduler
             # to choose a proper pool whose backend is same as group's backend.
             group = objects.Group.get_by_id(context, group_id)
-            request_spec['resource_backend'] = vol_utils.extract_host(
-                group.host)
+            request_spec['resource_backend'] = volume_utils.extract_host(
+                group.resource_backend)
         elif snapshot_id and CONF.snapshot_same_host:
             # NOTE(Rongze Zhu): A simple solution for bug 1008866.
             #
@@ -782,10 +761,11 @@ class VolumeCastTask(flow_utils.CinderTask):
             # before creating volume, we schedule this request to scheduler
             # service with the desired backend information.
             snapshot = objects.Snapshot.get_by_id(context, snapshot_id)
-            request_spec['resource_backend'] = snapshot.volume.host
+            request_spec['resource_backend'] = snapshot.volume.resource_backend
         elif source_volid:
             source_volume_ref = objects.Volume.get_by_id(context, source_volid)
-            request_spec['resource_backend'] = source_volume_ref.host
+            request_spec['resource_backend'] = (
+                source_volume_ref.resource_backend)
 
         self.scheduler_rpcapi.create_volume(
             context,
@@ -821,7 +801,7 @@ class VolumeCastTask(flow_utils.CinderTask):
         exc_info = False
         if all(flow_failures[-1].exc_info):
             exc_info = flow_failures[-1].exc_info
-        LOG.error('Unexpected build error:', exc_info=exc_info)
+        LOG.error('Unexpected build error:', exc_info=exc_info)  # noqa
 
 
 def get_flow(db_api, image_service_api, availability_zones, create_what,
