@@ -10,14 +10,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import binascii
 import traceback
 
+from castellan import key_manager
+import os_brick.initiator.connectors
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import timeutils
+import six
 import taskflow.engines
 from taskflow.patterns import linear_flow
 from taskflow.types import failure as ft
@@ -38,7 +42,7 @@ from cinder.objects import consistencygroup
 from cinder.objects import fields
 from cinder import utils
 from cinder.volume.flows import common
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -56,6 +60,11 @@ IMAGE_ATTRIBUTES = (
     'size',
 )
 
+REKEY_SUPPORTED_CONNECTORS = (
+    os_brick.initiator.connectors.iscsi.ISCSIConnector,
+    os_brick.initiator.connectors.fibre_channel.FibreChannelConnector,
+)
+
 
 class OnFailureRescheduleTask(flow_utils.CinderTask):
     """Triggers a rescheduling request to be sent when reverting occurs.
@@ -89,7 +98,6 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
             # they fail, rescheduling will likely attempt to create the volume
             # on another machine when it still exists locally.
             exception.MetadataCopyFailure,
-            exception.MetadataCreateFailure,
             exception.MetadataUpdateFailure,
             # The volume/snapshot has been removed from the database, that
             # can not be fixed by rescheduling.
@@ -400,7 +408,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                                " %(src_id)s metadata")
         src_type = None
         src_id = None
-        self._enable_bootable_flag(context, volume)
+        volume_utils.enable_bootable_flag(volume)
         try:
             if kwargs.get('snapshot_id'):
                 src_type = 'snapshot'
@@ -471,15 +479,145 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                                                      snapshot_id=snapshot_id)
         return model_update
 
-    def _enable_bootable_flag(self, context, volume):
+    @staticmethod
+    def _setup_encryption_keys(context, volume, encryption):
+        """Return encryption keys in passphrase form for a clone operation.
+
+        :param context: context
+        :param volume: volume being cloned
+        :param encryption: encryption info dict
+
+        :returns: tuple (source_pass, new_pass, new_key_id)
+        """
+
+        keymgr = key_manager.API(CONF)
+        key = keymgr.get(context, encryption['encryption_key_id'])
+        source_pass = binascii.hexlify(key.get_encoded()).decode('utf-8')
+
+        new_key_id = volume_utils.create_encryption_key(context,
+                                                        keymgr,
+                                                        volume.volume_type_id)
+        new_key = keymgr.get(context, new_key_id)
+        new_pass = binascii.hexlify(new_key.get_encoded()).decode('utf-8')
+
+        return (source_pass, new_pass, new_key_id)
+
+    def _rekey_volume(self, context, volume):
+        """Change encryption key on volume.
+
+        :returns: model update dict
+        """
+
+        LOG.debug('rekey volume %s', volume.name)
+
+        properties = utils.brick_get_connector_properties(False, False)
+        LOG.debug("properties: %s", properties)
+        attach_info = None
+        model_update = {}
+        new_key_id = None
+        original_key_id = volume.encryption_key_id
+        key_mgr = key_manager.API(CONF)
+
         try:
-            LOG.debug('Marking volume %s as bootable.', volume.id)
-            volume.bootable = True
-            volume.save()
-        except exception.CinderException as ex:
-            LOG.exception("Failed updating volume %(volume_id)s bootable "
-                          "flag to true", {'volume_id': volume.id})
-            raise exception.MetadataUpdateFailure(reason=ex)
+            attach_info, volume = self.driver._attach_volume(context,
+                                                             volume,
+                                                             properties)
+            if not any(c for c in REKEY_SUPPORTED_CONNECTORS
+                       if isinstance(attach_info['connector'], c)):
+                LOG.debug('skipping rekey, connector: %s',
+                          attach_info['connector'])
+                raise exception.RekeyNotSupported()
+
+            LOG.debug("attempting attach for rekey, attach_info: %s",
+                      attach_info)
+
+            if (isinstance(attach_info['device']['path'], six.string_types)):
+                image_info = image_utils.qemu_img_info(
+                    attach_info['device']['path'])
+            else:
+                # Should not happen, just a safety check
+                LOG.error('%s appears to not be encrypted',
+                          attach_info['device']['path'])
+                raise exception.RekeyNotSupported()
+
+            encryption = volume_utils.check_encryption_provider(
+                self.db,
+                volume,
+                context)
+
+            (source_pass, new_pass, new_key_id) = self._setup_encryption_keys(
+                context,
+                volume,
+                encryption)
+
+            if image_info.encrypted == 'yes':
+                key_str = source_pass + "\n" + new_pass + "\n"
+                del source_pass
+
+                (out, err) = utils.execute(
+                    'cryptsetup',
+                    'luksChangeKey',
+                    attach_info['device']['path'],
+                    '--force-password',
+                    run_as_root=True,
+                    process_input=key_str,
+                    log_errors=processutils.LOG_ALL_ERRORS)
+
+                del key_str
+                model_update = {'encryption_key_id': new_key_id}
+            else:
+                # volume has not been written to yet, format with luks
+                del source_pass
+
+                if image_info.file_format != 'raw':
+                    # Something has gone wrong if the image is not encrypted
+                    # and is detected as another format.
+                    raise exception.Invalid()
+
+                if encryption['provider'] == 'luks':
+                    # Force ambiguous "luks" provider to luks1 for
+                    # compatibility with new versions of cryptsetup.
+                    encryption['provider'] = 'luks1'
+
+                (out, err) = utils.execute(
+                    'cryptsetup',
+                    '--batch-mode',
+                    'luksFormat',
+                    '--force-password',
+                    '--type', encryption['provider'],
+                    '--cipher', encryption['cipher'],
+                    '--key-size', str(encryption['key_size']),
+                    '--key-file=-',
+                    attach_info['device']['path'],
+                    run_as_root=True,
+                    process_input=new_pass)
+                del new_pass
+                model_update = {'encryption_key_id': new_key_id}
+
+            # delete the original key that was cloned for this volume
+            # earlier
+            volume_utils.delete_encryption_key(context,
+                                               key_mgr,
+                                               original_key_id)
+        except exception.RekeyNotSupported:
+            pass
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                if new_key_id is not None:
+                    # Remove newly cloned key since it will not be used.
+                    volume_utils.delete_encryption_key(
+                        context,
+                        key_mgr,
+                        new_key_id)
+        finally:
+            if attach_info:
+                self.driver._detach_volume(context,
+                                           attach_info,
+                                           volume,
+                                           properties,
+                                           force=True)
+
+        return model_update
 
     def _create_from_source_volume(self, context, volume, source_volid,
                                    **kwargs):
@@ -492,6 +630,11 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         srcvol_ref = objects.Volume.get_by_id(context, source_volid)
         try:
             model_update = self.driver.create_cloned_volume(volume, srcvol_ref)
+            if model_update is None:
+                model_update = {}
+            if volume.encryption_key_id is not None:
+                rekey_model_update = self._rekey_volume(context, volume)
+                model_update.update(rekey_model_update)
         finally:
             self._cleanup_cg_in_volume(volume)
         # NOTE(harlowja): Subtasks would be useful here since after this
@@ -502,89 +645,10 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                 context, volume, source_volid=srcvol_ref.id)
         return model_update
 
-    def _copy_image_to_volume(self, context, volume,
-                              image_meta, image_location, image_service):
-
-        image_id = image_meta['id']
-        """Downloads Glance image to the specified volume."""
-        LOG.debug("Attempting download of %(image_id)s (%(image_location)s)"
-                  " to volume %(volume_id)s.",
-                  {'image_id': image_id, 'volume_id': volume.id,
-                   'image_location': image_location})
-        try:
-            image_encryption_key = image_meta.get('cinder_encryption_key_id')
-
-            if volume.encryption_key_id and image_encryption_key:
-                # If the image provided an encryption key, we have
-                # already cloned it to the volume's key in
-                # _get_encryption_key_id, so we can do a direct copy.
-                self.driver.copy_image_to_volume(
-                    context, volume, image_service, image_id)
-            elif volume.encryption_key_id:
-                # Creating an encrypted volume from a normal, unencrypted,
-                # image.
-                self.driver.copy_image_to_encrypted_volume(
-                    context, volume, image_service, image_id)
-            else:
-                self.driver.copy_image_to_volume(
-                    context, volume, image_service, image_id)
-        except processutils.ProcessExecutionError as ex:
-            LOG.exception("Failed to copy image %(image_id)s to volume: "
-                          "%(volume_id)s",
-                          {'volume_id': volume.id, 'image_id': image_id})
-            raise exception.ImageCopyFailure(reason=ex.stderr)
-        except exception.ImageUnacceptable as ex:
-            LOG.exception("Failed to copy image to volume: %(volume_id)s",
-                          {'volume_id': volume.id})
-            raise exception.ImageUnacceptable(ex)
-        except exception.ImageTooBig as ex:
-            with excutils.save_and_reraise_exception():
-                LOG.exception("Failed to copy image %(image_id)s to volume: "
-                              "%(volume_id)s",
-                              {'volume_id': volume.id, 'image_id': image_id})
-        except Exception as ex:
-            LOG.exception("Failed to copy image %(image_id)s to "
-                          "volume: %(volume_id)s",
-                          {'volume_id': volume.id, 'image_id': image_id})
-            if not isinstance(ex, exception.ImageCopyFailure):
-                raise exception.ImageCopyFailure(reason=ex)
-            else:
-                raise
-
-        LOG.debug("Downloaded image %(image_id)s (%(image_location)s)"
-                  " to volume %(volume_id)s successfully.",
-                  {'image_id': image_id, 'volume_id': volume.id,
-                   'image_location': image_location})
-
     def _capture_volume_image_metadata(self, context, volume_id,
                                        image_id, image_meta):
-
-        # Save some base attributes into the volume metadata
-        base_metadata = {
-            'image_id': image_id,
-        }
-        name = image_meta.get('name', None)
-        if name:
-            base_metadata['image_name'] = name
-
-        # Save some more attributes into the volume metadata from the image
-        # metadata
-        for key in IMAGE_ATTRIBUTES:
-            if key not in image_meta:
-                continue
-            value = image_meta.get(key, None)
-            if value is not None:
-                base_metadata[key] = value
-
-        # Save all the image metadata properties into the volume metadata
-        property_metadata = {}
-        image_properties = image_meta.get('properties', {})
-        for (key, value) in image_properties.items():
-            if value is not None:
-                property_metadata[key] = value
-
-        volume_metadata = dict(property_metadata)
-        volume_metadata.update(base_metadata)
+        volume_metadata = volume_utils.get_volume_image_metadata(
+            image_id, image_meta)
         LOG.debug("Creating volume glance metadata for volume %(volume_id)s"
                   " backed by image %(image_id)s with: %(vol_metadata)s.",
                   {'volume_id': volume_id, 'image_id': image_id,
@@ -669,8 +733,9 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                           {'volume_id': volume.id,
                            'updates': model_update})
         try:
-            self._copy_image_to_volume(context, volume, image_meta,
-                                       image_location, image_service)
+            volume_utils.copy_image_to_volume(self.driver, context, volume,
+                                              image_meta, image_location,
+                                              image_service)
         except exception.ImageTooBig:
             with excutils.save_and_reraise_exception():
                 LOG.exception("Failed to copy image to volume "
@@ -708,6 +773,14 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                     cache_entry['volume_id']
                 )
                 return model_update, True
+        except exception.SnapshotLimitReached:
+            # If this exception occurred when cloning the image-volume,
+            # it is because the image-volume reached its snapshot limit.
+            # Delete current cache entry and create a "fresh" entry
+            # NOTE: This will not delete the existing image-volume and
+            # only delete the cache entry
+            with excutils.save_and_reraise_exception():
+                self.image_volume_cache.evict(context, cache_entry)
         except NotImplementedError:
             LOG.warning('Backend does not support creating image-volume '
                         'clone. Image will be downloaded from Glance.')
@@ -791,17 +864,18 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                         image_id,
                         image_meta
                     )
+                except exception.SnapshotLimitReached:
+                    # This exception will be handled by the caller's
+                    # (_create_from_image) retry decorator
+                    with excutils.save_and_reraise_exception():
+                        LOG.debug("Snapshot limit reached. Creating new "
+                                  "image-volume.")
                 except exception.CinderException as e:
                     LOG.warning('Failed to create volume from image-volume '
                                 'cache, image will be downloaded from Glance. '
                                 'Error: %(exception)s',
                                 {'exception': e})
 
-                    # If an exception occurred when cloning the image-volume,
-                    # it may be the image-volume reached its snapshot limit.
-                    # Create another "fresh" cache entry.
-                    update_cache = True
-
                 # Don't cache unless directed.
                 if not cloned and update_cache:
                     should_create_cache_entry = True
@@ -841,8 +915,8 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
                         if should_create_cache_entry:
                             if virtual_size and virtual_size != original_size:
-                                    volume.size = virtual_size
-                                    volume.save()
+                                volume.size = virtual_size
+                                volume.save()
                         model_update = self._create_from_image_download(
                             context,
                             volume,
@@ -893,6 +967,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
         return model_update
 
+    @utils.retry(exception.SnapshotLimitReached, retries=1)
     def _create_from_image(self, context, volume,
                            image_location, image_id, image_meta,
                            image_service, **kwargs):
@@ -979,7 +1054,8 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                      "at the backend and then schedule the request to the "
                      "backup service to restore the volume with backup.",
                      {'id': backup_id})
-            model_update = self._create_raw_volume(volume, **kwargs) or {}
+            model_update = self._create_raw_volume(
+                context, volume, **kwargs) or {}
             volume.update(model_update)
             volume.save()
 
@@ -1001,9 +1077,17 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                   'backup_id': backup_id})
         return ret, need_update_volume
 
-    def _create_raw_volume(self, volume, **kwargs):
+    def _create_raw_volume(self, context, volume, **kwargs):
         try:
             ret = self.driver.create_volume(volume)
+        except Exception as ex:
+            with excutils.save_and_reraise_exception():
+                self.message.create(
+                    context,
+                    message_field.Action.CREATE_VOLUME_FROM_BACKEND,
+                    resource_uuid=volume.id,
+                    detail=message_field.Detail.DRIVER_FAILED_CREATE,
+                    exception=ex)
         finally:
             self._cleanup_cg_in_volume(volume)
         return ret
@@ -1036,7 +1120,8 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                  {'volume_spec': volume_spec, 'volume_id': volume_id,
                   'create_type': create_type})
         if create_type == 'raw':
-            model_update = self._create_raw_volume(volume, **volume_spec)
+            model_update = self._create_raw_volume(
+                context, volume, **volume_spec)
         elif create_type == 'snap':
             model_update = self._create_from_snapshot(context, volume,
                                                       **volume_spec)
